-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

AbilityEventConfig = {}
AbilityEventConfig.name = "AbilityEventConfig"
AbilityEventConfig.version = "$Id$"
AbilityEventConfig.AbilityBegin = "Ability cooldown has started"
AbilityEventConfig.AbilityEnd = "Ability cooldown has ended"
AbilityEventConfig.AbilityAvailable = "Ability is available"
AbilityEventConfig.AbilityNotAvailable = "Ability is not available"

--
-- Main functions
--

--
-- UI functions
--

function AbilityEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtAbilityConfig = {}
	evtAbilityConfig.wmeConfig = wmeConfig
	evtAbilityConfig.parentFrame = parentFrame
	evtAbilityConfig.callback = callback
	evtAbilityConfig.event = AbilityEventConfig.GetDefaultEvent()
	evtAbilityConfig.txtPicker = wmeConfig.txtPicker
	
	evtAbilityConfig.windowFrame = UI.CreateFrame("RiftWindow", "AbilityEventSelector", parentFrame)
	evtAbilityConfig.windowFrame:SetTitle("Ability Event Editor")
	evtAbilityConfig.windowFrame:SetWidth(700)
	evtAbilityConfig.windowFrame:SetHeight(770)
	evtAbilityConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtAbilityConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtAbilityConfig.windowFrame)
	evtAbilityConfig.okButton:SetPoint("BOTTOMLEFT", evtAbilityConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtAbilityConfig.okButton:SetText("OK")
	evtAbilityConfig.okButton.Event.LeftPress = function() AbilityEventConfig.WindowClose(evtAbilityConfig, true) end	

	evtAbilityConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtAbilityConfig.windowFrame)
	evtAbilityConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtAbilityConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtAbilityConfig.cancelButton:SetText("Cancel")
	evtAbilityConfig.cancelButton.Event.LeftPress = function() AbilityEventConfig.WindowClose(evtAbilityConfig, false) end	

	local margin = 4
	local height = 40
	evtAbilityConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtAbilityConfig.windowFrame)
	evtAbilityConfig.gridFrame:SetPoint("TOPLEFT", evtAbilityConfig.windowFrame, "TOPLEFT", 40, 80)
	evtAbilityConfig.rowLayout = RowLayout.Create(evtAbilityConfig.gridFrame, 4, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtAbilityConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtAbilityConfig.warningFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 2, 2, "Text")
	evtAbilityConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Ability Name")
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the ability to track")
	local txtFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Event Type")
	evtAbilityConfig.operatorFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 4, 2, "Text")
	evtAbilityConfig.operatorFrame:SetText("")

	evtAbilityConfig.nameFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 1, 2, "RiftTextfield")
	evtAbilityConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtAbilityConfig.abilityNameFrame = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 3, 2, "RiftTextfield")
	evtAbilityConfig.abilityNameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	evtAbilityConfig.warningButton = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtAbilityConfig.warningButton:SetText("Choose")
	evtAbilityConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtAbilityConfig, WarnMe.warningTypeAll, function(a, warningName) AbilityEventConfig.ChooseWarningCallback(evtAbilityConfig, warningName) end) end
	evtAbilityConfig.operatorButton = RowLayout.CreateFrame(evtAbilityConfig.rowLayout, 4, 3, "RiftButton", RowLayout.centerFrame)
	evtAbilityConfig.operatorButton:SetText("Choose")
	evtAbilityConfig.operatorButton.Event.LeftPress = function() AbilityEventConfig.ChooseEventType(evtAbilityConfig, function(a, eventType) AbilityEventConfig.ChooseEventTypeCallback(evtAbilityConfig, eventType) end) end
	RowLayout.SetColWidth(evtAbilityConfig.rowLayout, 3, evtAbilityConfig.warningButton:GetWidth())

	RowLayout.Redraw(evtAbilityConfig.rowLayout)

	evtAbilityConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtAbilityConfig.windowFrame)
	evtAbilityConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtAbilityConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtAbilityConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtAbilityConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtAbilityConfig.scopeWindow = ScopeSelector.Create(evtAbilityConfig.scopeFrame, xMargin, yMargin, evtAbilityConfig.event.scope)

	evtAbilityConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtAbilityConfig.parentFrame, evtAbilityConfig.windowFrame)
	
	AbilityEventConfig.Show(evtAbilityConfig)
	evtAbilityConfig.windowFrame:SetVisible(false)
	
	return evtAbilityConfig
end

function AbilityEventConfig.WindowClose(evtAbilityConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtAbilityConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtAbilityConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local ability = evtAbilityConfig.abilityNameFrame:GetText()
		if not ability or string.len(ability) < 1 then
			message = message .. "You must specify an ability for this event\n"
		end
				
		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtAbilityConfig.wrnWindow, message, function() end)
			return
		end
		
		AbilityEventConfig.FillEvent(evtAbilityConfig)
	end
	
	evtAbilityConfig.nameFrame:SetKeyFocus(false)
	evtAbilityConfig.abilityNameFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtAbilityConfig.scopeWindow)
	
	evtAbilityConfig.windowFrame:SetVisible(false)
	if evtAbilityConfig.callback then
		if okPressed == true then
			evtAbilityConfig.callback(evtAbilityConfig, evtAbilityConfig.event)
		else
			evtAbilityConfig.callback(evtAbilityConfig)
		end
	end
end

function AbilityEventConfig.Show(evtAbilityConfig, inEvent)
	if inEvent then
		evtAbilityConfig.event = AbilityEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtAbilityConfig.event
	evtAbilityConfig.nameFrame:SetText(event.name)
	evtAbilityConfig.warningFrame:SetText(event.warningName)
	evtAbilityConfig.abilityNameFrame:SetText(event.ability.name)
	if event.ability.abilityType == WarnMe.abilityTypeBegin then
		evtAbilityConfig.operatorFrame:SetText(AbilityEventConfig.AbilityBegin)
	elseif event.ability.abilityType == WarnMe.abilityTypeEnd then
		evtAbilityConfig.operatorFrame:SetText(AbilityEventConfig.AbilityEnd)
	elseif event.ability.abilityType == WarnMe.abilityTypeAvailable then
		evtAbilityConfig.operatorFrame:SetText(AbilityEventConfig.AbilityAvailable)
	elseif event.ability.abilityType == WarnMe.abilityTypeNotAvailable then
		evtAbilityConfig.operatorFrame:SetText(AbilityEventConfig.AbilityNotAvailable)
	end
	ScopeSelector.SetScope(evtAbilityConfig.scopeWindow, event.scope, event.scope.player)
	evtAbilityConfig.windowFrame:SetVisible(true)
end

function AbilityEventConfig.FillEvent(evtAbilityConfig)
	evtAbilityConfig.event.name = evtAbilityConfig.nameFrame:GetText()
	evtAbilityConfig.event.warningName = evtAbilityConfig.warningFrame:GetText()
	evtAbilityConfig.event.ability.name = evtAbilityConfig.abilityNameFrame:GetText()
	evtAbilityConfig.event.scope = WarnMe.CopyScope(evtAbilityConfig.scopeWindow.scope)
	local operatorText = evtAbilityConfig.operatorFrame:GetText()
	if operatorText == AbilityEventConfig.AbilityBegin then
		evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeBegin
	elseif operatorText == AbilityEventConfig.AbilityEnd then
		evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeEnd
	elseif operatorText == AbilityEventConfig.AbilityAvailable then
		evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeAvailable
	elseif operatorText == AbilityEventConfig.AbilityNotAvailable then
		evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeNotAvailable
	end
end

function AbilityEventConfig.ChooseWarningCallback(evtAbilityConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		AbilityEventConfig.FillEvent(evtAbilityConfig)
		evtAbilityConfig.event.warningName = warningName
		AbilityEventConfig.Show(evtAbilityConfig)
	end
	
	evtAbilityConfig.windowFrame:SetVisible(true)
end

function AbilityEventConfig.ChooseEventTypeCallback(evtAbilityConfig, eventType)
	if eventType and string.len(eventType) > 0 then
		AbilityEventConfig.FillEvent(evtAbilityConfig)
		if eventType == AbilityEventConfig.AbilityBegin then
			evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeBegin
		elseif eventType == AbilityEventConfig.AbilityEnd then
			evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeEnd
		elseif eventType == AbilityEventConfig.AbilityAvailable then
			evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeAvailable
		elseif eventType == AbilityEventConfig.AbilityNotAvailable then
			evtAbilityConfig.event.ability.abilityType = WarnMe.abilityTypeNotAvailable
		end
		AbilityEventConfig.Show(evtAbilityConfig)
	end
	
	evtAbilityConfig.windowFrame:SetVisible(true)
end

function AbilityEventConfig.ChooseEventType(wmeConfig, callback)
	local textList = {}
	table.insert(textList, AbilityEventConfig.AbilityBegin)
	table.insert(textList, AbilityEventConfig.AbilityEnd)
	table.insert(textList, AbilityEventConfig.AbilityAvailable)
	table.insert(textList, AbilityEventConfig.AbilityNotAvailable)
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Event Type", textList, callback)
end

function AbilityEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = event2.eventType
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.ability = {}
	event.ability.name = event2.ability.name
	event.ability.abilityType = event2.ability.abilityType
	
	return event
end

function AbilityEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeAbility
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.ability = {}
	event.ability.name = ""
	event.ability.abilityType = WarnMe.abilityTypeBegin
	
	return event
end

