-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

BuffEventConfig = {}
BuffEventConfig.name = "BuffEventConfig"
BuffEventConfig.version = "$Id$"
BuffEventConfig.BuffExistsStr = "Buff/Debuff exists"
BuffEventConfig.BuffNotExistsStr = "Buff/Debuff is missing"
BuffEventConfig.BuffLEStr = "Stacking Buff/Debuff is less than or equal to stacks"
BuffEventConfig.BuffGEStr = "Stacking Buff/Debuff is greater than or equal to stacks"

--
-- Main functions
--

--
-- UI functions
--

function BuffEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtBuffConfig = {}
	evtBuffConfig.wmeConfig = wmeConfig
	evtBuffConfig.parentFrame = parentFrame
	evtBuffConfig.callback = callback
	evtBuffConfig.event = BuffEventConfig.GetDefaultEvent()
	evtBuffConfig.txtPicker = wmeConfig.txtPicker
	
	evtBuffConfig.windowFrame = UI.CreateFrame("RiftWindow", "BuffEventSelector", parentFrame)
	evtBuffConfig.windowFrame:SetTitle("Buff/Debuff Event Editor")
	evtBuffConfig.windowFrame:SetWidth(700)
	evtBuffConfig.windowFrame:SetHeight(820)
	evtBuffConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtBuffConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtBuffConfig.windowFrame)
	evtBuffConfig.okButton:SetPoint("BOTTOMLEFT", evtBuffConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtBuffConfig.okButton:SetText("OK")
	evtBuffConfig.okButton.Event.LeftPress = function() BuffEventConfig.WindowClose(evtBuffConfig, true) end	

	evtBuffConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtBuffConfig.windowFrame)
	evtBuffConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtBuffConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtBuffConfig.cancelButton:SetText("Cancel")
	evtBuffConfig.cancelButton.Event.LeftPress = function() BuffEventConfig.WindowClose(evtBuffConfig, false) end	

	local margin = 4
	local height = 40
	evtBuffConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtBuffConfig.windowFrame)
	evtBuffConfig.gridFrame:SetPoint("TOPLEFT", evtBuffConfig.windowFrame, "TOPLEFT", 40, 80)
	evtBuffConfig.rowLayout = RowLayout.Create(evtBuffConfig.gridFrame, 7, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtBuffConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtBuffConfig.warningFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 2, 2, "Text")
	evtBuffConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Buff Name")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the buff/debuff to monitor")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Buff target")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Buff type")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Stack count")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 6, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The number of buff/debuff stacks required to fire the event")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("Player's Buff")
	local txtFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 7, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Only monitor buff cast by the player")

	evtBuffConfig.nameFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 1, 2, "RiftTextfield")
	evtBuffConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtBuffConfig.buffNameFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 3, 2, "RiftTextfield")
	evtBuffConfig.buffNameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	evtBuffConfig.buffUnitSpecFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 4, 2, "Text")
	evtBuffConfig.buffTypeFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 5, 2, "Text")
	
	evtBuffConfig.warningButton = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtBuffConfig.warningButton:SetText("Choose")
	evtBuffConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtBuffConfig, WarnMe.warningTypeAll, function(a, warningName) BuffEventConfig.ChooseWarningCallback(evtBuffConfig, warningName) end) end
	evtBuffConfig.targetButton = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 4, 3, "RiftButton", RowLayout.centerFrame)
	evtBuffConfig.targetButton:SetText("Choose")
	evtBuffConfig.targetButton.Event.LeftPress = function() BuffEventConfig.ChooseUnitSpec(evtBuffConfig, function(a, unitSpecName) BuffEventConfig.ChooseUnitSpecCallback(evtBuffConfig, unitSpecName) end) end
	evtBuffConfig.typeButton = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 5, 3, "RiftButton", RowLayout.centerFrame)
	evtBuffConfig.typeButton:SetText("Choose")
	evtBuffConfig.typeButton.Event.LeftPress = function() BuffEventConfig.ChooseBuffType(evtBuffConfig, function(a, buffType) BuffEventConfig.ChooseBuffTypeCallback(evtBuffConfig, buffType) end) end
	RowLayout.SetColWidth(evtBuffConfig.rowLayout, 3, evtBuffConfig.warningButton:GetWidth())

	local sliderFrame = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 6, 2, "Frame")
	evtBuffConfig.stackSlider = NumberSlider.Create(sliderFrame, 0, 50, 1, 0)

	evtBuffConfig.playerCheck = RowLayout.CreateFrame(evtBuffConfig.rowLayout, 7, 2, "RiftCheckbox", RowLayout.centerFrame)

	RowLayout.Redraw(evtBuffConfig.rowLayout)

	evtBuffConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtBuffConfig.windowFrame)
	evtBuffConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtBuffConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtBuffConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtBuffConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtBuffConfig.scopeWindow = ScopeSelector.Create(evtBuffConfig.scopeFrame, xMargin, yMargin, evtBuffConfig.event.scope)

	evtBuffConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtBuffConfig.parentFrame, evtBuffConfig.windowFrame)
	
	BuffEventConfig.Show(evtBuffConfig)
	evtBuffConfig.windowFrame:SetVisible(false)
	
	return evtBuffConfig
end

function BuffEventConfig.WindowClose(evtBuffConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtBuffConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtBuffConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local buffName = evtBuffConfig.buffNameFrame:GetText()
		if not buffName or string.len(buffName) < 1 then
			message = message .. "You must specify a buff name for this event\n"
		end
		local str = evtBuffConfig.buffUnitSpecFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify a target for this event\n"
		end
		local typeStr = evtBuffConfig.buffTypeFrame:GetText()
		if not typeStr or string.len(typeStr) < 1 then
			message = message .. "You must specify a buff type for this event\n"
		end
		if typeStr and typeStr ~= BuffEventConfig.BuffNotExistsStr and buffName and string.find(buffName, ",") then
			message = message .. "You can only enter many buff names when buff type is set to Missing\n"
		end

		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtBuffConfig.wrnWindow, message, function() end)
			return
		end
		
		BuffEventConfig.FillEvent(evtBuffConfig)
	end
	
	evtBuffConfig.nameFrame:SetKeyFocus(false)
	evtBuffConfig.buffNameFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtBuffConfig.scopeWindow)
	
	evtBuffConfig.windowFrame:SetVisible(false)
	if evtBuffConfig.callback then
		if okPressed == true then
			evtBuffConfig.callback(evtBuffConfig, evtBuffConfig.event)
		else
			evtBuffConfig.callback(evtBuffConfig)
		end
	end
end

function BuffEventConfig.Show(evtBuffConfig, inEvent)
	if inEvent then
		evtBuffConfig.event = BuffEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtBuffConfig.event
	evtBuffConfig.nameFrame:SetText(event.name)
	evtBuffConfig.warningFrame:SetText(event.warningName)
	evtBuffConfig.buffNameFrame:SetText(event.buff.name)
	evtBuffConfig.buffUnitSpecFrame:SetText(event.buff.unitSpec)
	evtBuffConfig.buffTypeFrame:SetText(BuffEventConfig.GetBuffTypeString(event.eventType))
	local stacks = 0
	if event.buff.stacks then
		stacks = event.buff.stacks
	end
	NumberSlider.SetValue(evtBuffConfig.stackSlider, stacks)
	if evtBuffConfig.event.eventType == WarnMe.eventTypeBuffStackGreater or
	   evtBuffConfig.event.eventType == WarnMe.eventTypeBuffStackLess then
		RowLayout.SetRowVisible(evtBuffConfig.rowLayout, 6, true)
	else
		RowLayout.SetRowVisible(evtBuffConfig.rowLayout, 6, false)
	end
	RowLayout.Redraw(evtBuffConfig.rowLayout)
	
	local playersOwnChecked = false
	if event.buff.playersOwn then
		playersOwnChecked = event.buff.playersOwn
	end
	evtBuffConfig.playerCheck:SetChecked(playersOwnChecked)

	ScopeSelector.SetScope(evtBuffConfig.scopeWindow, event.scope, event.scope.player)
	evtBuffConfig.windowFrame:SetVisible(true)
end

function BuffEventConfig.FillEvent(evtBuffConfig)
	evtBuffConfig.event.name = evtBuffConfig.nameFrame:GetText()
	evtBuffConfig.event.warningName = evtBuffConfig.warningFrame:GetText()
	evtBuffConfig.event.buff.name = evtBuffConfig.buffNameFrame:GetText()
	evtBuffConfig.event.buff.unitSpec = evtBuffConfig.buffUnitSpecFrame:GetText()
	evtBuffConfig.event.buff.playersOwn = evtBuffConfig.playerCheck:GetChecked()
	evtBuffConfig.event.eventType = BuffEventConfig.GetBuffType(evtBuffConfig.buffTypeFrame:GetText())
	if evtBuffConfig.event.eventType == WarnMe.eventTypeBuffStackGreater or
	   evtBuffConfig.event.eventType == WarnMe.eventTypeBuffStackLess then
		evtBuffConfig.event.buff.stacks = NumberSlider.GetValue(evtBuffConfig.stackSlider)
	else
		evtBuffConfig.event.buff.stacks = nil
	end
	evtBuffConfig.event.scope = WarnMe.CopyScope(evtBuffConfig.scopeWindow.scope)
end

function BuffEventConfig.ChooseUnitSpecCallback(evtBuffConfig, unitSpec)
	if unitSpec and string.len(unitSpec) > 0 then
		BuffEventConfig.FillEvent(evtBuffConfig)
		evtBuffConfig.event.buff.unitSpec = unitSpec
		BuffEventConfig.Show(evtBuffConfig)
	end
	
	evtBuffConfig.windowFrame:SetVisible(true)
end

function BuffEventConfig.ChooseUnitSpec(wmeConfig, callback)
	local textList = {}
	table.insert(textList, "player")
	table.insert(textList, "player.pet")
	table.insert(textList, "player.target")
	table.insert(textList, "focus")
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Unit", textList, callback)
end

function BuffEventConfig.ChooseBuffTypeCallback(evtBuffConfig, buffType)
	if buffType and string.len(buffType) > 0 then
		BuffEventConfig.FillEvent(evtBuffConfig)
		evtBuffConfig.event.eventType = BuffEventConfig.GetBuffType(buffType)
		BuffEventConfig.Show(evtBuffConfig)
	end
	
	evtBuffConfig.windowFrame:SetVisible(true)
end

function BuffEventConfig.GetBuffType(buffType)
	if buffType == BuffEventConfig.BuffExistsStr then
		return WarnMe.eventTypeBuffExists
	elseif buffType == BuffEventConfig.BuffNotExistsStr then
		return WarnMe.eventTypeBuffNotExists
	elseif buffType == BuffEventConfig.BuffLEStr then
		return WarnMe.eventTypeBuffStackLess
	elseif buffType == BuffEventConfig.BuffGEStr then
		return WarnMe.eventTypeBuffStackGreater
	end
	
	return WarnMe.eventTypeBuffExists
end

function BuffEventConfig.GetBuffTypeString(buffType)
	if buffType == WarnMe.eventTypeBuffExists then
		return BuffEventConfig.BuffExistsStr
	elseif buffType == WarnMe.eventTypeBuffNotExists then
		return BuffEventConfig.BuffNotExistsStr
	elseif buffType == WarnMe.eventTypeBuffStackGreater then
		return BuffEventConfig.BuffGEStr
	elseif buffType == WarnMe.eventTypeBuffStackLess then
		return BuffEventConfig.BuffLEStr
	end
		
	return ""
end

function BuffEventConfig.ChooseBuffType(wmeConfig, callback)
	local textList = {}
	table.insert(textList, BuffEventConfig.BuffExistsStr)
	table.insert(textList, BuffEventConfig.BuffNotExistsStr)
	table.insert(textList, BuffEventConfig.BuffLEStr)
	table.insert(textList, BuffEventConfig.BuffGEStr)
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Buff Type", textList, callback)
end

function BuffEventConfig.ChooseWarningCallback(evtBuffConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		BuffEventConfig.FillEvent(evtBuffConfig)
		evtBuffConfig.event.warningName = warningName
		BuffEventConfig.Show(evtBuffConfig)
	end
	
	evtBuffConfig.windowFrame:SetVisible(true)
end

function BuffEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = event2.eventType
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.buff = {}
	event.buff.name = event2.buff.name
	event.buff.unitSpec = event2.buff.unitSpec
	event.buff.playersOwn = event2.buff.playersOwn
	
	if event2.eventType == WarnMe.eventTypeBuffStackGreater or event2.eventType == WarnMe.eventTypeBuffStackLess then
		event.buff.stacks = event2.buff.stacks
	end
	
	return event
end

function BuffEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeBuffExists
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.buff = {}
	event.buff.name = ""
	event.buff.unitSpec = "player"
	event.buff.playersOwn = false
	
	return event
end

