-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

BuffManager = {}
BuffManager.name = "BuffManager"
BuffManager.version = "v0.1"
BuffManager.callbackID = 0
BuffManager.existsType = 1
BuffManager.notExistsType = 2
BuffManager.playerID = nil
BuffManager.data = {}
BuffManager.data.player = {}
BuffManager.data.player.isEmpty = true
BuffManager.data.player.isPlayer = true
BuffManager.data.player.buffByID = {}
BuffManager.data.player.buffCallbacks = {}
BuffManager.data.player.buffCallbackMap = {}
BuffManager.data.all = {}
BuffManager.data.all.isEmpty = true
BuffManager.data.all.buffByID = {}
BuffManager.data.all.buffCallbacks = {}
BuffManager.data.all.buffCallbackMap = {}

--
-- Main functions
--
local function GetBuffName(buffDetail)
	local buffName = ""
	if buffDetail then
		if buffDetail.name == "" then
			buffName = buffDetail.description
		else
			buffName = buffDetail.name
		end
	end
	
	if not buffName then
		buffName = ""
	end
	
	return buffName
end

function BuffManager.Initialise(addonName, playerID)
	BuffManager.playerID = playerID
	table.insert(Event.Buff.Add, { BuffManager.BuffAdd, addonName, "BuffAdd" })
	table.insert(Event.Buff.Change, { BuffManager.BuffChange, addonName, "BuffChange" })
	table.insert(Event.Buff.Remove, { BuffManager.BuffRemove, addonName, "BuffRemove" })
end

function BuffManager.Clear()
	BuffManager.data = {}
	BuffManager.data.player = {}
	BuffManager.data.player.isEmpty = true
	BuffManager.data.player.isPlayer = true
	BuffManager.data.player.buffByID = {}
	BuffManager.data.player.buffCallbacks = {}
	BuffManager.data.player.buffCallbackMap = {}
	BuffManager.data.all = {}
	BuffManager.data.all.isEmpty = true
	BuffManager.data.all.buffByID = {}
	BuffManager.data.all.buffCallbacks = {}
	BuffManager.data.all.buffCallbackMap = {}
end

function BuffManager.AddBuffExistsCallback(unitSpec, buffName, isPlayer, callback)
	local localData = BuffManager.data.all
	if isPlayer == true then
		localData = BuffManager.data.player
	end
	localData.isEmpty = false
	return BuffManager.InternalAddBuffCallback(localData, unitSpec, buffName, callback, BuffManager.existsType)
end

function BuffManager.AddBuffNotExistsCallback(unitSpec, buffName, isPlayer, callback)
	local localData = BuffManager.data.all
	if isPlayer == true then
		localData = BuffManager.data.player
	end
	localData.isEmpty = false
	return BuffManager.InternalAddBuffCallback(localData, unitSpec, buffName, callback, BuffManager.notExistsType)
end

function BuffManager.AddStacksLessEqualToCallback(unitSpec, buffName, isPlayer, stacks, callback)
	local localData = BuffManager.data.all
	if isPlayer == true then
		localData = BuffManager.data.player
	end
	localData.isEmpty = false
	return BuffManager.InternalAddBuffCallback(localData, unitSpec, buffName, callback, BuffManager.existsType, function(stack) return stack <= stacks end)
end

function BuffManager.AddStacksGreaterEqualToCallback(unitSpec, buffName, isPlayer, stacks, callback)
	local localData = BuffManager.data.all
	if isPlayer == true then
		localData = BuffManager.data.player
	end
	localData.isEmpty = false
	return BuffManager.InternalAddBuffCallback(localData, unitSpec, buffName, callback, BuffManager.existsType, function(stack) return stack >= stacks end)
end

function BuffManager.InternalAddBuffCallback(localData, unitSpec, buffName, callback, eventType, stackCheckFunction)
	if not unitSpec and not buffName then
		print("Ignoring buff with no unit speicifer and name")
		return -1
	end
	if not buffName then
		print("Ignoring " .. unitSpec .. " buff with no name")
		return -1
	end
	if not unitSpec then
		print("Ignoring buff " .. buffName .. " with no unit speicifer")
		return -1
	end
	
	local id = BuffManager.GetCallbackID()
	localData.buffCallbackMap[id] = {}
	localData.buffCallbackMap[id].unitSpec = unitSpec
	localData.buffCallbackMap[id].buffName = buffName
	
	if not localData.buffCallbacks[unitSpec] then
		localData.buffCallbacks[unitSpec] = {}
	end
	
	if not localData.buffCallbacks[unitSpec][buffName] then
		localData.buffCallbacks[unitSpec][buffName] = {}
		localData.buffCallbacks[unitSpec][buffName].duration = 0
		localData.buffCallbacks[unitSpec][buffName].stack = 0
	end

	if not localData.buffCallbacks[unitSpec][buffName].id then
		localData.buffCallbacks[unitSpec][buffName].id = {}
	end
	
	localData.buffCallbacks[unitSpec][buffName].id[id] = {}
	localData.buffCallbacks[unitSpec][buffName].id[id].callback = callback
	localData.buffCallbacks[unitSpec][buffName].id[id].eventType = eventType
	localData.buffCallbacks[unitSpec][buffName].id[id].stackCheckFunction = stackCheckFunction
	
	BuffManager.Refresh(unitSpec)
	return id
end

function BuffManager.GetCallbackID()
	BuffManager.callbackID = BuffManager.callbackID + 1
	return "#" .. BuffManager.callbackID
end

function BuffManager.Refresh(unitSpec)
	BuffManager.LocalDataRefresh(BuffManager.data.all, unitSpec)
	BuffManager.LocalDataRefresh(BuffManager.data.player, unitSpec)
end

function BuffManager.LocalDataRefresh(localData, unitSpec)
	if localData.isEmpty == true then
		return
	end
	
	if not localData.buffCallbacks[unitSpec] then
		return
	end

	local buffIDs = Inspect.Buff.List(unitSpec)
	if not buffIDs then
		-- unitSpec empty so hide everything
		for buffName, buffDetail in pairs(localData.buffCallbacks[unitSpec]) do
			buffDetail.exists = false
			for cbId, callback in pairs(buffDetail.id) do
				callback.callback(unitSpec, buffName, buffDetail.duration, false)
			end
		end
		
		return
	end
	
	for buffName, buffDetail in pairs(localData.buffCallbacks[unitSpec]) do
		buffDetail.exists = false
	end
	
	local buffIdDeleteList = {}
	for buffId, buff in pairs(localData.buffByID) do
		if buff.unitSpec == unitSpec then
			table.insert(buffIdDeleteList, buffId)
		end
	end
	
	for indx, buffId in ipairs(buffIdDeleteList) do
		localData.buffByID[buffId] = nil
	end

	local unitID = Inspect.Unit.Lookup(unitSpec)
	if unitID then
		BuffManager.BuffExists(localData, unitSpec, unitID, buffIDs, false)
	end
	
	for buffName, buffDetail in pairs(localData.buffCallbacks[unitSpec]) do
		if buffDetail.exists == true then
			for cbId, callback in pairs(buffDetail.id) do
				if callback.eventType == BuffManager.existsType then
					if callback.stackCheckFunction then
						callback.callback(unitSpec, buffName, buffDetail.duration, callback.stackCheckFunction(buffDetail.stack))
					else
						callback.callback(unitSpec, buffName, buffDetail.duration, true)
					end
				else
					callback.callback(unitSpec, buffName, buffDetail.duration, false)
				end
			end
		elseif buffDetail.exists == false then
			for cbId, callback in pairs(buffDetail.id) do
				if callback.eventType == BuffManager.existsType then
					if callback.stackCheckFunction then
						callback.callback(unitSpec, buffName, buffDetail.duration, callback.stackCheckFunction(0))
					else
						callback.callback(unitSpec, buffName, buffDetail.duration, false)
					end
				else
					callback.callback(unitSpec, buffName, buffDetail.duration, true)
				end
			end
		end
	end	

	-- remove buff info for no longer existing buffs that dont have callbacks
	for buffName, buffDetail in pairs(localData.buffCallbacks[unitSpec]) do
		if buffDetail.exists == false then
			local removeBuff = true
			for cbID, callback in pairs(buffDetail.id) do
				removeBuff = false
			end

			if removeBuff then
				localData.buffCallbacks[unitSpec][buffName] = nil
			end
		end
	end	
end

function BuffManager.BuffExists(localData, inUnitSpec, unitID, buffIDs, throwEvents)
	if localData.isEmpty == true then
		return
	end
	
	local unitSpec = inUnitSpec
	if not unitSpec then
		local found = false
		local idLookup = {}
		for key, _ in pairs(localData.buffCallbacks) do
			idLookup[key] = 1
			found = true
		end
		
		if found == false then
			return
		end
		
		for thisSpec, thisID in pairs(Inspect.Unit.Lookup(idLookup)) do
			if thisID == unitID then
				unitSpec = thisSpec
				break
			end
		end
		
		if not unitSpec then
			return
		end
	end
	
	if localData.buffCallbacks[unitSpec] then
		local buffDetails = Inspect.Buff.Detail(unitID, buffIDs)
		for id, buffDetail in pairs(buffDetails) do
			local buffName = GetBuffName(buffDetail)
			if buffName then
				if not localData.buffCallbacks[unitSpec][buffName] then
					localData.buffCallbacks[unitSpec][buffName] = {}
					localData.buffCallbacks[unitSpec][buffName].id = {}
				end
				
				-- ignore non-player cast buffs if this is the player data set
				local processBuff = true
				if localData.isPlayer == true and buffDetail.caster ~= BuffManager.playerID then
					processBuff = false
				end
				
				if processBuff == true then
					-- ignore the 2nd buff with the same name but a different ability id.
					if not localData.buffCallbacks[unitSpec][buffName].ability or buffDetail.ability == localData.buffCallbacks[unitSpec][buffName].ability then
						local duration = 0
						if buffDetail.duration then
							duration = math.ceil(buffDetail.duration)
						end

						local stack = 0
						if buffDetail.stack then
							stack = buffDetail.stack
						end

						localData.buffCallbacks[unitSpec][buffName].exists = true
						localData.buffCallbacks[unitSpec][buffName].duration = duration
						localData.buffCallbacks[unitSpec][buffName].stack = stack
						localData.buffCallbacks[unitSpec][buffName].ability = buffDetail.ability

						localData.buffByID[id] = {}
						localData.buffByID[id].unitSpec = unitSpec
						localData.buffByID[id].buffName = buffName
						localData.buffByID[id].ability = buffDetail.ability

						if throwEvents == true then
							for cbId, callback in pairs(localData.buffCallbacks[unitSpec][buffName].id) do
								if callback.eventType == BuffManager.existsType then
									if callback.stackCheckFunction then
										callback.callback(unitSpec, buffName, duration, callback.stackCheckFunction(stack))
									else
										callback.callback(unitSpec, buffName, duration, true)
									end
								else
									callback.callback(unitSpec, buffName, duration, false)
								end
							end
						end
					--else
					--	print ("Ignored " .. buffName .. " got one=" .. localData.buffCallbacks[unitSpec][buffName].ability .. " " .. Utility.Serialize.Inline(buffDetail))
					end
				end
			end
		end
	end
end

function BuffManager.LocalDataBuffRemove(localData, unitSpec, buffIDs)
	if localData.isEmpty == true then
		return
	end
	
	if localData.buffCallbacks[unitSpec] then
		for id, details in pairs(buffIDs) do
			if localData.buffByID[id] then
				local buffName = localData.buffByID[id].buffName
				if localData.buffCallbacks[unitSpec][buffName] then
					localData.buffCallbacks[unitSpec][buffName].exists = false
					
					local duration = localData.buffCallbacks[unitSpec][buffName].duration
					local removeBuff = true
					for cbID, callback in pairs(localData.buffCallbacks[unitSpec][buffName].id) do
						removeBuff = false
						if callback.eventType == BuffManager.notExistsType then
							callback.callback(unitSpec, buffName, duration, true)
						elseif callback.stackCheckFunction then
							callback.callback(unitSpec, buffName, duration, callback.stackCheckFunction(0))
						else
							callback.callback(unitSpec, buffName, duration, false)
						end
					end

					if removeBuff then
						localData.buffCallbacks[unitSpec][buffName] = nil
					end
				end

				localData.buffByID[id] = nil
			end
		end
	end
end


--
-- Event Handlers
--

function BuffManager.BuffAdd(unitID, buffIDs)
	BuffManager.BuffExists(BuffManager.data.all, nil, unitID, buffIDs, true)
	BuffManager.BuffExists(BuffManager.data.player, nil, unitID, buffIDs, true)
end

function BuffManager.BuffChange(unitID, buffIDs)
	BuffManager.BuffExists(BuffManager.data.all, nil, unitID, buffIDs, true)
	BuffManager.BuffExists(BuffManager.data.player, nil, unitID, buffIDs, true)
end

function BuffManager.BuffRemove(unitID, buffIDs)
	local found = false
	local idLookup = {}
	for key, _ in pairs(BuffManager.data.all.buffCallbacks) do
		idLookup[key] = 1
		found = true
	end
	for key, _ in pairs(BuffManager.data.player.buffCallbacks) do
		idLookup[key] = 1
		found = true
	end
	
	if found == false then
		return
	end
	
	local unitSpec = ""
	for thisSpec, thisID in pairs(Inspect.Unit.Lookup(idLookup)) do
		if unitID == thisID then
			unitSpec = thisSpec
			break
		end
	end

	BuffManager.LocalDataBuffRemove(BuffManager.data.all, unitSpec, buffIDs)
	BuffManager.LocalDataBuffRemove(BuffManager.data.player, unitSpec, buffIDs)
end
