-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

CastbarEventConfig = {}
CastbarEventConfig.name = "CastbarEventConfig"
CastbarEventConfig.version = "$Id$"
CastbarEventConfig.TargetAndFocus = "player.target or focus"
CastbarEventConfig.Named = "named"
CastbarEventConfig.All = "all"
CastbarEventConfig.FireEventAlways = "Always fire warning"
CastbarEventConfig.FireEventTarget = "Only fire warning when I am target"
CastbarEventConfig.FireEventRange = "Only fire warning when it is close"

--
-- Main functions
--

--
-- UI functions
--

function CastbarEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtCastbarConfig = {}
	evtCastbarConfig.wmeConfig = wmeConfig
	evtCastbarConfig.parentFrame = parentFrame
	evtCastbarConfig.callback = callback
	evtCastbarConfig.event = CastbarEventConfig.GetDefaultEvent()
	evtCastbarConfig.txtPicker = wmeConfig.txtPicker
	
	evtCastbarConfig.windowFrame = UI.CreateFrame("RiftWindow", "CastbarEventSelector", parentFrame)
	evtCastbarConfig.windowFrame:SetTitle("Castbar Event Editor")
	evtCastbarConfig.windowFrame:SetWidth(700)
	evtCastbarConfig.windowFrame:SetHeight(790)
	evtCastbarConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtCastbarConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtCastbarConfig.windowFrame)
	evtCastbarConfig.okButton:SetPoint("BOTTOMLEFT", evtCastbarConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtCastbarConfig.okButton:SetText("OK")
	evtCastbarConfig.okButton.Event.LeftPress = function() CastbarEventConfig.WindowClose(evtCastbarConfig, true) end	

	evtCastbarConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtCastbarConfig.windowFrame)
	evtCastbarConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtCastbarConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtCastbarConfig.cancelButton:SetText("Cancel")
	evtCastbarConfig.cancelButton.Event.LeftPress = function() CastbarEventConfig.WindowClose(evtCastbarConfig, false) end	

	local margin = 4
	local height = 40
	evtCastbarConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtCastbarConfig.windowFrame)
	evtCastbarConfig.gridFrame:SetPoint("TOPLEFT", evtCastbarConfig.windowFrame, "TOPLEFT", 40, 80)
	evtCastbarConfig.rowLayout = RowLayout.Create(evtCastbarConfig.gridFrame, 7, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtCastbarConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtCastbarConfig.warningFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 2, 2, "Text")
	evtCastbarConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Ability Name")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The ability to monitor.  Ability must use castbar.")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Target")
	evtCastbarConfig.targetFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 4, 2, "Text")
	evtCastbarConfig.targetFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Target Name")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 5, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("NPC name to monitor, only if target=named")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Fire warning")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 6, 3, "Text")
	txtFrame:SetWordwrap(true)
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("Range")
	local txtFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 7, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The range to the person being hit by the cast")

	evtCastbarConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtCastbarConfig.windowFrame)
	evtCastbarConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtCastbarConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtCastbarConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtCastbarConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtCastbarConfig.scopeWindow = ScopeSelector.Create(evtCastbarConfig.scopeFrame, xMargin, yMargin, evtCastbarConfig.event.scope)

	evtCastbarConfig.nameFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 1, 2, "RiftTextfield")
	evtCastbarConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtCastbarConfig.strFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 3, 2, "RiftTextfield")
	evtCastbarConfig.strFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtCastbarConfig.targetNameFrame = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 5, 2, "RiftTextfield")
	evtCastbarConfig.targetNameFrame:SetBackgroundColor(0.15, 0.15, 0.15)

	evtCastbarConfig.rangeSlider = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 7, 2, "SimpleSlider", RowLayout.centerFrame)
	evtCastbarConfig.rangeSlider:SetWidth(RowLayout.GetColWidth(evtCastbarConfig.rowLayout, 2) - 20)
	evtCastbarConfig.rangeSlider:SetRange(0, 20)
	
	local items = {}
	table.insert(items, CastbarEventConfig.FireEventAlways)
	table.insert(items, CastbarEventConfig.FireEventTarget)
	table.insert(items, CastbarEventConfig.FireEventRange)
	evtCastbarConfig.fireSelect = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 6, 2, "SimpleSelectExt", RowLayout.centerFrame, true)
	evtCastbarConfig.fireSelect:SetWidth(RowLayout.GetColWidth(evtCastbarConfig.rowLayout, 2) - 4)
	evtCastbarConfig.fireSelect:SetHeight(24)
	evtCastbarConfig.fireSelect:SetBorder(1, 0.5, 0.5, 0.5, 1)
	evtCastbarConfig.fireSelect:SetItems(items)
	evtCastbarConfig.fireSelect.Event.ItemSelect = function() local enableRow = false if evtCastbarConfig.fireSelect:GetSelectedItem() == CastbarEventConfig.FireEventRange then enableRow = true end evtCastbarConfig.rangeSlider:SetEnabled(enableRow) end
	
	evtCastbarConfig.warningButton = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtCastbarConfig.warningButton:SetText("Choose")
	evtCastbarConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtCastbarConfig, WarnMe.warningTypeAll, function(a, warningName) CastbarEventConfig.ChooseWarningCallback(evtCastbarConfig, warningName) end) end
	evtCastbarConfig.targetButton = RowLayout.CreateFrame(evtCastbarConfig.rowLayout, 4, 3, "RiftButton", RowLayout.centerFrame)
	evtCastbarConfig.targetButton:SetText("Choose")
	evtCastbarConfig.targetButton.Event.LeftPress = function() CastbarEventConfig.ChooseUnitSpec(evtCastbarConfig, function(a, unitSpecName) CastbarEventConfig.ChooseUnitSpecCallback(evtCastbarConfig, unitSpecName) end) end
	RowLayout.SetColWidth(evtCastbarConfig.rowLayout, 3, evtCastbarConfig.warningButton:GetWidth())

	evtCastbarConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtCastbarConfig.parentFrame, evtCastbarConfig.windowFrame)
	
	RowLayout.Redraw(evtCastbarConfig.rowLayout)

	CastbarEventConfig.Show(evtCastbarConfig)
	evtCastbarConfig.windowFrame:SetVisible(false)
	
	return evtCastbarConfig
end

function CastbarEventConfig.WindowClose(evtCastbarConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtCastbarConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtCastbarConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local str = evtCastbarConfig.strFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify an ability name for this event\n"
		end
		if evtCastbarConfig.targetFrame:GetText() == "named" then
			local str = evtCastbarConfig.targetNameFrame:GetText()
			if not str or string.len(str) < 1 then
				message = message .. "You must specify a target name for this event\n"
			end
		end
		
		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtCastbarConfig.wrnWindow, message, function() end)
			return
		end
		
		CastbarEventConfig.FillEvent(evtCastbarConfig)
	end
	
	evtCastbarConfig.nameFrame:SetKeyFocus(false)
	evtCastbarConfig.strFrame:SetKeyFocus(false)
	evtCastbarConfig.targetNameFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtCastbarConfig.scopeWindow)

	evtCastbarConfig.windowFrame:SetVisible(false)
	if evtCastbarConfig.callback then
		if okPressed == true then
			evtCastbarConfig.callback(evtCastbarConfig, evtCastbarConfig.event)
		else
			evtCastbarConfig.callback(evtCastbarConfig)
		end
	end
end

function CastbarEventConfig.Show(evtCastbarConfig, inEvent)
	if inEvent then
		evtCastbarConfig.event = CastbarEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtCastbarConfig.event
	evtCastbarConfig.nameFrame:SetText(event.name)
	evtCastbarConfig.warningFrame:SetText(event.warningName)
	evtCastbarConfig.strFrame:SetText(event.castbar.abilityName)
	evtCastbarConfig.targetNameFrame:SetText("")
	RowLayout.SetRowVisible(evtCastbarConfig.rowLayout, 5, false)

	evtCastbarConfig.rangeSlider:SetEnabled(false)
	if event.castbar.iAmTarget == true then
		evtCastbarConfig.fireSelect:SetSelectedItem(CastbarEventConfig.FireEventTarget)
	elseif event.castbar.range ~= nil then
		evtCastbarConfig.fireSelect:SetSelectedItem(CastbarEventConfig.FireEventRange)
		evtCastbarConfig.rangeSlider:SetEnabled(true)
		evtCastbarConfig.rangeSlider:SetPosition(event.castbar.range)
	else
		evtCastbarConfig.fireSelect:SetSelectedItem(CastbarEventConfig.FireEventAlways)
	end

	if event.castbar.unitSpecs then
		if #event.castbar.unitSpecs > 1 then
			evtCastbarConfig.targetFrame:SetText(CastbarEventConfig.TargetAndFocus)
		elseif #event.castbar.unitSpecs == 1 then
			evtCastbarConfig.targetFrame:SetText(event.castbar.unitSpecs[1])
			if event.castbar.unitSpecs[1] == "named" then
				if evtCastbarConfig.event.castbar.targetName then
					evtCastbarConfig.targetNameFrame:SetText(evtCastbarConfig.event.castbar.targetName)
				end
				RowLayout.SetRowVisible(evtCastbarConfig.rowLayout, 5, true)
			end
		else
			evtCastbarConfig.targetFrame:SetText("")
		end
	end
	ScopeSelector.SetScope(evtCastbarConfig.scopeWindow, event.scope, event.scope.player)
	RowLayout.Redraw(evtCastbarConfig.rowLayout)
	evtCastbarConfig.windowFrame:SetVisible(true)
end

function CastbarEventConfig.FillEvent(evtCastbarConfig)
	evtCastbarConfig.event.name = evtCastbarConfig.nameFrame:GetText()
	evtCastbarConfig.event.warningName = evtCastbarConfig.warningFrame:GetText()
	evtCastbarConfig.event.castbar.abilityName = evtCastbarConfig.strFrame:GetText()
	evtCastbarConfig.event.castbar.targetName = evtCastbarConfig.targetNameFrame:GetText()
	evtCastbarConfig.event.scope = WarnMe.CopyScope(evtCastbarConfig.scopeWindow.scope)
	
	evtCastbarConfig.event.castbar.iAmTarget = nil
	evtCastbarConfig.event.castbar.range = nil
	local selectedItem = evtCastbarConfig.fireSelect:GetSelectedItem()
	if selectedItem == CastbarEventConfig.FireEventTarget then
		evtCastbarConfig.event.castbar.iAmTarget = true
	elseif selectedItem == CastbarEventConfig.FireEventRange then
		evtCastbarConfig.event.castbar.range = evtCastbarConfig.rangeSlider:GetPosition()
	end
	
	if evtCastbarConfig.targetFrame:GetText() == CastbarEventConfig.TargetAndFocus then
		evtCastbarConfig.event.castbar.unitSpecs = {}
		table.insert(evtCastbarConfig.event.castbar.unitSpecs, "player.target")
		table.insert(evtCastbarConfig.event.castbar.unitSpecs, "focus")
	elseif evtCastbarConfig.targetFrame:GetText() ~= "" then
		evtCastbarConfig.event.castbar.unitSpecs = {}
		table.insert(evtCastbarConfig.event.castbar.unitSpecs, evtCastbarConfig.targetFrame:GetText())
	else
		evtCastbarConfig.event.castbar.unitSpecs = nil
	end
end

function CastbarEventConfig.ChooseWarningCallback(evtCastbarConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		CastbarEventConfig.FillEvent(evtCastbarConfig)
		evtCastbarConfig.event.warningName = warningName
		CastbarEventConfig.Show(evtCastbarConfig)
	end
	
	evtCastbarConfig.windowFrame:SetVisible(true)
end

function CastbarEventConfig.ChooseUnitSpecCallback(evtCastbarConfig, unitSpec)
	if unitSpec and string.len(unitSpec) > 0 then
		CastbarEventConfig.FillEvent(evtCastbarConfig)
		if unitSpec == CastbarEventConfig.TargetAndFocus then
			evtCastbarConfig.event.castbar.unitSpecs = {}
			table.insert(evtCastbarConfig.event.castbar.unitSpecs, "player.target")
			table.insert(evtCastbarConfig.event.castbar.unitSpecs, "focus")
		else
			evtCastbarConfig.event.castbar.unitSpecs = {}
			table.insert(evtCastbarConfig.event.castbar.unitSpecs, unitSpec)
		end
		CastbarEventConfig.Show(evtCastbarConfig)
	end
	
	evtCastbarConfig.windowFrame:SetVisible(true)
end

function CastbarEventConfig.ChooseUnitSpec(wmeConfig, callback)
	local textList = {}
	table.insert(textList, "player")
	table.insert(textList, "player.target")
	table.insert(textList, "focus")
	table.insert(textList, CastbarEventConfig.TargetAndFocus)
	table.insert(textList, CastbarEventConfig.All)
	table.insert(textList, CastbarEventConfig.Named)
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Unit", textList, callback)
end


function CastbarEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = WarnMe.eventTypeCastbar
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.castbar = {}
	event.castbar.abilityName = event2.castbar.abilityName
	event.castbar.targetName = event2.castbar.targetName
	if event2.castbar.iAmTarget == true then
		event.castbar.iAmTarget = event2.castbar.iAmTarget
	elseif event2.castbar.range ~= nil then
		event.castbar.range = event2.castbar.range
	end
	
	event.castbar.unitSpecs = {}
	for _, spec in ipairs(event2.castbar.unitSpecs) do
		table.insert(event.castbar.unitSpecs, spec)
	end
	
	return event
end

function CastbarEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeCastbar
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.castbar = {}
	event.castbar.abilityName = ""
	event.castbar.targetName = ""
	event.castbar.unitSpecs = {}
	
	return event
end

