-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

ColourChooser = {}
ColourChooser.name = "ColourChooser"
ColourChooser.version = "v0.1"

--
-- Main functions
--

--
-- UI functions
--

function ColourChooser.Create(parentFrame, colour, callback)
	local clrChooser = {}
	clrChooser.parentFrame = parentFrame
	clrChooser.callback = callback
	
	local xMargin = 20
	local yMargin = 20
	
	clrChooser.colourFrame = UI.CreateFrame("RiftWindow", "ColourFrame", parentFrame)
	clrChooser.colourFrame:SetPoint("CENTER", parentFrame, "CENTER")
	clrChooser.colourFrame:SetWidth(300)
	clrChooser.colourFrame:SetHeight(328)
	clrChooser.colourFrame:SetTitle("Colour Chooser")
	clrChooser.colourFrame:SetLayer(5)

	clrChooser.okButton = UI.CreateFrame("RiftButton", "OKButton", clrChooser.colourFrame)
	clrChooser.okButton:SetPoint("BOTTOMLEFT", clrChooser.colourFrame, "BOTTOMLEFT", xMargin, -yMargin)
	clrChooser.okButton:SetText("OK")
	clrChooser.okButton.Event.LeftPress = function() ColourChooser.WindowClose(clrChooser, true) end	

	clrChooser.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", clrChooser.colourFrame)
	clrChooser.cancelButton:SetPoint("BOTTOMRIGHT", clrChooser.colourFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	clrChooser.cancelButton:SetText("Cancel")
	clrChooser.cancelButton.Event.LeftPress = function() ColourChooser.WindowClose(clrChooser, false) end	

	clrChooser.gridFrame = UI.CreateFrame("Frame", "GridFrame", clrChooser.colourFrame)
	clrChooser.gridFrame:SetPoint("TOPLEFT", clrChooser.colourFrame, "TOPLEFT", xMargin, yMargin + 40)
	clrChooser.gridFrame:SetPoint("TOPRIGHT", clrChooser.colourFrame, "TOPRIGHT", -xMargin, yMargin + 40)

	local margin = 4
	clrChooser.rowLayout = RowLayout.Create(clrChooser.gridFrame, 3, 2, 40, math.floor((clrChooser.gridFrame:GetWidth() - margin) / 2), margin, margin)
	
	local redTextFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 1, 1, "Text")
	redTextFrame:SetText("Red")
	local redNumberFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 1, 2, "Frame")
	clrChooser.redSlider = NumberSlider.Create(redNumberFrame, 0, 100, 1, 0, function(value) ColourChooser.RedSliderChanged(clrChooser, value) end)
	local greenTextFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 2, 1, "Text")
	greenTextFrame:SetText("Green")
	local colWidth = greenTextFrame:GetWidth()
	local greenNumberFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 2, 2, "Frame")
	clrChooser.greenSlider = NumberSlider.Create(greenNumberFrame, 0, 100, 1, 0, function(value) ColourChooser.GreenSliderChanged(clrChooser, value) end)
	local blueTextFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 3, 1, "Text")
	blueTextFrame:SetText("Blue")
	local blueNumberFrame = RowLayout.CreateFrame(clrChooser.rowLayout, 3, 2, "Frame")
	clrChooser.blueSlider = NumberSlider.Create(blueNumberFrame, 0, 100, 1, 0, function(value) ColourChooser.BlueSliderChanged(clrChooser, value) end)
	
	RowLayout.SetColWidth(clrChooser.rowLayout, 1, colWidth)
	RowLayout.SetColWidth(clrChooser.rowLayout, 2, clrChooser.gridFrame:GetWidth() - colWidth - margin)
	
	RowLayout.Redraw(clrChooser.rowLayout)

	clrChooser.colourSample = UI.CreateFrame("Frame", "ColourSample", clrChooser.colourFrame)
	clrChooser.colourSample:SetPoint("TOPLEFT", clrChooser.gridFrame, "BOTTOMLEFT", 0, yMargin)
	clrChooser.colourSample:SetPoint("TOPRIGHT", clrChooser.gridFrame, "BOTTOMRIGHT", 0, yMargin)
	clrChooser.colourSample:SetPoint("BOTTOMLEFT", clrChooser.okButton, "TOPLEFT", 0, -yMargin)
	clrChooser.colourSample:SetPoint("BOTTOMRIGHT", clrChooser.cancelButton, "TOPRIGHT", 0, -yMargin)

	ColourChooser.Show(clrChooser, colour)
	clrChooser.colourFrame:SetVisible(false)

	return clrChooser
end

function ColourChooser.Show(clrChooser, colour, callback)
	if callback then
		clrChooser.callback = callback
	end
	
	clrChooser.colour = {}
	clrChooser.colour.red = colour.red
	clrChooser.colour.green = colour.green
	clrChooser.colour.blue = colour.blue
	NumberSlider.SetValue(clrChooser.redSlider, math.floor(clrChooser.colour.red * 100), false)
	NumberSlider.SetValue(clrChooser.greenSlider, math.floor(clrChooser.colour.green * 100), false)
	NumberSlider.SetValue(clrChooser.blueSlider, math.floor(clrChooser.colour.blue * 100), false)
	clrChooser.colourSample:SetBackgroundColor(clrChooser.colour.red, clrChooser.colour.green, clrChooser.colour.blue)
	clrChooser.colourFrame:SetVisible(true)
end

function ColourChooser.WindowClose(clrChooser, okPressed)
	clrChooser.colourFrame:SetVisible(false)
	
	if okPressed then
		clrChooser.callback(clrChooser, clrChooser.colour)
	else
		clrChooser.callback(clrChooser)
	end
end

function ColourChooser.RedSliderChanged(clrChooser, value)
	clrChooser.colour.red = value / 100
	ColourChooser.Show(clrChooser, clrChooser.colour)
end

function ColourChooser.GreenSliderChanged(clrChooser, value)
	clrChooser.colour.green = value / 100
	ColourChooser.Show(clrChooser, clrChooser.colour)
end

function ColourChooser.BlueSliderChanged(clrChooser, value)
	clrChooser.colour.blue = value / 100
	ColourChooser.Show(clrChooser, clrChooser.colour)
end
