-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

CustomEvents = {}
CustomEvents.name = "CustomEvents"
CustomEvents.version = "$Id$"
CustomEvents.events = {}
CustomEvents.warnings = {}
CustomEvents.warning2eventMap = nil

--
-- Main functions
--

function CustomEvents.GetWarningEventMap()
	if not CustomEvents.warning2eventMap then
		CustomEvents.warning2eventMap = {}
		
		for name, event in pairs(CustomEvents.events) do
			if event and event.warningName then
				local warning = CustomEvents.warnings[event.warningName]
				if warning then
					if warning.warningType == WarnMe.warningTypeComposite then
						for _, compositeName in ipairs(warning.warningNames) do
							local compositeWarning = CustomEvents.warnings[compositeName]
							if not compositeWarning then print (compositeName) end
							if compositeWarning.warningType == WarnMe.warningTypeTimer then
								if compositeWarning.slider and compositeWarning.slider.enabled == true then
									if not CustomEvents.warning2eventMap[compositeName] then
										CustomEvents.warning2eventMap[compositeName] = {}
									end
									CustomEvents.warning2eventMap[compositeName][event.name] = 1
								end
						
								if compositeWarning.warningName then
									if not CustomEvents.warning2eventMap[compositeWarning.warningName] then
										CustomEvents.warning2eventMap[compositeWarning.warningName] = {}
									end
									CustomEvents.warning2eventMap[compositeWarning.warningName][event.name] = 1
								end
							else
								if not CustomEvents.warning2eventMap[compositeName] then
									CustomEvents.warning2eventMap[compositeName] = {}
								end
								CustomEvents.warning2eventMap[compositeName][event.name] = 1
							end
						end
					elseif warning.warningType == WarnMe.warningTypeTimer then
						if warning.slider and warning.slider.enabled == true then
							if not CustomEvents.warning2eventMap[event.warningName] then
								CustomEvents.warning2eventMap[event.warningName] = {}
							end
							CustomEvents.warning2eventMap[event.warningName][event.name] = 1
						end
						
						if warning.warningName then
							if not CustomEvents.warning2eventMap[warning.warningName] then
								CustomEvents.warning2eventMap[warning.warningName] = {}
							end
							CustomEvents.warning2eventMap[warning.warningName][event.name] = 1
						end
					else
						if not CustomEvents.warning2eventMap[event.warningName] then
							CustomEvents.warning2eventMap[event.warningName] = {}
						end
						CustomEvents.warning2eventMap[event.warningName][event.name] = 1
					end
				end
			end
		end
	end
	
	return CustomEvents.warning2eventMap
end

function CustomEvents.GetCustomEvents(enabledCustomWarnings)
	local events = {}
	local warnings = {}
	
	if enabledCustomWarnings then
		local warning2eventMap = CustomEvents.GetWarningEventMap()

		for name, _ in pairs(enabledCustomWarnings) do
			if warning2eventMap[name] then
				for eventName, _ in pairs(warning2eventMap[name]) do
					if not events[eventName] then
						local event = CustomEvents.events[eventName]
						if event then
							events[eventName] = event

							local warning = CustomEvents.warnings[event.warningName]
							if warning.warningType == WarnMe.warningTypeComposite then
								local newWarning = {
									name = warning.name,
									showIfAllVisible = warning.showIfAllVisible,
									warningNames = {
									},
									warningType = WarnMe.warningTypeComposite
								}

								for _, embeddedName in ipairs(warning.warningNames) do
									local compositeWarning = CustomEvents.warnings[embeddedName]
									if compositeWarning.warningType == WarnMe.warningTypeTimer then
										local newEmbeddedWarning = {
											duration = compositeWarning.duration,
											initialDelay = compositeWarning.initialDelay,
											name = compositeWarning.name,
											repeats = compositeWarning.repeats,
											warningType = WarnMe.warningTypeTimer,
											slider = { enabled = false }
										}

										local addWarning = false
										if enabledCustomWarnings[compositeWarning.warningName] then
											addWarning = true
											newEmbeddedWarning.warningName = compositeWarning.warningName

											if not warnings[compositeWarning.warningName] then
												warnings[compositeWarning.warningName] = CustomEvents.warnings[compositeWarning.warningName]
											end
										end

										if enabledCustomWarnings[embeddedName] then
											addWarning = true
											newEmbeddedWarning.slider.enabled = true
											newEmbeddedWarning.slider.xPct = compositeWarning.slider.xPct
											newEmbeddedWarning.slider.yPct = compositeWarning.slider.yPct
											newEmbeddedWarning.slider.widthPct = compositeWarning.slider.widthPct
											newEmbeddedWarning.slider.fontSize = compositeWarning.slider.fontSize
											newEmbeddedWarning.slider.colour = compositeWarning.slider.colour
											newEmbeddedWarning.slider.fontColour = compositeWarning.slider.fontColour
											newEmbeddedWarning.slider.text = compositeWarning.slider.text
										end

										if addWarning == true then
											warnings[embeddedName] = newEmbeddedWarning
											table.insert(newWarning.warningNames, embeddedName)
										end
									elseif enabledCustomWarnings[embeddedName] then
										table.insert(newWarning.warningNames, embeddedName)

										if not warnings[embeddedName] then
											warnings[embeddedName] = CustomEvents.warnings[embeddedName]
										end
									end
								end

								warnings[event.warningName] = newWarning
							elseif warning.warningType == WarnMe.warningTypeTimer then
								local newWarning = {
									duration = warning.duration,
									initialDelay = warning.initialDelay,
									name = warning.name,
									repeats = warning.repeats,
									warningType = WarnMe.warningTypeTimer,
									slider = { enabled = false }
								}

								local addWarning = false
								if enabledCustomWarnings[warning.warningName] then
									addWarning = true
									newWarning.warningName = warning.warningName

									if not warnings[warning.warningName] then
										warnings[warning.warningName] = CustomEvents.warnings[warning.warningName]
									end
								end

								if enabledCustomWarnings[event.warningName] then
									addWarning = true
									newWarning.slider.enabled = true
									newWarning.slider.xPct = warning.slider.xPct
									newWarning.slider.yPct = warning.slider.yPct
									newWarning.slider.widthPct = warning.slider.widthPct
									newWarning.slider.fontSize = warning.slider.fontSize
									newWarning.slider.colour = warning.slider.colour
									newWarning.slider.fontColour = warning.slider.fontColour
									newWarning.slider.text = warning.slider.text
								end

								if addWarning == true then
									warnings[event.warningName] = newWarning
								end
							else
								warnings[event.warningName] = warning
							end
						end
					end
				end
			end
		end
	end
	
	return events, warnings
end

-- Akylios ORB
local warning1 = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "Akylios: Orb Text",
	str = "ORB: Run to melee",
	warningType = WarnMe.warningTypeText,
	xPct = 50,
	yPct = 40
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 1,
		red = 1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Akylios: Orb Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Orb Flash and Text",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	emote = {
		message = "Jornaru launches a suffocating orb at {Player}."
	},
	eventType = WarnMe.eventTypeEmote,
	name = "Akylios: Orb",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Mind Decay
local warning1 = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "Akylios: Mind Decay Text",
	str = "Mind Decay: Run to melee",
	warningType = WarnMe.warningTypeText,
	xPct = 50,
	yPct = 50
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Akylios: Mind Decay Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Mind Decay Flash and Text",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Mind Decay",
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "Akylios: Mind Decay",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Waves P1
local warning = {
	addon = "WarnMe",
	alpha = 1,
	duration = 5,
	flashAlpha = 0.2,
	flashFrequency = 0,
	icon = "BlueWaveCircle.png",
	name = "Akylios: Waves Phase1 Texture",
	scale = 100,
	str = "",
	warningType = WarnMe.warningTypeTexture,
	xPct = 50,
	yPct = 55
}
CustomEvents.warnings[warning.name] = warning

local warning1 = {
	duration = 40,
	initialDelay = 40,
	name = "Akylios: Waves Phase1 Texture Timer",
	repeats = 4,
	warningName = "Akylios: Waves Phase1 Texture",
	slider = {
		enabled = false,
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	duration = 40,
	initialDelay = 43,
	name = "Akylios: Waves Phase1 Timer Bar",
	repeats = 4,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 5,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0,
			green = 0,
			blue = 1
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Wave in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Waves Phase1 Timer and Texture",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	eventType = WarnMe.eventTypeStat,
	name = "Akylios: Waves Phase1",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	stat = {
		pct = 50,
		isLess = false,
		targetName = "Jornaru",
		unitSpecs = {
			"named"
		}
	},
	statType = WarnMe.statTypeHealth,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Waves P4
local warning = {
	addon = "WarnMe",
	alpha = 1,
	duration = 4,
	flashAlpha = 0.2,
	flashFrequency = 0,
	icon = "BlueWaveCircle.png",
	name = "Akylios: Waves Phase4 Texture",
	scale = 100,
	str = "",
	warningType = WarnMe.warningTypeTexture,
	xPct = 50,
	yPct = 55
}
CustomEvents.warnings[warning.name] = warning

local warning1 = {
	duration = 50,
	initialDelay = 48,
	name = "Akylios: Waves Phase4 Texture Timer",
	repeats = 10,
	warningName = "Akylios: Waves Phase4 Texture",
	slider = {
		enabled = false
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	duration = 50,
	initialDelay = 50,
	name = "Akylios: Waves Phase4 Timer Bar",
	repeats = 10,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 5,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0,
			green = 0,
			blue = 1
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Wave in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Waves Phase4 Timer and Texture",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	eventType = WarnMe.eventTypeStat,
	name = "Akylios: Waves Phase4",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	stat = {
		pct = 55,
		isLess = true,
		targetName = "Akylios",
		unitSpecs = {
			"named"
		}
	},
	statType = WarnMe.statTypeHealth,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Adds P1
local warning1 = {
	duration = 70,
	name = "Akylios: Adds Phase1 Timer",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Next adds in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "Akylios: Adds Text",
	str = "Adds",
	warningType = WarnMe.warningTypeText,
	xPct = 30,
	yPct = 40
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Adds Phase1 Timer and Text",
	warningNames = {
		warning1.name,
		warning2.name
	},
	showIfAllVisible = true,
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	eventType = WarnMe.eventTypeStat,
	name = "Akylios: Adds Phase1 Health",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	stat = {
		pct = 51,
		isLess = false,
		targetName = "Jornaru",
		unitSpecs = {
			"named"
		}
	},
	statType = WarnMe.statTypeHealth,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

local event = {
	castbar = {
		abilityName = "Summon the Abyss",
		targetName = "Jornaru",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Akylios: Adds Phase1 Castbar",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Adds P2
local warning1 = {
	duration = 80,
	name = "Akylios: Adds Phase2 Timer",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Next adds in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning = {
	name = "Akylios: Adds Phase2 Timer and Text",
	warningNames = {
		warning1.name,
		warning2.name -- reused from P1 Adds
	},
	showIfAllVisible = true,
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	eventType = WarnMe.eventTypeStat,
	name = "Akylios: Adds Phase2 Health",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	stat = {
		pct = 50,
		isLess = true,
		targetName = "Jornaru",
		unitSpecs = {
			"named"
		}
	},
	statType = WarnMe.statTypeHealth,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

local event = {
	castbar = {
		abilityName = "Summon the Abyss",
		targetName = "Jornaru",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Akylios: Adds Phase2 Castbar",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}

	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Akylios Breath
local warning1 = {
	duration = 25,
	name = "Akylios: Breath Timer",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 15,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.67,
			green = 0,
			blue = 0.67
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Next breath in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0.67,
		green = 0,
		red = 0.67
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 1,
		green = 0,
		red = 0
	},
	flashFrequency = 0,
	name = "Akylios: Breath Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Akylios: Adds Breath Timer and Flash",
	warningNames = {
		warning1.name,
		warning2.name
	},
	showIfAllVisible = true,
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Breath of Madness",
		targetName = "Akylios",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Akylios: Breath Castbar",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Grugonim Rampant Decay
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 0.1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Grugonim: Rampant Decay Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Rampant Decay",
		targetName = "Grugonim",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Grugonim: Rampant Decay",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Grugonim Corrosive Bile
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0.67,
		green = 0,
		red = 0.67
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Grugonim: Corrosive Bile Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local warning = {
	duration = 2,
	initialDelay = 0,
	name = "Grugonim: Corrosive Bile Flash Timer",
	repeats = 0,
	warningName = "Grugonim: Corrosive Bile Flash",
	slider = {
		enabled = false,
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Corrosive Bile",
		targetName = "Grugonim",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Grugonim: Corrosive Bile",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Inquisitor Garau Arcane Porter
local warning1 = {
	duration = 45,
	name = "Inquisitor Garau: Arcane Porter Timer",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Next porter in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Inquisitor Garau: Arcane Porter Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Inquisitor Garau: Arcane Porter Flash and Timer",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	NpcChat = {
		message = "Power my creation!",
		name = "Inquisitor Garau"
	},
	eventType = WarnMe.eventTypeNpcChat,
	name = "Inquisitor Garau: Arcane Porter",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Inquisitor Garau Glacial Shield
local warning = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 0,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "Inquisitor Garau: Glacial Shield",
	str = "Shield",
	warningType = WarnMe.warningTypeText,
	xPct = 50,
	yPct = 40
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Glacial Shield",
		unitSpec = "player.target"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "Inquisitor Garau: Glacial Shield",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Matron Zamira Mark of Oblivion
local warning1 = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "Matron Zamira: Mark of Oblivion Text",
	str = "Mark: Run to melee",
	warningType = WarnMe.warningTypeText,
	xPct = 50,
	yPct = 40
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 1,
		red = 1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Matron Zamira: Mark of Oblivion Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Matron Zamira: Mark of Oblivion Flash and Text",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Mark of Oblivion",
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "Matron Zamira: Mark of Oblivion",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Matron Zamira: Hideous Blast
local warning = {
	alpha = 0.5,
	colour = {
		blue = 1,
		green = 0.67,
		red = 0.01
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Matron Zamira: Hideous Blast Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Hideous Blast",
		targetName = "Matron Zamira",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Matron Zamira: Hideous Blast",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Inwar Darktide: Surge
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 0.1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Inwar Darktide: Surge Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	emote = {
		message = "Rage surges through Inwar Darktide's body"
	},
	eventType = WarnMe.eventTypeEmote,
	name = "Inwar Darktide: Surge",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Murdantix: Soul Trauma
local warning = {
	alpha = 0.5,
	colour = {
		blue = 1,
		green = 0.67,
		red = 0.01
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Murdantix: Soul Trauma Surge Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Soul Trauma",
		targetName = "Murdantix",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Murdantix: Soul Trauma",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Soulrender Zilas: Soulrender's Grasp
local warning1 = {
	duration = 60,
	name = "Soulrender Zilas: Soulrender's Grasp Timer",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Next kite in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 1,
		green = 0.67,
		red = 0.01
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Soulrender Zilas: Soulrender's Grasp Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "Soulrender Zilas: Soulrender's Grasp Flash and Timer",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	emote = {
		message = "Soulrender Zilas surges with the power of Death!"
	},
	eventType = WarnMe.eventTypeEmote,
	name = "Soulrender Zilas: Soulrender's Grasp",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Beruhast: Inferno Lash
local warning = {
	alpha = 0.5,
	colour = {
		blue = 1,
		green = 0.67,
		red = 0.01
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Beruhast: Inferno Lash Surge Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Inferno Lash",
		targetName = "Beruhast",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "Beruhast: Inferno Lash",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- General Silgen: Heat Funnel
local warning1 = {
	alpha = 1,
	colour = {
		blue = 1,
		green = 1,
		red = 1
	},
	duration = 2,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 1,
	fontSize = 100,
	name = "General Silgen: Heat Funnel Text",
	str = "MOVE",
	warningType = WarnMe.warningTypeText,
	xPct = 50,
	yPct = 40
}
CustomEvents.warnings[warning1.name] = warning1

local warning2 = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "General Silgen: Heat Funnel Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning2.name] = warning2

local warning = {
	name = "General Silgen: Heat Funnel Flash and Text",
	warningNames = {
		warning1.name,
		warning2.name
	},
	warningType = WarnMe.warningTypeComposite
}
CustomEvents.warnings[warning.name] = warning

local event = {
	castbar = {
		abilityName = "Heat Funnel",
		targetName = "General Silgen",
		range = 7,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	name = "General Silgen: Heat Funnel",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- General Silgen: Tether
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0.02,
		green = 0.56,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "General Silgen: Tether Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Anchored in Flames",
		playersOwn = false,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "General Silgen: Tether",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- General Silgen: Incinerate
local warning = {
	alpha = 0.5,
	colour = {
		blue = 1,
		green = 0.67,
		red = 0.01
	},
	duration = 1,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "General Silgen: Incinerate Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	emote = {
		message = "You'll never escape!"
	},
	eventType = WarnMe.eventTypeEmote,
	name = "General Silgen: Incinerate",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- High Priestess Arakhurn: Slow Burn
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 5,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "High Priestess Arakhurn: Slow Burn Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Slow Burn",
		playersOwn = false,
		stacks = 6,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffStackGreater,
	name = "High Priestess Arakhurn: Slow Burn",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Warmaster Galenir: Essence Transfer
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 10,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Warmaster Galenir: Essence Transfer Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Essence Transfer",
		playersOwn = false,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "Warmaster Galenir: Essence Transfer",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["River of Souls"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Alsbeth the Discordant: Punish Soul
local warning = {
	alpha = 0.5,
	colour = {
		blue = 0,
		green = 0,
		red = 1
	},
	duration = 10,
	flashAlpha = 0.2,
	flashColour = {
		blue = 0,
		green = 0,
		red = 1
	},
	flashFrequency = 0,
	name = "Alsbeth the Discordant: Punish Soul Flash",
	screenPct = 4,
	warningType = WarnMe.warningTypeScreenFlash
}
CustomEvents.warnings[warning.name] = warning

local event = {
	buff = {
		name = "Punish Soul",
		playersOwn = false,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	name = "Alsbeth the Discordant: Punish Soul",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["River of Souls"] = 1
		}
	},
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- High Priest Arakhurn AOE Initial
local warning = {
	duration = 40,
	initialDelay = 0,
	name = "High Priest Arakhurn: AOE Initial Timer Bar",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 5,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0,
			green = 0,
			blue = 1
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "AOE in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning.name] = warning

local event = {
	eventType = WarnMe.eventTypeStat,
	name = "High Priest Arakhurn: AOE Initial",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	stat = {
		pct = 100,
		isLess = true,
		targetName = "High Priest Arakhurn",
		unitSpecs = {
			"named"
		}
	},
	statType = WarnMe.statTypeHealth,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- High Priest Arakhurn AOE Repeat
local warning = {
	duration = 62,
	initialDelay = 0,
	name = "High Priest Arakhurn: AOE Repeat Timer Bar",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 5,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0,
			green = 0,
			blue = 1
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "AOE in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning.name] = warning

local event = {
	name = "High Priest Arakhurn: AOE Repeat",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Rise of the Phoenix"] = 1
		}
	},
	castbar = {
		abilityName = "Fire Nova",
		targetName = "High Priest Arakhurn",
		iAmTarget = false,
		unitSpecs = {
			"named"
		}
	},
	eventType = WarnMe.eventTypeCastbar,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Sicaron Unholy Contract
local warning = {
	duration = 16,
	initialDelay = 0,
	name = "Sicaron: Unholy Contract Timer Bar",
	repeats = 0,
	slider = {
		enabled = true,
		xPct = 5,
		yPct = 5,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0,
			green = 0,
			blue = 1
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "Ends in {Countdown}"
	},
	warningType = WarnMe.warningTypeTimer
}
CustomEvents.warnings[warning.name] = warning

local event = {
	name = "Sicaron: Unholy Contract",
	scope = {
		inCombat = true,
		inRaid = true,
		zone = {
			["Hammerknell Fortress"] = 1
		}
	},
	buff = {
		name = "Unholy Contract",
		playersOwn = false,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	warningName = warning.name
}
CustomEvents.events[event.name] = event

-- Cancel Sheep
local warning = {
	name = "Sheep: Cancel Warn",
	target = "player",
	buff = "You've been turned into a Sheep.",
	warningType = WarnMe.warningTypeCancelBuff
}
CustomEvents.warnings[warning.name] = warning

local event = {
	name = "Sheep: Cancel",
	scope = {
		all = true
	},
	buff = {
		name = warning.buff,
		playersOwn = false,
		unitSpec = "player"
	},
	eventType = WarnMe.eventTypeBuffExists,
	warningName = warning.name
}
CustomEvents.events[event.name] = event
