-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

CustomWarningConfig = {}
CustomWarningConfig.name = "CustomWarningConfig"
CustomWarningConfig.version = "$Id$"

--
-- Main functions
--

--
-- UI functions
--

function CustomWarningConfig.Create(parentFrame, callback)
	local custWarnConfig = {}
	custWarnConfig.parentFrame = parentFrame
	custWarnConfig.callback = callback
	
	custWarnConfig.windowFrame = UI.CreateFrame("RiftWindow", "CustomWarningSelector", parentFrame)
	custWarnConfig.windowFrame:SetTitle("Custom Warnings")
	custWarnConfig.windowFrame:SetWidth(350)
	custWarnConfig.windowFrame:SetHeight(320)
	custWarnConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	custWarnConfig.textFrame = UI.CreateFrame("Text", "TextFrame", custWarnConfig.windowFrame)
	custWarnConfig.textFrame:SetPoint("TOPLEFT", custWarnConfig.windowFrame, "TOPLEFT", 20, 50)
	custWarnConfig.textFrame:SetPoint("TOPRIGHT", custWarnConfig.windowFrame, "TOPRIGHT", -20, 50)
	custWarnConfig.textFrame:SetHeight(100)
	custWarnConfig.textFrame:SetText("Custom warnings are provided by the addon for certain\nboss mechanics.\nYou can turn them on by pressing the Enable button.\nYou can also overwrite the warning by pressing the\nOverwrite button.\nOnce you overwrite a warning it will be availble to edit\nthrough the normal screens.")

	custWarnConfig.enableButton = UI.CreateFrame("RiftButton", "EnableButton", custWarnConfig.windowFrame)
	custWarnConfig.enableButton:SetPoint("TOPLEFT", custWarnConfig.textFrame, "BOTTOMLEFT", 0, 20)
	custWarnConfig.enableButton:SetText("Enable")
	custWarnConfig.enableButton.Event.LeftPress = function() CustomWarningConfig.EnableWarnings(custWarnConfig) end	

	custWarnConfig.overwriteButton = UI.CreateFrame("RiftButton", "OverwriteButton", custWarnConfig.windowFrame)
	custWarnConfig.overwriteButton:SetPoint("TOPRIGHT", custWarnConfig.textFrame, "BOTTOMRIGHT", 0, 20)
	custWarnConfig.overwriteButton:SetText("Overwrite")
	custWarnConfig.overwriteButton.Event.LeftPress = function() CustomWarningConfig.OverwriteWarnings(custWarnConfig) end	

	custWarnConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", custWarnConfig.windowFrame)
	custWarnConfig.cancelButton:SetPoint("BOTTOMRIGHT", custWarnConfig.windowFrame, "BOTTOMRIGHT", -20, -20)
	custWarnConfig.cancelButton:SetText("Cancel")
	custWarnConfig.cancelButton.Event.LeftPress = function() CustomWarningConfig.WindowClose(custWarnConfig, false) end	

	custWarnConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", custWarnConfig.windowFrame)
	custWarnConfig.okButton:SetPoint("BOTTOMLEFT", custWarnConfig.windowFrame, "BOTTOMLEFT", 20, -20)
	custWarnConfig.okButton:SetText("OK")
	custWarnConfig.okButton.Event.LeftPress = function() CustomWarningConfig.WindowClose(custWarnConfig, true) end	

	custWarnConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", custWarnConfig.windowFrame)
	custWarnConfig.cancelButton:SetPoint("BOTTOMRIGHT", custWarnConfig.windowFrame, "BOTTOMRIGHT", -20, -20)
	custWarnConfig.cancelButton:SetText("Cancel")
	custWarnConfig.cancelButton.Event.LeftPress = function() CustomWarningConfig.WindowClose(custWarnConfig, false) end	

	custWarnConfig.wrnWindow = WarningWindow.CreateWarningWindow(custWarnConfig.parentFrame, custWarnConfig.windowFrame)
	
	local warning2EventMap = CustomEvents.GetWarningEventMap()
	local warningNames = {}
	if warning2EventMap then
		for name, _ in pairs(warning2EventMap) do
			if name ~= "Akylios: Adds Text" then
				table.insert(warningNames, name)
			end
		end
	end

	table.sort(warningNames)
	custWarnConfig.warningNames = warningNames

	CustomWarningConfig.Show(custWarnConfig)
	custWarnConfig.windowFrame:SetVisible(false)
	
	return custWarnConfig
end

function CustomWarningConfig.WindowClose(custWarnConfig, okPressed)
	custWarnConfig.windowFrame:SetVisible(false)
	if custWarnConfig.callback then
		custWarnConfig.callback(custWarnConfig)
	end
end

function CustomWarningConfig.Show(custWarnConfig)
	custWarnConfig.windowFrame:SetVisible(true)
end

function CustomWarningConfig.WarningsSelected(custWarnConfig, checklist)
	if checklist then	
		local checkedWarnings = {}
		for i = 1, #custWarnConfig.warningNames do
			if checklist[i] == true then
				local warningName = custWarnConfig.warningNames[i]
				checkedWarnings[warningName] = 1
				if not WarnMe.enabledCustomWarnings[warningName] then
					WarnMe.enabledCustomWarnings[warningName] = {}
				end
			end
		end

		local warningsToDelete = {}
		for name, _ in pairs(WarnMe.enabledCustomWarnings) do
			if not checkedWarnings[name] then
				table.insert(warningsToDelete, name)
			end
		end

		for _, name in ipairs(warningsToDelete) do
			WarnMe.enabledCustomWarnings[name] = nil
		end
		
	end
	
	CustomWarningConfig.Show(custWarnConfig)
end

function CustomWarningConfig.EnableWarnings(custWarnConfig)
	local checkList = {}
	for _, name in ipairs(custWarnConfig.warningNames) do
		if WarnMe.enabledCustomWarnings[name] then
			table.insert(checkList, true)
		else
			table.insert(checkList, false)
		end
	end
	
	custWarnConfig.windowFrame:SetVisible(false)
	local txtSelector = TextPicker.Create(custWarnConfig.windowFrame, true)
	TextPicker.Show(txtSelector, "Select Custom Warnings to Enable", custWarnConfig.warningNames, function(a, checked) CustomWarningConfig.WarningsSelected(custWarnConfig, checked) end, checkList)
end

function CustomWarningConfig.OverwriteSelected(custWarnConfig, checklist)
	if checklist then	
		local checkedWarnings = {}
		for i = 1, #custWarnConfig.warningNames do
			if checklist[i] == true then
				local warningName = custWarnConfig.warningNames[i]
				checkedWarnings[warningName] = 1
			end
		end

		for name, _ in pairs(checkedWarnings) do
			if WarnMe.totalWarnings[name] then
				print("Warning " .. name .. " is already overwritten, ignoring")
			else
				WarnMe.totalWarnings[name] = CustomEvents.warnings[name]
			end
		end
	end
	
	CustomWarningConfig.Show(custWarnConfig)
end

function CustomWarningConfig.OverwriteWarnings(custWarnConfig)
	custWarnConfig.windowFrame:SetVisible(false)
	local txtSelector = TextPicker.Create(custWarnConfig.windowFrame, true)
	TextPicker.Show(txtSelector, "Select Custom Warnings to Overwrite", custWarnConfig.warningNames, function(a, checked) CustomWarningConfig.OverwriteSelected(custWarnConfig, checked) end, checkList)
end
