-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

EmoteEventConfig = {}
EmoteEventConfig.name = "EmoteEventConfig"
EmoteEventConfig.version = "$Id$"

--
-- Main functions
--

--
-- UI functions
--

function EmoteEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtEmoteConfig = {}
	evtEmoteConfig.wmeConfig = wmeConfig
	evtEmoteConfig.parentFrame = parentFrame
	evtEmoteConfig.callback = callback
	evtEmoteConfig.event = EmoteEventConfig.GetDefaultEvent()
	evtEmoteConfig.txtPicker = wmeConfig.txtPicker
	
	evtEmoteConfig.windowFrame = UI.CreateFrame("RiftWindow", "EmoteEventSelector", parentFrame)
	evtEmoteConfig.windowFrame:SetTitle("Emote Event Editor")
	evtEmoteConfig.windowFrame:SetWidth(700)
	evtEmoteConfig.windowFrame:SetHeight(700)
	evtEmoteConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtEmoteConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtEmoteConfig.windowFrame)
	evtEmoteConfig.okButton:SetPoint("BOTTOMLEFT", evtEmoteConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtEmoteConfig.okButton:SetText("OK")
	evtEmoteConfig.okButton.Event.LeftPress = function() EmoteEventConfig.WindowClose(evtEmoteConfig, true) end	

	evtEmoteConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtEmoteConfig.windowFrame)
	evtEmoteConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtEmoteConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtEmoteConfig.cancelButton:SetText("Cancel")
	evtEmoteConfig.cancelButton.Event.LeftPress = function() EmoteEventConfig.WindowClose(evtEmoteConfig, false) end	

	local margin = 4
	local height = 40
	evtEmoteConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtEmoteConfig.windowFrame)
	evtEmoteConfig.gridFrame:SetPoint("TOPLEFT", evtEmoteConfig.windowFrame, "TOPLEFT", 40, 80)
	evtEmoteConfig.rowLayout = RowLayout.Create(evtEmoteConfig.gridFrame, 3, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtEmoteConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtEmoteConfig.warningFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 2, 2, "Text")
	evtEmoteConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Emote")
	local txtFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The emote message to monitor")

	evtEmoteConfig.nameFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 1, 2, "RiftTextfield")
	evtEmoteConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtEmoteConfig.strFrame = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 3, 2, "RiftTextfield")
	evtEmoteConfig.strFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	evtEmoteConfig.warningButton = RowLayout.CreateFrame(evtEmoteConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtEmoteConfig.warningButton:SetText("Choose")
	evtEmoteConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtEmoteConfig, WarnMe.warningTypeAll, function(a, warningName) EmoteEventConfig.ChooseWarningCallback(evtEmoteConfig, warningName) end) end
	RowLayout.SetColWidth(evtEmoteConfig.rowLayout, 3, evtEmoteConfig.warningButton:GetWidth())

	RowLayout.Redraw(evtEmoteConfig.rowLayout)

	evtEmoteConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtEmoteConfig.windowFrame)
	evtEmoteConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtEmoteConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtEmoteConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtEmoteConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtEmoteConfig.scopeWindow = ScopeSelector.Create(evtEmoteConfig.scopeFrame, xMargin, yMargin, evtEmoteConfig.event.scope)

	evtEmoteConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtEmoteConfig.parentFrame, evtEmoteConfig.windowFrame)
	
	EmoteEventConfig.Show(evtEmoteConfig)
	evtEmoteConfig.windowFrame:SetVisible(false)
	
	return evtEmoteConfig
end

function EmoteEventConfig.WindowClose(evtEmoteConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtEmoteConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtEmoteConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local str = evtEmoteConfig.strFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify an emote string for this event\n"
		end
		
		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtEmoteConfig.wrnWindow, message, function() end)
			return
		end
		
		EmoteEventConfig.FillEvent(evtEmoteConfig)
	end
	
	evtEmoteConfig.nameFrame:SetKeyFocus(false)
	evtEmoteConfig.strFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtEmoteConfig.scopeWindow)
	
	evtEmoteConfig.windowFrame:SetVisible(false)
	if evtEmoteConfig.callback then
		if okPressed == true then
			evtEmoteConfig.callback(evtEmoteConfig, evtEmoteConfig.event)
		else
			evtEmoteConfig.callback(evtEmoteConfig)
		end
	end
end

function EmoteEventConfig.Show(evtEmoteConfig, inEvent)
	if inEvent then
		evtEmoteConfig.event = EmoteEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtEmoteConfig.event
	evtEmoteConfig.nameFrame:SetText(event.name)
	evtEmoteConfig.warningFrame:SetText(event.warningName)
	evtEmoteConfig.strFrame:SetText(event.emote.message)
	ScopeSelector.SetScope(evtEmoteConfig.scopeWindow, event.scope, event.scope.player)
	evtEmoteConfig.windowFrame:SetVisible(true)
end

function EmoteEventConfig.FillEvent(evtEmoteConfig)
	evtEmoteConfig.event.name = evtEmoteConfig.nameFrame:GetText()
	evtEmoteConfig.event.warningName = evtEmoteConfig.warningFrame:GetText()
	evtEmoteConfig.event.emote.message = evtEmoteConfig.strFrame:GetText()
	evtEmoteConfig.event.scope = WarnMe.CopyScope(evtEmoteConfig.scopeWindow.scope)
end

function EmoteEventConfig.ChooseWarningCallback(evtEmoteConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		EmoteEventConfig.FillEvent(evtEmoteConfig)
		evtEmoteConfig.event.warningName = warningName
		EmoteEventConfig.Show(evtEmoteConfig)
	end
	
	evtEmoteConfig.windowFrame:SetVisible(true)
end

function EmoteEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = WarnMe.eventTypeEmote
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.emote = {}
	event.emote.message = event2.emote.message
	
	return event
end

function EmoteEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeEmote
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.emote = {}
	event.emote.message = ""
	
	return event
end

