-- Helper Functions

local function ListWidgetSelectionChange(widget)
  if widget.Event.SelectionChange then
    widget.Event.SelectionChange(widget)
  end
end

local function SetRows(widget, rows)
	if rows ~= nil and type(rows) == "number" and rows > 0 then
		widget.rows = rows
	else
		widget.rows = widget.maxRows
	end
	
	widget:SetHeight(widget:GetHeight())
end

local function GetRows(widget)
	if widget.rows == nil then
		SetRows(widget, widget.maxRows)
	end
	
	return widget.rows
end

local function GetHeight(widget)
	if #widget.listWidget.itemFrames < 1 then
		widget.listWidget:SetItems({" "})
	end
	
	return widget.listWidget.itemFrames[1]:GetHeight() * GetRows(widget)
end

local function SetItems(widget, items, values, levels)
	widget.listWidget:SetItems(items, values, levels)
	
	if #widget.listWidget.items > 0 then
		if #widget.listWidget.items < widget.maxRows then
			SetRows(widget, #widget.listWidget.items)
		else
			SetRows(widget, widget.maxRows)
		end
	end
end

-- Constructor Function

function Library.LibSimpleWidgetsExt.ScrollList(name, parent)
  local widget = UI.CreateFrame("SimpleScrollView", name, parent)
  local listWidget = UI.CreateFrame("SimpleList", name .. "List", widget)
  listWidget.Event.SelectionChange = function () ListWidgetSelectionChange(widget) end
  widget.listWidget = listWidget
  widget.maxRows = 5

  widget.GetFontSize = function(a, ...) listWidget:GetFontSize(...) end
  widget.SetFontSize = function(a, ...) listWidget:SetFontSize(...) end
  widget.GetEnabled = function(a, ...) listWidget:GetEnabled(...) end
  widget.SetEnabled = function(a, ...) listWidget:SetEnabled(...) end
  widget.SetLevelIndentSize = function(a, ...) listWidget:SetLevelIndentSize(...) end
  widget.SetLevelFontSize = function(a, ...) listWidget:SetLevelFontSize(...) end
  widget.SetLevelFontColor = function(a, ...) listWidget:SetLevelFontColor(...) end
  widget.SetLevelBackgroundColor = function(a, ...) listWidget:SetLevelBackgroundColor(...) end
  widget.SetLevelSelectable = function(a, ...) listWidget:SetLevelSelectable(...) end
  widget.GetItems = function(a, ...) listWidget:GetItems(...) end
  widget.SetItems = SetItems
  widget.GetValues = function(a, ...) listWidget:GetValues(...) end
  widget.SetSelectionMode = function(a, ...) listWidget:SetSelectionMode(...) end
  widget.GetSelectedIndex = function(a, ...) listWidget:GetSelectedIndex(...) end
  widget.SetSelectedIndex = function(a, ...) listWidget:SetSelectedIndex(...) end
  widget.GetSelectedItem = function(a, ...) listWidget:GetSelectedItem(...) end
  widget.SetSelectedItem = function(a, ...) listWidget:SetSelectedItem(...) end
  widget.GetSelectedValue = function(a, ...) listWidget:GetSelectedValue(...) end
  widget.SetSelectedValue = function(a, ...) listWidget:SetSelectedValue(...) end
  widget.AddSelectedIndex = function(a, ...) listWidget:AddSelectedIndex(...) end
  widget.RemoveSelectedIndex = function(a, ...) listWidget:RemoveSelectedIndex(...) end
  widget.GetSelection = function(a, ...) listWidget:GetSelection(...) end
  widget.ClearSelection = function(a, ...) listWidget:ClearSelection(...) end
  widget.GetSelectedIndices = function(a, ...) listWidget:GetSelectedIndices(...) end
  widget.SetSelectedIndices = function(a, ...) listWidget:SetSelectedIndices(...) end
  widget.GetSelectedItems = function(a, ...) listWidget:GetSelectedItems(...) end
  widget.SetSelectedItems = function(a, ...) listWidget:SetSelectedItems(...) end
  widget.GetSelectedValues = function(a, ...) listWidget:GetSelectedValues(...) end
  widget.SetSelectedValues = function(a, ...) listWidget:SetSelectedValues(...) end
  widget.SetRows = SetRows
  widget.GetRows = GetRows
  widget.GetHeight = GetHeight

  Library.LibSimpleWidgets.EventProxy(widget, {"ItemSelect", "SelectionChange"})

  widget:SetContent(listWidget)
  
  return widget
end
