-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

NpcChatEventConfig = {}
NpcChatEventConfig.name = "NpcChatEventConfig"
NpcChatEventConfig.version = "$Id$"

--
-- Main functions
--

--
-- UI functions
--

function NpcChatEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtNpcChatConfig = {}
	evtNpcChatConfig.wmeConfig = wmeConfig
	evtNpcChatConfig.parentFrame = parentFrame
	evtNpcChatConfig.callback = callback
	evtNpcChatConfig.event = NpcChatEventConfig.GetDefaultEvent()
	evtNpcChatConfig.txtPicker = wmeConfig.txtPicker
	
	evtNpcChatConfig.windowFrame = UI.CreateFrame("RiftWindow", "NpcChatEventSelector", parentFrame)
	evtNpcChatConfig.windowFrame:SetTitle("NPC Chat Event Editor")
	evtNpcChatConfig.windowFrame:SetWidth(700)
	evtNpcChatConfig.windowFrame:SetHeight(700)
	evtNpcChatConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtNpcChatConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtNpcChatConfig.windowFrame)
	evtNpcChatConfig.okButton:SetPoint("BOTTOMLEFT", evtNpcChatConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtNpcChatConfig.okButton:SetText("OK")
	evtNpcChatConfig.okButton.Event.LeftPress = function() NpcChatEventConfig.WindowClose(evtNpcChatConfig, true) end	

	evtNpcChatConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtNpcChatConfig.windowFrame)
	evtNpcChatConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtNpcChatConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtNpcChatConfig.cancelButton:SetText("Cancel")
	evtNpcChatConfig.cancelButton.Event.LeftPress = function() NpcChatEventConfig.WindowClose(evtNpcChatConfig, false) end	

	local margin = 4
	local height = 40
	evtNpcChatConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtNpcChatConfig.windowFrame)
	evtNpcChatConfig.gridFrame:SetPoint("TOPLEFT", evtNpcChatConfig.windowFrame, "TOPLEFT", 40, 80)
	evtNpcChatConfig.rowLayout = RowLayout.Create(evtNpcChatConfig.gridFrame, 4, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtNpcChatConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtNpcChatConfig.warningFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 2, 2, "Text")
	evtNpcChatConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Npc Name")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the Npc to monitor")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Npc Message")
	local txtFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 4, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The message said by the Npc to fire the event")

	evtNpcChatConfig.nameFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 1, 2, "RiftTextfield")
	evtNpcChatConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtNpcChatConfig.npcNameFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 3, 2, "RiftTextfield")
	evtNpcChatConfig.npcNameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtNpcChatConfig.npcMsgFrame = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 4, 2, "RiftTextfield")
	evtNpcChatConfig.npcMsgFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	evtNpcChatConfig.warningButton = RowLayout.CreateFrame(evtNpcChatConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtNpcChatConfig.warningButton:SetText("Choose")
	evtNpcChatConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtNpcChatConfig, WarnMe.warningTypeAll, function(a, warningName) NpcChatEventConfig.ChooseWarningCallback(evtNpcChatConfig, warningName) end) end
	RowLayout.SetColWidth(evtNpcChatConfig.rowLayout, 3, evtNpcChatConfig.warningButton:GetWidth())

	RowLayout.Redraw(evtNpcChatConfig.rowLayout)

	evtNpcChatConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtNpcChatConfig.windowFrame)
	evtNpcChatConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtNpcChatConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtNpcChatConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtNpcChatConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtNpcChatConfig.scopeWindow = ScopeSelector.Create(evtNpcChatConfig.scopeFrame, xMargin, yMargin, evtNpcChatConfig.event.scope)

	evtNpcChatConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtNpcChatConfig.parentFrame, evtNpcChatConfig.windowFrame)
	
	NpcChatEventConfig.Show(evtNpcChatConfig)
	evtNpcChatConfig.windowFrame:SetVisible(false)
	
	return evtNpcChatConfig
end

function NpcChatEventConfig.WindowClose(evtNpcChatConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtNpcChatConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtNpcChatConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local str = evtNpcChatConfig.npcNameFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify a npc name for this event\n"
		end
		local str = evtNpcChatConfig.npcMsgFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify a npc message for this event\n"
		end
		
		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtNpcChatConfig.wrnWindow, message, function() end)
			return
		end
		
		NpcChatEventConfig.FillEvent(evtNpcChatConfig)
	end

	evtNpcChatConfig.nameFrame:SetKeyFocus(false)
	evtNpcChatConfig.npcNameFrame:SetKeyFocus(false)
	evtNpcChatConfig.npcMsgFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtNpcChatConfig.scopeWindow)
	
	evtNpcChatConfig.windowFrame:SetVisible(false)
	if evtNpcChatConfig.callback then
		if okPressed == true then
			evtNpcChatConfig.callback(evtNpcChatConfig, evtNpcChatConfig.event)
		else
			evtNpcChatConfig.callback(evtNpcChatConfig)
		end
	end
end

function NpcChatEventConfig.Show(evtNpcChatConfig, inEvent)
	if inEvent then
		evtNpcChatConfig.event = NpcChatEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtNpcChatConfig.event
	evtNpcChatConfig.nameFrame:SetText(event.name)
	evtNpcChatConfig.warningFrame:SetText(event.warningName)
	evtNpcChatConfig.npcNameFrame:SetText(event.NpcChat.name)
	evtNpcChatConfig.npcMsgFrame:SetText(event.NpcChat.message)
	ScopeSelector.SetScope(evtNpcChatConfig.scopeWindow, event.scope, event.scope.player)
	evtNpcChatConfig.windowFrame:SetVisible(true)
end

function NpcChatEventConfig.FillEvent(evtNpcChatConfig)
	evtNpcChatConfig.event.name = evtNpcChatConfig.nameFrame:GetText()
	evtNpcChatConfig.event.warningName = evtNpcChatConfig.warningFrame:GetText()
	evtNpcChatConfig.event.NpcChat.name = evtNpcChatConfig.npcNameFrame:GetText()
	evtNpcChatConfig.event.NpcChat.message = evtNpcChatConfig.npcMsgFrame:GetText()
	evtNpcChatConfig.event.scope = WarnMe.CopyScope(evtNpcChatConfig.scopeWindow.scope)
end

function NpcChatEventConfig.ChooseWarningCallback(evtNpcChatConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		NpcChatEventConfig.FillEvent(evtNpcChatConfig)
		evtNpcChatConfig.event.warningName = warningName
		NpcChatEventConfig.Show(evtNpcChatConfig)
	end
	
	evtNpcChatConfig.windowFrame:SetVisible(true)
end

function NpcChatEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = WarnMe.eventTypeNpcChat
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.NpcChat = {}
	event.NpcChat.name = event2.NpcChat.name
	event.NpcChat.message = event2.NpcChat.message
	
	return event
end

function NpcChatEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeNpcChat
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.NpcChat = {}
	event.NpcChat.name = ""
	event.NpcChat.message = ""
	
	return event
end

