-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

ScopeSelector = {}
ScopeSelector.name = "ScopeSelector"
ScopeSelector.version = "$Id$"
ScopeSelector.textList = { "Drowned Halls", "Gilded Prophecy", "Greenscale's Blight", "Hammerknell Fortress", "Rise of the Phoenix", "River of Souls" }
ScopeSelector.MaxRoles = 6

--
-- Main functions
--

--
-- UI functions
--

function ScopeSelector.Create(parentFrame, xMargin, yMargin, scope)
	local scopeSelector = {}
	scopeSelector.parentFrame = parentFrame
	scopeSelector.callback = callback
	scopeSelector.colour = colour
	scopeSelector.scope = scope
	
	scopeSelector.titleFrame = UI.CreateFrame("Text", "TitleFrame", parentFrame)
	scopeSelector.titleFrame:SetText("Specify when this event fires:")
	scopeSelector.titleFrame:SetFontSize(16)
	scopeSelector.titleFrame:SetPoint("TOPLEFT", parentFrame, "TOPLEFT", xMargin, yMargin)

	local titleHeight = scopeSelector.titleFrame:GetHeight()
	scopeSelector.gridFrame = UI.CreateFrame("Frame", "GridFrame", parentFrame)
	scopeSelector.gridFrame:SetPoint("TOPLEFT", parentFrame, "TOPLEFT", xMargin, yMargin + titleHeight)
	scopeSelector.gridFrame:SetPoint("TOPRIGHT", parentFrame, "TOPRIGHT", -xMargin, yMargin + titleHeight)

	local margin = 4
	local cols = ScopeSelector.MaxRoles
	scopeSelector.rowLayout = RowLayout.Create(scopeSelector.gridFrame, 10, cols, 20, math.floor((scopeSelector.gridFrame:GetWidth() - ((cols - 1) * margin)) / cols), margin, margin)
	
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 1, 1, "Text", RowLayout.centerFrame)
	textFrame:SetText("Always")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 3, 1, "Text", RowLayout.centerFrame)
	textFrame:SetText("Solo")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 3, 2, "Text", RowLayout.centerFrame)
	textFrame:SetText("Party")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 3, 3, "Text", RowLayout.centerFrame)
	textFrame:SetText("Raid")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 3, 4, "Text", RowLayout.centerFrame)
	textFrame:SetText("WF")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 1, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 1")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 2, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 2")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 3, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 3")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 4, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 4")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 5, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 5")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 5, 6, "Text", RowLayout.centerFrame)
	textFrame:SetText("Role 6")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 7, 1, "Text", RowLayout.centerFrame)
	textFrame:SetText("In Combat")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 7, 2, "Text", RowLayout.centerFrame)
	textFrame:SetText("Out combat")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 9, 1, "Text", RowLayout.centerFrame)
	textFrame:SetText("All characters")
	local textFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, 9, 2, "Text", RowLayout.centerFrame)
	textFrame:SetText("character(s)")
	
	ScopeSelector.CreateCheckbox(scopeSelector, 2, 1)
	ScopeSelector.CreateCheckbox(scopeSelector, 4, 4)
	ScopeSelector.CreateCheckbox(scopeSelector, 6, ScopeSelector.MaxRoles)
	ScopeSelector.CreateCheckbox(scopeSelector, 8, 2)
	ScopeSelector.CreateCheckbox(scopeSelector, 10, 1)

	RowLayout.Redraw(scopeSelector.rowLayout)

	scopeSelector.playerFrame = UI.CreateFrame("RiftTextfield", "PlayerFrame", parentFrame)
	scopeSelector.playerFrame:SetPoint("BOTTOMLEFT", scopeSelector.gridFrame, "BOTTOMLEFT", RowLayout.GetColWidth(scopeSelector.rowLayout, 1) + margin, 0)
	scopeSelector.playerFrame:SetPoint("BOTTOMRIGHT", scopeSelector.gridFrame, "BOTTOMRIGHT", 0, 0)
	scopeSelector.playerFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	scopeSelector.playerFrame.Event.TextfieldChange = function() ScopeSelector.ScopeChanged(scopeSelector) end
	
	local zoneLabel = UI.CreateFrame("Text", "ZoneLabelFrame", parentFrame)
	zoneLabel:SetText("Zone")
	local xOffset = math.floor((RowLayout.GetColWidth(scopeSelector.rowLayout, 1) + margin - zoneLabel:GetWidth()) / 2)
	zoneLabel:SetPoint("TOPLEFT", scopeSelector.gridFrame, "BOTTOMLEFT", xOffset, yMargin)
	
	scopeSelector.zoneButton = UI.CreateFrame("RiftButton", "ZoneButton", parentFrame)
	scopeSelector.zoneButton:SetPoint("TOPRIGHT", scopeSelector.playerFrame, "BOTTOMRIGHT", 0, yMargin)
	scopeSelector.zoneButton:SetText("Change")
	scopeSelector.zoneButton.Event.LeftPress = function() ScopeSelector.ShowZonePicker(scopeSelector) end
	
	scopeSelector.zoneFrame = UI.CreateFrame("Text", "ZoneFrame", parentFrame)
	scopeSelector.zoneFrame:SetPoint("TOPLEFT", scopeSelector.playerFrame, "BOTTOMLEFT", 0, yMargin)
	scopeSelector.zoneFrame:SetPoint("TOPRIGHT", scopeSelector.zoneButton, "TOPLEFT", -xMargin, 0)
	scopeSelector.zoneFrame:SetText("")
	
	scopeSelector.descFrame = UI.CreateFrame("Text", "DescFrame", parentFrame)
	scopeSelector.descFrame:SetPoint("TOPLEFT", zoneLabel, "BOTTOMLEFT", 0, yMargin)
	scopeSelector.descFrame:SetPoint("TOPRIGHT", scopeSelector.zoneFrame, "BOTTOMRIGHT", 0, yMargin)
	scopeSelector.descFrame:SetWordwrap(true)
	scopeSelector.descFrame:SetHeight(90)

	ScopeSelector.InternalSetScope(scopeSelector, scope)
	
	ScopeSelector.SetCallbacks(scopeSelector, 2, 1)
	ScopeSelector.SetCallbacks(scopeSelector, 4, 4)
	ScopeSelector.SetCallbacks(scopeSelector, 6, ScopeSelector.MaxRoles)
	ScopeSelector.SetCallbacks(scopeSelector, 8, 2)
	ScopeSelector.SetCallbacks(scopeSelector, 10, 1)
	
	scopeSelector.txtPicker = TextPicker.Create(parentFrame, true)
	
	return scopeSelector
end

function ScopeSelector.CreateCheckbox(scopeSelector, row, cols)
	for col = 1, cols do
		local chkFrame = RowLayout.CreateFrame(scopeSelector.rowLayout, row, col, "RiftCheckbox", RowLayout.centerFrame)
		chkFrame:SetWidth(16)
		chkFrame:SetHeight(16)
	end
end

function ScopeSelector.SetCheckbox(scopeSelector, row, col, value)
	local chkFrame = RowLayout.GetCellFrame(scopeSelector.rowLayout, row, col)
	scopeSelector.inCheckedCallback = true
	chkFrame:SetChecked(value)
	scopeSelector.inCheckedCallback = false
end

function ScopeSelector.SetCheckboxs(scopeSelector, row, cols, value)
	for col = 1, cols do
		ScopeSelector.SetCheckbox(scopeSelector, row, col, value)
	end
end

function ScopeSelector.SetCallbacks(scopeSelector, row, cols)
	for col = 1, cols do
		RowLayout.GetCellFrame(scopeSelector.rowLayout, row, col).Event.CheckboxChange = function() ScopeSelector.ScopeChanged(scopeSelector) end
	end
end

function ScopeSelector.ZonesChosen(scopeSelector, checkList)
	if checkList then
		scopeSelector.scope.zone = {}
		for i = 1, #checkList do
			if checkList[i] == true then
				scopeSelector.scope.zone[ScopeSelector.textList[i]] = 1
			end
		end
		
		ScopeSelector.SetScope(scopeSelector, scopeSelector.scope)
	end
end

function ScopeSelector.ShowZonePicker(scopeSelector)
	local checkList = {}
	for _, name in ipairs(ScopeSelector.textList) do
		if scopeSelector.scope.zone and scopeSelector.scope.zone[name] then
			table.insert(checkList, true)
		else
			table.insert(checkList, false)
		end
	end
	
	TextPicker.Show(scopeSelector.txtPicker, "Choose Zones", ScopeSelector.textList, function(a, outCheckList) ScopeSelector.ZonesChosen(scopeSelector, outCheckList) end, checkList)
end

function ScopeSelector.SetScope(scopeSelector, scope, playerName)
	scopeSelector.inCheckedCallback = true
	
	if playerName then
		scopeSelector.playerFrame:SetText(playerName)
	end
	
	ScopeSelector.InternalSetScope(scopeSelector, scope)
	
	scopeSelector.inCheckedCallback = false
end

function ScopeSelector.InternalSetScope(scopeSelector, scope)
	scopeSelector.scope = scope
	if scope.all == true then
		ScopeSelector.SetCheckboxs(scopeSelector, 2, 1, true)
		ScopeSelector.SetCheckboxs(scopeSelector, 4, 4, false)
		ScopeSelector.SetCheckboxs(scopeSelector, 6, ScopeSelector.MaxRoles, false)
		ScopeSelector.SetCheckboxs(scopeSelector, 8, 2, false)
		ScopeSelector.SetCheckboxs(scopeSelector, 10, 1, true)
		scopeSelector.zoneFrame:SetText("")
	else
		ScopeSelector.SetCheckboxs(scopeSelector, 2, 1, false)
		ScopeSelector.SetCheckboxs(scopeSelector, 4, 4, false)
		if scope.isSolo and scope.isSolo == true then
			ScopeSelector.SetCheckbox(scopeSelector, 4, 1, true)
		end

		if scope.inParty and scope.inParty == true then
			ScopeSelector.SetCheckbox(scopeSelector, 4, 2, true)
		end

		if scope.inRaid and scope.inRaid == true  then
			ScopeSelector.SetCheckbox(scopeSelector, 4, 3, true)
		end

		if scope.inCrossShardRaid and scope.inCrossShardRaid == true then
			ScopeSelector.SetCheckbox(scopeSelector, 4, 4, true)
		end

		if not scope.roles or scope.roles == "0" then
			ScopeSelector.SetCheckboxs(scopeSelector, 6, ScopeSelector.MaxRoles, true)
		else
			ScopeSelector.SetCheckboxs(scopeSelector, 6, ScopeSelector.MaxRoles, false)
			for role = 1, ScopeSelector.MaxRoles do
				if string.find(scope.roles, tostring(role)) then
					ScopeSelector.SetCheckbox(scopeSelector, 6, role, true)
				end
			end
		end

		if scope.inCombat ~= nil then
			if scope.inCombat == true then
				ScopeSelector.SetCheckbox(scopeSelector, 8, 1, true)
				ScopeSelector.SetCheckbox(scopeSelector, 8, 2, false)
			else
				ScopeSelector.SetCheckbox(scopeSelector, 8, 1, false)
				ScopeSelector.SetCheckbox(scopeSelector, 8, 2, true)
			end
		else
			ScopeSelector.SetCheckboxs(scopeSelector, 8, 2, true)
		end

		if scope.player then
			ScopeSelector.SetCheckbox(scopeSelector, 10, 1, false)
			scopeSelector.playerFrame:SetText(scope.player)
		else
			ScopeSelector.SetCheckbox(scopeSelector, 10, 1, true)
			scopeSelector.playerFrame:SetText("")
		end
		
		local zoneString = ScopeSelector.GetZoneString(scope.zone)
		scopeSelector.zoneFrame:SetText(zoneString)
	end
	
	ScopeSelector.PopulateDescription(scopeSelector)
end

function ScopeSelector.GetZoneString(zone)
	local zoneString = ""
	if zone then
		local firstTime = true
		for name, _ in pairs(zone) do
			if firstTime == false then
				zoneString = zoneString .. ", "
			end

			zoneString = zoneString .. name
			firstTime = false
		end
	end
	
	return zoneString
end

function ScopeSelector.PopulateDescription(scopeSelector)
	local desc = ""
	local scope = scopeSelector.scope
	if scope.all == true then
		desc = "Event will fire everytime it occurs"
	else
		desc = "Event will fire when it occurs and the following conditions are met: \n"
		desc = desc .. "  Player is"
		local orNeeded = false
		if scope.isSolo and scope.isSolo == true then
			desc = desc .. " is solo"
			orNeeded = true
		end

		if scope.inParty and scope.inParty == true then
			desc = desc .. " in a party"
			orNeeded = true
		end

		if scope.inRaid and scope.inRaid == true  then
			if orNeeded then
				desc = desc .. " or"
			end
			desc = desc .. " in a raid"
			orNeeded = true
		end

		if scope.inCrossShardRaid and scope.inCrossShardRaid == true then
			if orNeeded then
				desc = desc .. " or"
			end
			desc = desc .. " in a wf"
			orNeeded = true
		end

		desc = desc .. "\n"
		
		if not scope.roles or scope.roles == "0" then
			desc = desc .. "  Player is in any role\n"
		else
			desc = desc .. "  Player is in roles: " .. scope.roles .. "\n"
		end

		if scope.inCombat then
			if scope.inCombat == true then
				desc = desc .. "  Player is in combat\n"
			else
				desc = desc .. "  Player is out of combat\n"
			end
		else
			desc = desc .. "  Player is either in or out of combat\n"
		end

		if scope.player then
			desc = desc .. "  Player name is " .. scope.player .. "\n"
		else
			desc = desc .. "  Player is any character you have on this shard\n"
		end

		local zoneString = ScopeSelector.GetZoneString(scope.zone)
		if zoneString and string.len(zoneString) > 0 then
			desc = desc .. "  Player is in " .. zoneString .. "\n"
		end
	end
	
	scopeSelector.descFrame:SetText(desc)
end

function ScopeSelector.ScopeChanged(scopeSelector)
	if scopeSelector.inCheckedCallback == true then
		return
	end
	
	local scope = {}
	local alwaysChecked = RowLayout.GetCellFrame(scopeSelector.rowLayout, 2, 1):GetChecked()
	if alwaysChecked then
		scope.all = true
	else
		if RowLayout.GetCellFrame(scopeSelector.rowLayout, 4, 1):GetChecked() == true then
			scope.isSolo = true
		end

		if RowLayout.GetCellFrame(scopeSelector.rowLayout, 4, 2):GetChecked() == true then
			scope.inParty = true
		end

		if RowLayout.GetCellFrame(scopeSelector.rowLayout, 4, 3):GetChecked() == true then
			scope.inRaid = true
		end

		if RowLayout.GetCellFrame(scopeSelector.rowLayout, 4, 4):GetChecked() == true then
			scope.inCrossShardRaid = true
		end

		local roles = ""
		for role = 1, ScopeSelector.MaxRoles do
			if RowLayout.GetCellFrame(scopeSelector.rowLayout, 6, role):GetChecked() == true then
				roles = roles .. role
			end
		end
		if roles ~= "" then
			scope.roles = roles
		end

		local inCombatValue = RowLayout.GetCellFrame(scopeSelector.rowLayout, 8, 1):GetChecked()
		local outCombatValue = RowLayout.GetCellFrame(scopeSelector.rowLayout, 8, 2):GetChecked()

		if inCombatValue == true and outCombatValue == true then
			scope.inCombat = nil
		elseif inCombatValue == true then
			scope.inCombat = true
		elseif outCombatValue == true then
			scope.inCombat = false
		end

		if RowLayout.GetCellFrame(scopeSelector.rowLayout, 10, 1):GetChecked() == false then
			local playerName = scopeSelector.playerFrame:GetText()
			scope.player = playerName
		end
		
		local zones = Utilities.Split(scopeSelector.zoneFrame:GetText(), ',')
		scope.zone = {}
		for _, zoneName in ipairs(zones) do
			local trimZoneName = Utilities.Trim(zoneName)
			if trimZoneName ~= "" then
				scope.zone[Utilities.Trim(zoneName)] = 1
			end
		end
	end
	
	ScopeSelector.InternalSetScope(scopeSelector, scope)
end

function ScopeSelector.Hide(scopeSelector)
	scopeSelector.playerFrame:SetKeyFocus(false)
end
