-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

ScreenFlashWarning = {}
ScreenFlashWarning.name = "ScreenFlashWarning"
ScreenFlashWarning.version = "v0.1"
ScreenFlashWarning.CountdownTemplate = "{Countdown}"

--
-- Main functions
--

function ScreenFlashWarning.Create(name, parentFrame, screenPct, duration, alpha, colour, flashFrequency, flashAlpha, flashColour)
	local scrnFlashWarn = {}
	scrnFlashWarn.name = name
	scrnFlashWarn.screenPct = screenPct
	scrnFlashWarn.fontSize = fontSize
	scrnFlashWarn.duration = duration
	scrnFlashWarn.alpha = alpha
	scrnFlashWarn.colour = colour
	scrnFlashWarn.flashFrequency = flashFrequency
	scrnFlashWarn.flashAlpha = flashAlpha
	scrnFlashWarn.flashColour = flashColour
	scrnFlashWarn.isFlashing = false
	scrnFlashWarn.isCountdown = false
	scrnFlashWarn.flashTimer = nil
	scrnFlashWarn.hideTimer = nil
	scrnFlashWarn.delayTimer = nil

	scrnFlashWarn.frame = UI.CreateFrame("Frame", "ScreenFlashWarning", parentFrame)
	scrnFlashWarn.frame:SetVisible(false)
	scrnFlashWarn.frame:SetAllPoints(UIParent)

	if scrnFlashWarn.screenPct ~= 100 then
		local totalHeight = UIParent:GetHeight()
		local totalWidth = UIParent:GetWidth()
		local height1 = math.floor(totalHeight * (screenPct / 200))
		local height2 = totalHeight - height1
		local width1 = math.floor(totalWidth * (screenPct / 200))
		local width2 = totalWidth - width1
		scrnFlashWarn.frame1 = UI.CreateFrame("Frame", "ScreenFlashWarning1", scrnFlashWarn.frame)
		scrnFlashWarn.frame1:SetPoint("TOPLEFT", scrnFlashWarn.frame, "TOPLEFT")
		scrnFlashWarn.frame1:SetPoint("BOTTOMRIGHT", scrnFlashWarn.frame, "TOPRIGHT", 0, height1)
		scrnFlashWarn.frame2 = UI.CreateFrame("Frame", "ScreenFlashWarning2", scrnFlashWarn.frame)
		scrnFlashWarn.frame2:SetPoint("TOPLEFT", scrnFlashWarn.frame1, "BOTTOMLEFT")
		scrnFlashWarn.frame2:SetPoint("BOTTOMRIGHT", scrnFlashWarn.frame, "TOPLEFT", width1, height2)
		scrnFlashWarn.frame3 = UI.CreateFrame("Frame", "ScreenFlashWarning3", scrnFlashWarn.frame)
		scrnFlashWarn.frame3:SetPoint("BOTTOMLEFT", scrnFlashWarn.frame, "TOPLEFT", width2, height2)
		scrnFlashWarn.frame3:SetPoint("TOPRIGHT", scrnFlashWarn.frame1, "BOTTOMRIGHT")
		scrnFlashWarn.frame4 = UI.CreateFrame("Frame", "ScreenFlashWarning4", scrnFlashWarn.frame)
		scrnFlashWarn.frame4:SetPoint("TOPLEFT", scrnFlashWarn.frame, "TOPLEFT", 0, height2)
		scrnFlashWarn.frame4:SetPoint("BOTTOMRIGHT", scrnFlashWarn.frame, "BOTTOMRIGHT")
	end
	
	return scrnFlashWarn
end

function ScreenFlashWarning.Show(scrnFlashWarn)
	if scrnFlashWarn.duration > 0 then
		if scrnFlashWarn.hideTimer then
			Timer.RemoveCallback(scrnFlashWarn.hideTimer)
			scrnFlashWarn.hideTimer = nil
		end
		
		scrnFlashWarn.hideTimer = Timer.AddCallback("ScreenFlashWarnHide-" .. scrnFlashWarn.name, scrnFlashWarn.duration, function() ScreenFlashWarning.Hide(scrnFlashWarn) end)
	end

	if scrnFlashWarn.flashFrequency > 0 then
		if scrnFlashWarn.flashTimer then
			Timer.RemoveCallback(scrnFlashWarn.flashTimer)
			scrnFlashWarn.flashTimer = nil
		end
		
		scrnFlashWarn.flashTimer = Timer.AddCallback("ScreenFlashWarnFlash-" .. scrnFlashWarn.name, scrnFlashWarn.flashFrequency, function() ScreenFlashWarning.FlashCallback(scrnFlashWarn) end, true)
	end
	
	ScreenFlashWarning.SetColour(scrnFlashWarn, scrnFlashWarn.colour.red, scrnFlashWarn.colour.green, scrnFlashWarn.colour.blue, scrnFlashWarn.alpha)
	scrnFlashWarn.frame:SetVisible(true)
end

function ScreenFlashWarning.Hide(scrnFlashWarn)
	if scrnFlashWarn.flashTimer then
		Timer.RemoveCallback(scrnFlashWarn.flashTimer)
		scrnFlashWarn.flashTimer = nil
	end
	
	scrnFlashWarn.frame:SetVisible(false)
end

function ScreenFlashWarning.SetVisible(scrnFlashWarn, visible)
	if visible == true then
		ScreenFlashWarning.Show(scrnFlashWarn)
	else
		ScreenFlashWarning.Hide(scrnFlashWarn)
	end
end

function ScreenFlashWarning.SetColour(scrnFlashWarn, red, green, blue, alpha)
	if scrnFlashWarn.frame1 then
		scrnFlashWarn.frame1:SetBackgroundColor(red, green, blue, alpha)
		scrnFlashWarn.frame2:SetBackgroundColor(red, green, blue, alpha)
		scrnFlashWarn.frame3:SetBackgroundColor(red, green, blue, alpha)
		scrnFlashWarn.frame4:SetBackgroundColor(red, green, blue, alpha)
	else
		scrnFlashWarn.frame:SetBackgroundColor(red, green, blue, alpha)
	end
end

--
-- Event Handlers
--

function ScreenFlashWarning.FlashCallback(scrnFlashWarn)
	if scrnFlashWarn.isFlashing then
		scrnFlashWarn.isFlashing = false
		if scrnFlashWarn.alpha == 0 then
			scrnFlashWarn.frame:SetVisible(false)
		else
			scrnFlashWarn.frame:SetVisible(true)
		end
		ScreenFlashWarning.SetColour(scrnFlashWarn, scrnFlashWarn.colour.red, scrnFlashWarn.colour.green, scrnFlashWarn.colour.blue, scrnFlashWarn.alpha)
	else
		scrnFlashWarn.isFlashing = true
		if scrnFlashWarn.flashAlpha == 0 then
			scrnFlashWarn.frame:SetVisible(false)
		else
			scrnFlashWarn.frame:SetVisible(true)
		end
		ScreenFlashWarning.SetColour(scrnFlashWarn, scrnFlashWarn.flashColour.red, scrnFlashWarn.flashColour.green, scrnFlashWarn.flashColour.blue, scrnFlashWarn.flashAlpha)
	end
end

--
-- Register events
--

