-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

StatEventConfig = {}
StatEventConfig.name = "StatEventConfig"
StatEventConfig.version = "$Id$"
StatEventConfig.TargetAndFocus = "player.target or focus"
StatEventConfig.StatLE = "Stat % is less than or equal"
StatEventConfig.StatGE = "Stat % is greater than or equal"
StatEventConfig.StatRange = "Stat % is greater than low% and less than high%"
StatEventConfig.StatTypeHealth = "Health"
StatEventConfig.StatTypeMana = "Mana"
StatEventConfig.StatTypeEnergy = "Energy"
StatEventConfig.StatTypeCharge = "Charge"

--
-- Main functions
--

--
-- UI functions
--

function StatEventConfig.Create(wmeConfig, parentFrame, callback)
	local evtStatConfig = {}
	evtStatConfig.wmeConfig = wmeConfig
	evtStatConfig.parentFrame = parentFrame
	evtStatConfig.callback = callback
	evtStatConfig.event = StatEventConfig.GetDefaultEvent()
	evtStatConfig.txtPicker = wmeConfig.txtPicker
	
	evtStatConfig.windowFrame = UI.CreateFrame("RiftWindow", "StatEventSelector", parentFrame)
	evtStatConfig.windowFrame:SetTitle("Stat Event Editor")
	evtStatConfig.windowFrame:SetWidth(700)
	evtStatConfig.windowFrame:SetHeight(850)
	evtStatConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	local xMargin = 20
	local yMargin = 20
	evtStatConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", evtStatConfig.windowFrame)
	evtStatConfig.okButton:SetPoint("BOTTOMLEFT", evtStatConfig.windowFrame, "BOTTOMLEFT", xMargin, - yMargin)
	evtStatConfig.okButton:SetText("OK")
	evtStatConfig.okButton.Event.LeftPress = function() StatEventConfig.WindowClose(evtStatConfig, true) end	

	evtStatConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", evtStatConfig.windowFrame)
	evtStatConfig.cancelButton:SetPoint("BOTTOMRIGHT", evtStatConfig.windowFrame, "BOTTOMRIGHT", - xMargin, - yMargin)
	evtStatConfig.cancelButton:SetText("Cancel")
	evtStatConfig.cancelButton.Event.LeftPress = function() StatEventConfig.WindowClose(evtStatConfig, false) end	

	local margin = 4
	local height = 40
	evtStatConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", evtStatConfig.windowFrame)
	evtStatConfig.gridFrame:SetPoint("TOPLEFT", evtStatConfig.windowFrame, "TOPLEFT", 40, 80)
	evtStatConfig.rowLayout = RowLayout.Create(evtStatConfig.gridFrame, 8, 3, height, 400, margin, margin)
	RowLayout.SetColWidth(evtStatConfig.rowLayout, 1, 80)
	
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the event must be unique")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	evtStatConfig.warningFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 2, 2, "Text")
	evtStatConfig.warningFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Stat Type")
	evtStatConfig.statTypeFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 3, 2, "Text")
	evtStatConfig.statTypeFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Target")
	evtStatConfig.targetFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 4, 2, "Text")
	evtStatConfig.targetFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Target Name")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 5, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the Npc, only for named targets")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Event Type")
	evtStatConfig.operatorFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 6, 2, "Text")
	evtStatConfig.operatorFrame:SetText("")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("Stat%")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 7, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The % of target stat to trigger the event")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 8, 1, "Text")
	txtFrame:SetText("High Stat%")
	local txtFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 8, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The high % of target stat range")

	evtStatConfig.nameFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 1, 2, "RiftTextfield")
	evtStatConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	evtStatConfig.targetNameFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 5, 2, "RiftTextfield")
	evtStatConfig.targetNameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	evtStatConfig.warningButton = RowLayout.CreateFrame(evtStatConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	evtStatConfig.warningButton:SetText("Choose")
	evtStatConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(evtStatConfig, WarnMe.warningTypeAll, function(a, warningName) StatEventConfig.ChooseWarningCallback(evtStatConfig, warningName) end) end
	evtStatConfig.statTypeButton = RowLayout.CreateFrame(evtStatConfig.rowLayout, 3, 3, "RiftButton", RowLayout.centerFrame)
	evtStatConfig.statTypeButton:SetText("Choose")
	evtStatConfig.statTypeButton.Event.LeftPress = function() StatEventConfig.ChooseStatType(evtStatConfig, function(a, statType) StatEventConfig.ChooseStatTypeCallback(evtStatConfig, statType) end) end
	evtStatConfig.targetButton = RowLayout.CreateFrame(evtStatConfig.rowLayout, 4, 3, "RiftButton", RowLayout.centerFrame)
	evtStatConfig.targetButton:SetText("Choose")
	evtStatConfig.targetButton.Event.LeftPress = function() StatEventConfig.ChooseUnitSpec(evtStatConfig, function(a, unitSpecName) StatEventConfig.ChooseUnitSpecCallback(evtStatConfig, unitSpecName) end) end
	evtStatConfig.operatorButton = RowLayout.CreateFrame(evtStatConfig.rowLayout, 6, 3, "RiftButton", RowLayout.centerFrame)
	evtStatConfig.operatorButton:SetText("Choose")
	evtStatConfig.operatorButton.Event.LeftPress = function() StatEventConfig.ChooseEventType(evtStatConfig, function(a, eventType) StatEventConfig.ChooseEventTypeCallback(evtStatConfig, eventType) end) end
	RowLayout.SetColWidth(evtStatConfig.rowLayout, 3, evtStatConfig.warningButton:GetWidth())

	local xyFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 7, 2, "Frame")
	evtStatConfig.statSlider = NumberSlider.Create(xyFrame, 0, 100, 1, 0)
	local xyFrame = RowLayout.CreateFrame(evtStatConfig.rowLayout, 8, 2, "Frame")
	evtStatConfig.statSlider2 = NumberSlider.Create(xyFrame, 0, 100, 1, 0)
	RowLayout.SetRowVisible(evtStatConfig.rowLayout, 8, false)

	RowLayout.Redraw(evtStatConfig.rowLayout)

	evtStatConfig.scopeFrame = UI.CreateFrame("Frame", "ScopeFrame", evtStatConfig.windowFrame)
	evtStatConfig.scopeFrame:SetPoint("TOPLEFT", RowLayout.GetFrame(evtStatConfig.rowLayout), "BOTTOMLEFT", - xMargin, 0)
	evtStatConfig.scopeFrame:SetPoint("BOTTOMRIGHT", evtStatConfig.cancelButton, "TOPRIGHT", 0, - yMargin)
	evtStatConfig.scopeWindow = ScopeSelector.Create(evtStatConfig.scopeFrame, xMargin, yMargin, evtStatConfig.event.scope)

	evtStatConfig.wrnWindow = WarningWindow.CreateWarningWindow(evtStatConfig.parentFrame, evtStatConfig.windowFrame)
	
	StatEventConfig.Show(evtStatConfig)
	evtStatConfig.windowFrame:SetVisible(false)
	
	return evtStatConfig
end

function StatEventConfig.WindowClose(evtStatConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = evtStatConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this event\n"
		end
		local warn = evtStatConfig.warningFrame:GetText()
		if not warn or string.len(warn) < 1 then
			message = message .. "You must specify a warning for this event\n"
		end
		local target = evtStatConfig.targetFrame:GetText()
		if not target or string.len(target) < 1 then
			message = message .. "You must specify a target for this event\n"
		else
			if target and not (target ~= "player" or target ~= "player.pet") then
				local targetName = evtStatConfig.targetNameFrame:GetText()
				if not targetName or string.len(targetName) < 1 then
					message = message .. "You must specify an npc name for this event\n"
				end
			end
		end
				
		if string.len(message) > 0 then
			WarningWindow.ShowWarningWindow(evtStatConfig.wrnWindow, message, function() end)
			return
		end
		
		StatEventConfig.FillEvent(evtStatConfig)
	end
	
	evtStatConfig.nameFrame:SetKeyFocus(false)
	evtStatConfig.targetNameFrame:SetKeyFocus(false)
	ScopeSelector.Hide(evtStatConfig.scopeWindow)
	
	evtStatConfig.windowFrame:SetVisible(false)
	if evtStatConfig.callback then
		if okPressed == true then
			evtStatConfig.callback(evtStatConfig, evtStatConfig.event)
		else
			evtStatConfig.callback(evtStatConfig)
		end
	end
end

function StatEventConfig.Show(evtStatConfig, inEvent)
	if inEvent then
		evtStatConfig.event = StatEventConfig.CopyEvent(inEvent)
	end
	
	local event = evtStatConfig.event
	evtStatConfig.nameFrame:SetText(event.name)
	evtStatConfig.warningFrame:SetText(event.warningName)
	evtStatConfig.targetFrame:SetText("")
	evtStatConfig.targetNameFrame:SetText("")
	RowLayout.SetRowVisible(evtStatConfig.rowLayout, 5, false)

	if event.stat.unitSpecs then
		if #event.stat.unitSpecs > 1 then
			evtStatConfig.targetFrame:SetText(StatEventConfig.TargetAndFocus)
		elseif #event.stat.unitSpecs == 1 then
			evtStatConfig.targetFrame:SetText(event.stat.unitSpecs[1])
			if event.stat.unitSpecs[1] == "named" then
				if evtStatConfig.event.stat.targetName then
					evtStatConfig.targetNameFrame:SetText(evtStatConfig.event.stat.targetName)
				end
				RowLayout.SetRowVisible(evtStatConfig.rowLayout, 5, true)
			end
		end
	end

	evtStatConfig.statTypeFrame:SetText(StatEventConfig.GetStringFromStatType(event.statType))
	NumberSlider.SetValue(evtStatConfig.statSlider, evtStatConfig.event.stat.pct)
	
	if event.stat.pct2 then
		evtStatConfig.operatorFrame:SetText(StatEventConfig.StatRange)
		RowLayout.SetRowVisible(evtStatConfig.rowLayout, 8, true)
		NumberSlider.SetValue(evtStatConfig.statSlider2, evtStatConfig.event.stat.pct2)
	else
		RowLayout.SetRowVisible(evtStatConfig.rowLayout, 8, false)
		if event.stat.isLess == true then
			evtStatConfig.operatorFrame:SetText(StatEventConfig.StatLE)
		else
			evtStatConfig.operatorFrame:SetText(StatEventConfig.StatGE)
		end
	end
	
	RowLayout.Redraw(evtStatConfig.rowLayout)
	ScopeSelector.SetScope(evtStatConfig.scopeWindow, event.scope, event.scope.player)
	evtStatConfig.windowFrame:SetVisible(true)
end

function StatEventConfig.FillEvent(evtStatConfig)
	evtStatConfig.event.name = evtStatConfig.nameFrame:GetText()
	evtStatConfig.event.warningName = evtStatConfig.warningFrame:GetText()
	evtStatConfig.event.stat.targetName = nil
	evtStatConfig.event.stat.pct = NumberSlider.GetValue(evtStatConfig.statSlider)
	evtStatConfig.event.scope = WarnMe.CopyScope(evtStatConfig.scopeWindow.scope)
	if evtStatConfig.event.stat.pct2 then
		evtStatConfig.event.stat.pct2 = NumberSlider.GetValue(evtStatConfig.statSlider2)
	end
	
	if evtStatConfig.statTypeFrame:GetText() ~= "" then
		evtStatConfig.event.statType = StatEventConfig.GetStatTypeFromString(evtStatConfig.statTypeFrame:GetText())
	end
	
	local frameText = tostring(evtStatConfig.targetFrame:GetText())
	evtStatConfig.event.stat.unitSpecs = {}
	if frameText == StatEventConfig.TargetAndFocus then
		table.insert(evtStatConfig.event.stat.unitSpecs, "player.target")
		table.insert(evtStatConfig.event.stat.unitSpecs, "focus")
	elseif frameText == "named" then
		table.insert(evtStatConfig.event.stat.unitSpecs, frameText)
		evtStatConfig.event.stat.targetName = tostring(evtStatConfig.targetNameFrame:GetText())
	elseif frameText ~= "" then
		table.insert(evtStatConfig.event.stat.unitSpecs, frameText)
	end
end

function StatEventConfig.ChooseWarningCallback(evtStatConfig, warningName)
	if warningName and string.len(warningName) > 0 then
		StatEventConfig.FillEvent(evtStatConfig)
		evtStatConfig.event.warningName = warningName
		StatEventConfig.Show(evtStatConfig)
	end
	
	evtStatConfig.windowFrame:SetVisible(true)
end

function StatEventConfig.ChooseStatTypeCallback(evtStatConfig, statType)
	if statType and string.len(statType) > 0 then
		StatEventConfig.FillEvent(evtStatConfig)
		evtStatConfig.event.statType = StatEventConfig.GetStatTypeFromString(statType)
		StatEventConfig.Show(evtStatConfig)
	end
	
	evtStatConfig.windowFrame:SetVisible(true)
end

function StatEventConfig.GetStatTypeFromString(statType)
	if statType == StatEventConfig.StatTypeHealth then
		return WarnMe.statTypeHealth
	elseif statType == StatEventConfig.StatTypeMana then
		return WarnMe.statTypeMana
	elseif statType == StatEventConfig.StatTypeEnergy then
		return WarnMe.statTypeEnergy
	elseif statType == StatEventConfig.StatTypeCharge then
		return WarnMe.statTypeCharge
	end
end

function StatEventConfig.GetStringFromStatType(statType)
	if statType == WarnMe.statTypeHealth then
		return StatEventConfig.StatTypeHealth
	elseif statType == WarnMe.statTypeMana then
		return StatEventConfig.StatTypeMana
	elseif statType == WarnMe.statTypeEnergy then
		return StatEventConfig.StatTypeEnergy
	elseif statType == WarnMe.statTypeCharge then
		return StatEventConfig.StatTypeCharge
	end
	
	return ""
end

function StatEventConfig.ChooseStatType(wmeConfig, callback)
	local textList = {}
	table.insert(textList, StatEventConfig.StatTypeHealth)
	table.insert(textList, StatEventConfig.StatTypeMana)
	table.insert(textList, StatEventConfig.StatTypeEnergy)
	table.insert(textList, StatEventConfig.StatTypeCharge)
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Stat Type", textList, callback)
end

function StatEventConfig.ChooseUnitSpecCallback(evtStatConfig, unitSpec)
	if unitSpec and string.len(unitSpec) > 0 then
		StatEventConfig.FillEvent(evtStatConfig)
		if unitSpec == StatEventConfig.TargetAndFocus then
			evtStatConfig.event.stat.unitSpecs = {}
			table.insert(evtStatConfig.event.stat.unitSpecs, "player.target")
			table.insert(evtStatConfig.event.stat.unitSpecs, "focus")
		else
			evtStatConfig.event.stat.unitSpecs = {}
			table.insert(evtStatConfig.event.stat.unitSpecs, unitSpec)
		end
		StatEventConfig.Show(evtStatConfig)
	end
	
	evtStatConfig.windowFrame:SetVisible(true)
end

function StatEventConfig.ChooseUnitSpec(wmeConfig, callback)
	local textList = {}
	table.insert(textList, "player")
	table.insert(textList, "player.pet")
	table.insert(textList, "player.target")
	table.insert(textList, "focus")
	table.insert(textList, StatEventConfig.TargetAndFocus)
	table.insert(textList, "named")
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Unit", textList, callback)
end

function StatEventConfig.ChooseEventTypeCallback(evtStatConfig, eventType)
	if eventType and string.len(eventType) > 0 then
		if eventType == StatEventConfig.StatRange then
			evtStatConfig.event.stat.isLess = nil
			evtStatConfig.event.stat.pct2 = 0
		else
			if eventType == StatEventConfig.StatLE then
				evtStatConfig.event.stat.isLess = true
			else
				evtStatConfig.event.stat.isLess = false
			end
		end
		StatEventConfig.FillEvent(evtStatConfig)
		StatEventConfig.Show(evtStatConfig)
	end
	
	evtStatConfig.windowFrame:SetVisible(true)
end

function StatEventConfig.ChooseEventType(wmeConfig, callback)
	local textList = {}
	table.insert(textList, StatEventConfig.StatLE)
	table.insert(textList, StatEventConfig.StatGE)
	table.insert(textList, StatEventConfig.StatRange)
	
	wmeConfig.windowFrame:SetVisible(false)
	TextPicker.Show(wmeConfig.txtPicker, "Choose Event Type", textList, callback)
end

function StatEventConfig.CopyEvent(event2)
	local event = {}
	event.eventType = WarnMe.eventTypeStat
	event.statType = event2.statType
	event.name = event2.name
	event.warningName = event2.warningName
	event.scope = WarnMe.CopyScope(event2.scope)
	event.stat = {}
	event.stat.targetName = event2.stat.targetName
	event.stat.pct = event2.stat.pct
	if event2.stat.pct2 then
		event.stat.pct2 = event2.stat.pct2
	else
		event.stat.isLess = event2.stat.isLess
	end
	event.stat.unitSpecs = {}
	for _, spec in ipairs(event2.stat.unitSpecs) do
		table.insert(event.stat.unitSpecs, spec)
	end
	
	return event
end

function StatEventConfig.GetDefaultEvent()
	local event = {}
	event.eventType = WarnMe.eventTypeStat
	event.statType = WarnMe.statTypeHealth
	event.name = ""
	event.warningName = ""
	event.scope = {}
	event.scope.inRaid = true
	event.scope.inCombat = true
	event.stat = {}
	event.stat.targetName = ""
	event.stat.isLess = true
	event.stat.pct = 0
	event.stat.unitSpecs = {}
	
	return event
end

