-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextPicker = {}
TextPicker.name = "TextPicker"
TextPicker.version = "v0.1"

--
-- Main functions
--

--
-- UI functions
--

function TextPicker.Create(parentFrame, multiSelect)
	local txtPicker = {}
	txtPicker.parentFrame = parentFrame
	txtPicker.multiSelect = multiSelect
	txtPicker.page = 1
	txtPicker.maxPage = 1
	txtPicker.textList = {}
	
	txtPicker.textFrame = UI.CreateFrame("RiftWindow", "TextFrame", WarnMe.parentFrame)
	txtPicker.textFrame:SetPoint("CENTER", parentFrame, "CENTER")
	txtPicker.textFrame:SetWidth(400)
	txtPicker.textFrame:SetHeight(628)
	txtPicker.textFrame:SetVisible(false)
	txtPicker.textFrame:SetLayer(4)

	local xMargin = 20
	local yMargin = 20
	txtPicker.okButton = UI.CreateFrame("RiftButton", "OKButton", txtPicker.textFrame)
	txtPicker.okButton:SetPoint("BOTTOMLEFT", txtPicker.textFrame, "BOTTOMLEFT", xMargin, -yMargin)
	txtPicker.okButton:SetText("OK")
	txtPicker.okButton.Event.LeftPress = function() TextPicker.WindowClose(txtPicker, true) end	

	txtPicker.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtPicker.textFrame)
	txtPicker.cancelButton:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	txtPicker.cancelButton:SetText("Cancel")
	txtPicker.cancelButton.Event.LeftPress = function() TextPicker.WindowClose(txtPicker, false) end	

	local pathMargin = -yMargin
	if txtPicker.multiSelect == true then
		pathMargin = 0
	end

	txtPicker.pathLabel = UI.CreateFrame("Text", "PathLabel", txtPicker.textFrame)
	txtPicker.pathLabel:SetPoint("BOTTOMLEFT", txtPicker.okButton, "TOPLEFT", 0, pathMargin)
	txtPicker.pathLabel:SetText("Value:")

	txtPicker.pathText = UI.CreateFrame("RiftTextfield", "PathText", txtPicker.textFrame)
	txtPicker.pathText:SetPoint("BOTTOMRIGHT", txtPicker.cancelButton, "TOPRIGHT", 0, pathMargin)
	txtPicker.pathText:SetPoint("BOTTOMLEFT", txtPicker.pathLabel, "BOTTOMRIGHT", xMargin, 0)
	txtPicker.pathText:SetPoint("TOPLEFT", txtPicker.pathLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.pathText:SetText("")

	local frameMiddle = math.floor(txtPicker.textFrame:GetWidth() / 2)
	txtPicker.prevButton = UI.CreateFrame("RiftButton", "PrevButton", txtPicker.textFrame)
	txtPicker.prevButton:SetPoint("BOTTOMRIGHT", txtPicker.pathLabel, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)
	txtPicker.prevButton:SetText("<<")
	txtPicker.prevButton.Event.LeftPress = function() TextPicker.PageTurn(txtPicker, -1) end	

	txtPicker.nextButton = UI.CreateFrame("RiftButton", "NextButton", txtPicker.textFrame)
	txtPicker.nextButton:SetPoint("BOTTOMLEFT", txtPicker.pathText, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.nextButton:SetText(">>")
	txtPicker.nextButton.Event.LeftPress = function() TextPicker.PageTurn(txtPicker, 1) end
	
	txtPicker.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtPicker.textFrame)
	txtPicker.gridFrame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT", xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("TOPRIGHT", txtPicker.textFrame, "TOPRIGHT", -xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("BOTTOMLEFT", txtPicker.prevButton, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.gridFrame:SetPoint("BOTTOMRIGHT", txtPicker.nextButton, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)

	local margin = 4
	local textSize = txtPicker.pathLabel:GetHeight()
	local rows = math.floor((txtPicker.gridFrame:GetHeight() + margin) / (textSize + margin))
	local cols = 1
	local colWidth = txtPicker.textFrame:GetWidth() - (2 * xMargin)
	if txtPicker.multiSelect == true then
		cols = 2
		colWidth = colWidth - 32 - margin
	end
	txtPicker.rowLayout = RowLayout.Create(txtPicker.gridFrame, rows, cols, textSize, colWidth, margin, margin)
	for row = 1, rows do
		local txtFrame = RowLayout.CreateFrame(txtPicker.rowLayout, row, cols, "Text")
		if row % 2 == 1 then
			txtFrame:SetBackgroundColor(0, 0, 0.15)
		else
			txtFrame:SetBackgroundColor(0.15, 0.15, 0.15)
		end
		
		if txtPicker.multiSelect == true then
			local checkFrame = RowLayout.CreateFrame(txtPicker.rowLayout, row, 1, "RiftCheckbox", RowLayout.centerFrame)
		end
	end

	if txtPicker.multiSelect == true then
		RowLayout.SetColWidth(txtPicker.rowLayout, 1, 32)
		txtPicker.pathLabel:SetVisible(false)
		txtPicker.pathText:SetVisible(false)
		txtPicker.pathLabel:SetVisible(false)
		txtPicker.pathText:SetVisible(false)
	end
	
	TextPicker.Redraw(txtPicker)
	RowLayout.Redraw(txtPicker.rowLayout)

	return txtPicker
end

function TextPicker.Show(txtPicker, title, textList, callback, checkList)
	txtPicker.callback = callback
	txtPicker.textList = textList
	txtPicker.textFrame:SetTitle(title)
	if txtPicker.multiSelect == true then
		if checkList and #checkList == #textList then
			txtPicker.checkList = checkList
		else
			txtPicker.checkList = {}
			for i = 1, #textList do
				table.insert(txtPicker.checkList, false)
			end
		end
	end
	txtPicker.page = 1
	txtPicker.maxPage = math.ceil(#textList / RowLayout.GetNumRows(txtPicker.rowLayout))
	txtPicker.pathText:SetText("")
	TextPicker.Redraw(txtPicker)
	txtPicker.textFrame:SetVisible(true)
end

function TextPicker.WindowClose(txtPicker, okPressed)
	txtPicker.pathText:SetKeyFocus(false)
	txtPicker.textFrame:SetVisible(false)
	
	if okPressed then
		if txtPicker.multiSelect == true then
			txtPicker.callback(txtPicker, txtPicker.checkList)
		else
			local text = txtPicker.pathText:GetText()
			if text and string.len(text) > 0 then
				txtPicker.callback(txtPicker, txtPicker.pathText:GetText())
			else
				txtPicker.callback(txtPicker)
			end
		end		
	else
		txtPicker.callback(txtPicker)
	end
end

function TextPicker.Redraw(txtPicker)
	txtPicker.prevButton:SetEnabled(txtPicker.page ~= 1)
	txtPicker.nextButton:SetEnabled(txtPicker.page ~= txtPicker.maxPage)
	
	local rows = RowLayout.GetNumRows(txtPicker.rowLayout)
	local cols = RowLayout.GetNumCols(txtPicker.rowLayout)
	for row = 1, rows do
		local chkFrame = nil
		if txtPicker.multiSelect == true then
			chkFrame = RowLayout.GetCellFrame(txtPicker.rowLayout, row, 1)	
		end
		
		local txtFrame = RowLayout.GetCellFrame(txtPicker.rowLayout, row, cols)
		local textIndex = (rows * (txtPicker.page - 1)) + row
		if textIndex <= #txtPicker.textList then
			txtFrame:SetText(txtPicker.textList[textIndex])
			txtFrame.Event.LeftClick = function() TextPicker.ChooseText(txtPicker, textIndex) end
			txtFrame:SetVisible(true)
			if txtPicker.multiSelect == true then
				txtFrame.Event.LeftClick = function() TextPicker.TextCheckChanged(txtPicker, textIndex, chkFrame) end
				chkFrame.Event.CheckboxChange = function() end
				chkFrame:SetChecked(txtPicker.checkList[textIndex])
				chkFrame.Event.CheckboxChange = function() TextPicker.CheckChanged(txtPicker, textIndex, chkFrame:GetChecked()) end
				chkFrame:SetVisible(true)
			end
		else
			txtFrame:SetVisible(false)
			txtFrame.Event.LeftClick = function() end
			if txtPicker.multiSelect == true then
				chkFrame:SetVisible(false)
				chkFrame.Event.CheckboxChange = function() end
			end
		end
	end
end

function TextPicker.TextCheckChanged(txtPicker, textIndex, chkFrame)
	chkFrame:SetChecked(not chkFrame:GetChecked())
end

function TextPicker.CheckChanged(txtPicker, textIndex, checked)
	txtPicker.checkList[textIndex] = checked
end

function TextPicker.ChooseText(txtPicker, textIndex)
	txtPicker.pathText:SetText(txtPicker.textList[textIndex])
end

function TextPicker.PageTurn(txtPicker, pageIncrement)
	txtPicker.page = txtPicker.page + pageIncrement
	if txtPicker.page < 1 then
		txtPicker.page = 1
	end
	if txtPicker.page > txtPicker.maxPage then
		txtPicker.page = txtPicker.maxPage
	end
	
	TextPicker.Redraw(txtPicker)
end
