-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextWarning = {}
TextWarning.name = "TextWarning"
TextWarning.version = "v0.1"
TextWarning.CountdownTemplate = "{Countdown}"

--
-- Main functions
--

function TextWarning.Create(name, parentFrame, str, xPct, yPct, fontSize, duration, alpha, colour, flashFrequency, flashAlpha, flashColour)
	local txtWarn = {}
	txtWarn.name = name
	txtWarn.parentFrame = parentFrame
	if not str then
		txtWarn.str = ""
	else
		txtWarn.str = str
	end
	txtWarn.xPct = xPct
	txtWarn.yPct = yPct
	txtWarn.fontSize = fontSize
	txtWarn.duration = duration
	txtWarn.alpha = alpha
	txtWarn.colour = colour
	txtWarn.flashFrequency = flashFrequency
	txtWarn.flashAlpha = flashAlpha
	txtWarn.flashColour = flashColour
	txtWarn.isCountdown = false
	txtWarn.flashTimer = nil
	txtWarn.hideTimer = nil
	txtWarn.delayTimer = nil
	txtWarn.isCountdown = false
	txtWarn.countdownTimer = nil

	txtWarn.frame = UI.CreateFrame("Text", "TextWarning", parentFrame)
	txtWarn.frame:SetVisible(false)
	txtWarn.frame:SetFontColor(txtWarn.colour.red, txtWarn.colour.green, txtWarn.colour.blue, txtWarn.alpha)
	txtWarn.frame:SetFontSize(txtWarn.fontSize)
	TextWarning.SetText(txtWarn, txtWarn.str)
	
	return txtWarn
end

function TextWarning.SetText(txtWarn, newText)
	txtWarn.str = newText
	
	local position = string.find(newText, TextWarning.CountdownTemplate)
	if position and txtWarn.duration > 0 then
		txtWarn.countdownPrefix = string.sub(newText, 1, position - 1)
		txtWarn.countdownSuffix = string.sub(newText, position + string.len(TextWarning.CountdownTemplate))
		txtWarn.isCountdown = true
		txtWarn.countdownValue = txtWarn.duration
	end
	
	TextWarning.RefreshText(txtWarn)
end

function TextWarning.RefreshText(txtWarn)
	if txtWarn.isCountdown == true then
		txtWarn.frame:SetText(txtWarn.countdownPrefix .. txtWarn.countdownValue .. txtWarn.countdownSuffix)
	else
		txtWarn.frame:SetText(txtWarn.str)
	end
	TextWarning.SetPosition(txtWarn)
end

function TextWarning.GetPosition(txtWarn)
	return txtWarn.xPct, txtWarn.yPct
end

function TextWarning.SetPosition(txtWarn)
	local width = txtWarn.frame:GetWidth()
	local height = txtWarn.frame:GetHeight()
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	
	local xPct = txtWarn.xPct
	if xPct < 0 then
		xPct = 0
	end
	if xPct > 100 then
		xPct = 100
	end
	
	local yPct = txtWarn.yPct
	if yPct < 0 then
		yPct = 0
	end
	if yPct > 100 then
		yPct = 100
	end
	
	local x = math.floor(totalWidth * xPct / 100 - width / 2)
	if x < 0 then
		x = 0
	end
	if x + width > totalWidth then
		x = totalWidth - width
	end
	
	local y = math.floor(totalHeight * yPct / 100 - height / 2)
	if y < 0 then
		y = 0
	end
	if y + height > totalHeight then
		y = totalHeight - height
	end
	
	txtWarn.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
end

function TextWarning.Show(txtWarn)
	if txtWarn.duration > 0 then
		if txtWarn.hideTimer then
			Timer.RemoveCallback(txtWarn.hideTimer)
			txtWarn.hideTimer = nil
		end
		
		txtWarn.hideTimer = Timer.AddCallback("TextWarnHide-" .. txtWarn.name, txtWarn.duration, function() TextWarning.Hide(txtWarn) end)
	end

	if txtWarn.flashFrequency > 0 then
		if txtWarn.flashTimer then
			Timer.RemoveCallback(txtWarn.flashTimer)
			txtWarn.flashTimer = nil
		end
		
		local initialFlashDuration = txtWarn.flashFrequency - Timer.GetCurrentTime() % txtWarn.flashFrequency
		if initialFlashDuration <= 0 then
			txtWarn.flashTimer = Timer.AddCallback("TextWarnFlash-" .. txtWarn.name, txtWarn.flashFrequency, function(timeVal) TextWarning.FlashCallback(txtWarn, timeVal) end, true)
		else
			txtWarn.flashTimer = Timer.AddCallback("TextWarnInitFlash-" .. txtWarn.name, initialFlashDuration, function(timeVal) TextWarning.InitialFlashCallback(txtWarn, timeVal) end, false)
		end
	end
	
	if txtWarn.duration > 0 and txtWarn.isCountdown == true then
		if txtWarn.countdownTimer then
			Timer.RemoveCallback(txtWarn.countdownTimer)
			txtWarn.countdownTimer = nil
		end
		
		txtWarn.countdownValue = txtWarn.duration
		TextWarning.RefreshText(txtWarn)
		txtWarn.countdownTimer = Timer.AddCallback("TextWarnCountdown-" .. txtWarn.name, 1, function() TextWarning.CountdownCallback(txtWarn) end, true)
	else
		txtWarn.countdownValue = nil
	end
	
	txtWarn.frame:SetFontColor(txtWarn.colour.red, txtWarn.colour.green, txtWarn.colour.blue, txtWarn.alpha)
	txtWarn.frame:SetVisible(true)
end

function TextWarning.Hide(txtWarn)
	if txtWarn.flashTimer then
		Timer.RemoveCallback(txtWarn.flashTimer)
		txtWarn.flashTimer = nil
	end
	
	if txtWarn.countdownTimer then
		Timer.RemoveCallback(txtWarn.countdownTimer)
		txtWarn.countdownTimer = nil
	end
	
	txtWarn.frame:SetVisible(false)
end

function TextWarning.SetVisible(txtWarn, visible)
	if visible == true then
		TextWarning.Show(txtWarn)
	else
		TextWarning.Hide(txtWarn)
	end
end

function TextWarning.Dragged(txtWarn, xIn, yIn)
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	local widthOffset = txtWarn.frame:GetWidth() / (2 * totalWidth)
	local heightOffset = txtWarn.frame:GetHeight() / (2 * totalHeight)
	local x = Utilities.Round((xIn / UIParent:GetWidth() + widthOffset) * 100, 2)
	local y = Utilities.Round((yIn / UIParent:GetHeight() + heightOffset) * 100, 2)
	if x < 0 then x = 0 end
	if x > 100 then x = 100 end
	if y < 0 then y = 0 end
	if y > 100 then y = 100 end
	txtWarn.xPct = x
	txtWarn.yPct = y
	TextWarning.SetPosition(txtWarn)
end

function TextWarning.SetDragWindowVisible(txtWarn, visible)
	if visible == true then
		if not txtWarn.dragFrame then
			txtWarn.dragFrame = DragFrame.Create(txtWarn.frame, txtWarn.frame:GetWidth(), txtWarn.frame:GetHeight(), function(dragFrame) TextWarning.Dragged(txtWarn, dragFrame.x, dragFrame.y) end, txtWarn.parentFrame.Event.LeftClick)
		end
		txtWarn.frame:SetVisible(true)
	else
		txtWarn.frame:SetVisible(false)
	end
end

--
-- Event Handlers
--

function TextWarning.InitialFlashCallback(txtWarn, timeVal)
	if txtWarn.flashTimer then
		Timer.RemoveCallback(txtWarn.flashTimer)
		txtWarn.flashTimer = nil
	end
	txtWarn.flashTimer = Timer.AddCallback("TextWarnFlash-" .. txtWarn.name, txtWarn.flashFrequency, function(timeVal) TextWarning.FlashCallback(txtWarn, timeVal) end, true)
	
	TextWarning.FlashCallback(txtWarn, timeVal)
end

function TextWarning.FlashCallback(txtWarn, timeVal)
	local isFlashing = math.floor(timeVal / txtWarn.flashFrequency) % 2
	if isFlashing == 0 then
		if txtWarn.alpha == 0 then
			txtWarn.frame:SetVisible(false)
		else
			txtWarn.frame:SetVisible(true)
		end
		txtWarn.frame:SetFontColor(txtWarn.colour.red, txtWarn.colour.green, txtWarn.colour.blue, txtWarn.alpha)
	else
		if txtWarn.flashAlpha == 0 then
			txtWarn.frame:SetVisible(false)
		else
			txtWarn.frame:SetVisible(true)
		end
		txtWarn.frame:SetFontColor(txtWarn.flashColour.red, txtWarn.flashColour.green, txtWarn.flashColour.blue, txtWarn.flashAlpha)
	end
end

function TextWarning.CountdownCallback(txtWarn)
	if txtWarn.countdownValue then
		txtWarn.countdownValue = txtWarn.countdownValue - 1
		TextWarning.RefreshText(txtWarn)
	end
end

--
-- Register events
--

