-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextWarningConfig = {}
TextWarningConfig.name = "TextWarningConfig"
TextWarningConfig.version = "$Id$"

--
-- Main functions
--

--
-- UI functions
--

function TextWarningConfig.Create(parentFrame, callback)
	local txtWarnConfig = {}
	txtWarnConfig.parentFrame = parentFrame
	txtWarnConfig.callback = callback
	txtWarnConfig.warning = TextWarningConfig.GetDefaultWarning()
	
	txtWarnConfig.windowFrame = UI.CreateFrame("RiftWindow", "TxtWarningSelector", parentFrame)
	txtWarnConfig.windowFrame:SetTitle("Text Warning Editor")
	txtWarnConfig.windowFrame:SetWidth(700)
	txtWarnConfig.windowFrame:SetHeight(620)
	txtWarnConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	txtWarnConfig.xySelector = XYSelector.Create(txtWarnConfig.parentFrame, function(x,y) TextWarningConfig.SetXY(txtWarnConfig, x, y) end)
	
	txtWarnConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", txtWarnConfig.windowFrame)
	txtWarnConfig.okButton:SetPoint("BOTTOMLEFT", txtWarnConfig.windowFrame, "BOTTOMLEFT", 20, -20)
	txtWarnConfig.okButton:SetText("OK")
	txtWarnConfig.okButton.Event.LeftPress = function() TextWarningConfig.WindowClose(txtWarnConfig, true) end	

	txtWarnConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtWarnConfig.windowFrame)
	txtWarnConfig.cancelButton:SetPoint("BOTTOMRIGHT", txtWarnConfig.windowFrame, "BOTTOMRIGHT", -20, -20)
	txtWarnConfig.cancelButton:SetText("Cancel")
	txtWarnConfig.cancelButton.Event.LeftPress = function() TextWarningConfig.WindowClose(txtWarnConfig, false) end	

	local margin = 4
	local height = 40
	txtWarnConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtWarnConfig.windowFrame)
	txtWarnConfig.gridFrame:SetPoint("TOPLEFT", txtWarnConfig.windowFrame, "TOPLEFT", 40, 80)
	txtWarnConfig.rowLayout = RowLayout.Create(txtWarnConfig.gridFrame, 11, 3, height, 200, margin, margin)

	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the warning must be unique")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Display String")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The value you will see on the screen")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("x%,y%")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Font Size")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Display duration (secs)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 5, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Duration of 0 means show until event ends")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Alpha Level (0-1)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 6, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Sets the transparency of the warning")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("Text colour")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 8, 1, "Text")
	txtFrame:SetText("Flash frequency (secs)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 8, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("How often the warning flashes, 0 means never")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 9, 1, "Text")
	txtFrame:SetText("Flash Alpha level")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 9, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Set the transparency of the flashed text")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 1, "Text")
	txtFrame:SetText("Flash colour")

	txtWarnConfig.colourFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 7, 2, "Frame")
	txtWarnConfig.flashColourFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 2, "Frame")

	txtWarnConfig.nameFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 2, "RiftTextfield")
	txtWarnConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	txtWarnConfig.strFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 2, "RiftTextfield")
	txtWarnConfig.strFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	local xyFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 2, "Frame")
	local xyFrame1 = UI.CreateFrame("Frame", "xyFrame1", xyFrame)
	xyFrame1:SetPoint("TOPLEFT", xyFrame, "TOPLEFT")
	xyFrame1:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMLEFT", math.floor((RowLayout.GetColWidth(txtWarnConfig.rowLayout, 2) - margin) / 2), 0)	
	txtWarnConfig.xSlider = NumberSlider.Create(xyFrame1, 0, 100, 0.01, 0)
	local xyFrame2 = UI.CreateFrame("Frame", "xyFrame2", xyFrame)
	xyFrame2:SetPoint("TOPLEFT", xyFrame1, "TOPRIGHT", margin, 0)
	xyFrame2:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMRIGHT")
	txtWarnConfig.ySlider = NumberSlider.Create(xyFrame2, 0, 100, 0.01, 0)

	txtWarnConfig.fontSizeSlider = TextWarningConfig.CreateSlider(txtWarnConfig, 4, 2, 10, 1000, 10)
	txtWarnConfig.durationSlider = TextWarningConfig.CreateSlider(txtWarnConfig, 5, 2, 0, 300, 1)
	txtWarnConfig.alphaSlider = TextWarningConfig.CreateSlider(txtWarnConfig, 6, 2, 0, 1, 0.05)
	txtWarnConfig.flashDurationSlider = TextWarningConfig.CreateSlider(txtWarnConfig, 8, 2, 0, 30, 1)
	txtWarnConfig.flashAlphaSlider = TextWarningConfig.CreateSlider(txtWarnConfig, 9, 2, 0, 1, 0.05)
	
	txtWarnConfig.xyButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.xyButton:SetText("Choose")
	txtWarnConfig.xyButton.Event.LeftPress = function() TextWarningConfig.ShowXY(txtWarnConfig) end
	txtWarnConfig.colourButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 7, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.colourButton:SetText("Choose")
	txtWarnConfig.colourButton.Event.LeftPress = function() TextWarningConfig.ShowChooseColour(txtWarnConfig) end
	txtWarnConfig.flashColourButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.flashColourButton:SetText("Choose")
	txtWarnConfig.flashColourButton.Event.LeftPress = function() TextWarningConfig.ShowChooseFlashColour(txtWarnConfig) end
	RowLayout.SetColWidth(txtWarnConfig.rowLayout, 3, txtWarnConfig.flashColourButton:GetWidth())

	RowLayout.Redraw(txtWarnConfig.rowLayout)

	txtWarnConfig.clrChooser = ColourChooser.Create(txtWarnConfig.windowFrame, txtWarnConfig.warning.colour)
	txtWarnConfig.wrnWindow = WarningWindow.CreateWarningWindow(txtWarnConfig.parentFrame, txtWarnConfig.windowFrame)
	
	TextWarningConfig.Show(txtWarnConfig)
	txtWarnConfig.windowFrame:SetVisible(false)
	
	return txtWarnConfig
end

function TextWarningConfig.WindowClose(txtWarnConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = txtWarnConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this warning\n"
		end
		local str = txtWarnConfig.strFrame:GetText()
		if not str or string.len(str) < 1 then
			message = message .. "You must specify a display string for this warning\n"
		end
		
		if string.len(message) > 0 then
			TextWarningConfig.SetEnabled(txtWarnConfig, false)
			WarningWindow.ShowWarningWindow(txtWarnConfig.wrnWindow, message, function() TextWarningConfig.SetEnabled(txtWarnConfig, true) end)
			return
		end
		
		TextWarningConfig.FillWarning(txtWarnConfig)
	end
	
	txtWarnConfig.nameFrame:SetKeyFocus(false)
	txtWarnConfig.strFrame:SetKeyFocus(false)
	
	txtWarnConfig.windowFrame:SetVisible(false)
	if txtWarnConfig.callback then
		if okPressed == true then
			txtWarnConfig.callback(txtWarnConfig, txtWarnConfig.warning)
		else
			txtWarnConfig.callback(txtWarnConfig)
		end
	end
end

function TextWarningConfig.Show(txtWarnConfig, inWarning)
	if inWarning then
		txtWarnConfig.warning = TextWarningConfig.CopyWarning(inWarning)
	end
	
	local warning = txtWarnConfig.warning
	txtWarnConfig.colourFrame:SetBackgroundColor(warning.colour.red, warning.colour.green, warning.colour.blue)
	txtWarnConfig.flashColourFrame:SetBackgroundColor(warning.flashColour.red, warning.flashColour.green, warning.flashColour.blue)
	txtWarnConfig.nameFrame:SetText(warning.name)
	txtWarnConfig.strFrame:SetText(warning.str)
	NumberSlider.SetValue(txtWarnConfig.xSlider, warning.xPct)
	NumberSlider.SetValue(txtWarnConfig.ySlider, warning.yPct)
	NumberSlider.SetValue(txtWarnConfig.fontSizeSlider, warning.fontSize)
	NumberSlider.SetValue(txtWarnConfig.durationSlider, warning.duration)
	NumberSlider.SetValue(txtWarnConfig.alphaSlider, warning.alpha)
	NumberSlider.SetValue(txtWarnConfig.flashDurationSlider, warning.flashFrequency)
	NumberSlider.SetValue(txtWarnConfig.flashAlphaSlider, warning.flashAlpha)
	txtWarnConfig.windowFrame:SetVisible(true)
end

function TextWarningConfig.FillWarning(txtWarnConfig)
	txtWarnConfig.warning.name = txtWarnConfig.nameFrame:GetText()
	txtWarnConfig.warning.str = txtWarnConfig.strFrame:GetText()
	txtWarnConfig.warning.xPct = NumberSlider.GetValue(txtWarnConfig.xSlider)
	txtWarnConfig.warning.yPct = NumberSlider.GetValue(txtWarnConfig.ySlider)
	txtWarnConfig.warning.fontSize = NumberSlider.GetValue(txtWarnConfig.fontSizeSlider)
	txtWarnConfig.warning.duration = NumberSlider.GetValue(txtWarnConfig.durationSlider)
	txtWarnConfig.warning.alpha = NumberSlider.GetValue(txtWarnConfig.alphaSlider)
	txtWarnConfig.warning.flashFrequency = NumberSlider.GetValue(txtWarnConfig.flashDurationSlider)
	txtWarnConfig.warning.flashAlpha = NumberSlider.GetValue(txtWarnConfig.flashAlphaSlider)
end

function TextWarningConfig.SetEnabled(txtWarnConfig, enabled)
	--txtWarnConfig.nameFrame:SetEnabled(enabled)
	--txtWarnConfig.strFrame:SetEnabled(enabled)
	txtWarnConfig.okButton:SetEnabled(enabled)
	txtWarnConfig.cancelButton:SetEnabled(enabled)
	txtWarnConfig.xyButton:SetEnabled(enabled)
	txtWarnConfig.colourButton:SetEnabled(enabled)
	txtWarnConfig.flashColourButton:SetEnabled(enabled)
	NumberSlider.SetEnabled(txtWarnConfig.xSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.ySlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.fontSizeSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.durationSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.alphaSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.flashDurationSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.flashAlphaSlider, enabled)
end

function TextWarningConfig.CreateSlider(txtWarnConfig, row, col, minValue, maxValue, stepSize)
	local xyFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, row, col, "Frame")
	return NumberSlider.Create(xyFrame, minValue, maxValue, stepSize, minValue)
end

function TextWarningConfig.ShowXY(txtWarnConfig)
	txtWarnConfig.windowFrame:SetVisible(false)
	XYSelector.SetVisible(txtWarnConfig.xySelector, true)
end

function TextWarningConfig.SetXY(txtWarnConfig, x, y)
	txtWarnConfig.windowFrame:SetVisible(true)
	TextWarningConfig.FillWarning(txtWarnConfig)
	txtWarnConfig.warning.xPct = x
	txtWarnConfig.warning.yPct = y
	TextWarningConfig.Show(txtWarnConfig)
end

function TextWarningConfig.ShowChooseColour(txtWarnConfig)
	TextWarningConfig.SetEnabled(txtWarnConfig, false)
	ColourChooser.Show(txtWarnConfig.clrChooser, txtWarnConfig.warning.colour, function(x, colour) TextWarningConfig.ChooseColour(txtWarnConfig, colour) end)
end

function TextWarningConfig.ChooseColour(txtWarnConfig, colour)
	TextWarningConfig.SetEnabled(txtWarnConfig, true)
	if colour then
		txtWarnConfig.warning.colour = colour
		TextWarningConfig.FillWarning(txtWarnConfig)
		TextWarningConfig.Show(txtWarnConfig)
	end
end

function TextWarningConfig.ShowChooseFlashColour(txtWarnConfig)
	TextWarningConfig.SetEnabled(txtWarnConfig, false)
	ColourChooser.Show(txtWarnConfig.clrChooser, txtWarnConfig.warning.flashColour, function(x, colour) TextWarningConfig.ChooseFlashColour(txtWarnConfig, colour) end)
end

function TextWarningConfig.ChooseFlashColour(txtWarnConfig, colour)
	TextWarningConfig.SetEnabled(txtWarnConfig, true)
	if colour then
		txtWarnConfig.warning.flashColour = colour
		TextWarningConfig.FillWarning(txtWarnConfig)
		TextWarningConfig.Show(txtWarnConfig)
	end
end

function TextWarningConfig.CopyWarning(warning2)
	local warning = {}
	warning.warningType = WarnMe.warningTypeText
	warning.name = warning2.name
	warning.str = warning2.str
	warning.xPct = warning2.xPct
	warning.yPct = warning2.yPct
	warning.fontSize = warning2.fontSize
	warning.duration = warning2.duration
	warning.alpha = warning2.alpha
	warning.colour = {}
	warning.colour.red = warning2.colour.red
	warning.colour.green = warning2.colour.green
	warning.colour.blue = warning2.colour.blue
	warning.flashFrequency = warning2.flashFrequency
	warning.flashAlpha = warning2.flashAlpha
	warning.flashColour = {}
	warning.flashColour.red = warning2.flashColour.red
	warning.flashColour.green = warning2.flashColour.green
	warning.flashColour.blue = warning2.flashColour.blue
	
	return warning
end

function TextWarningConfig.GetDefaultWarning()
	local warning = {}
	warning.warningType = WarnMe.warningTypeText
	warning.name = ""
	warning.str = ""
	warning.xPct = 50
	warning.yPct = 50
	warning.fontSize = 200
	warning.duration = 5
	warning.alpha = 1
	warning.colour = {}
	warning.colour.red = 1
	warning.colour.green = 1
	warning.colour.blue = 1
	warning.flashFrequency = 1
	warning.flashAlpha = 0.2
	warning.flashColour = {}
	warning.flashColour.red = 1
	warning.flashColour.green = 0
	warning.flashColour.blue = 0
	
	return warning
end

