-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextureList = {}
TextureList.name = "TextureList"
TextureList.version = "v0.1"
TextureList.textureByName = nil

--
-- Main functions
--

-- TextureList.GetTexturePath
--    Function to retrieve the texture file path of the given ability
-- Params:
--   abilityName - the ability to lookup
-- Returns: the path to the texture or nil if the ability is not found
function TextureList.GetTexturePath(abilityName)
	if not TextureList.textureByName then
		TextureList.Initialise()
	end
	
	return TextureList.textureByName[abilityName]
end

-- TextureList.GetAllTexturePaths
--    Function to retrieve information on all texture paths
-- Returns: a table keyed by ability pathname where the value of each entry is the texture name
function TextureList.GetAllTexturePaths()
	if not TextureList.textureByName then
		TextureList.Initialise()
	end
	
	return TextureList.textureByPath
end

-- TextureList.GetAllTextureNames
--    Function to retrieve information on all texture names
-- Returns: a table keyed by ability name where the value of each entry is the texture pathname
function TextureList.GetAllTextureNames()
	if not TextureList.textureByName then
		TextureList.Initialise()
	end
	
	return TextureList.textureByName
end

function TextureList.Initialise()
	local textures = {}
	for path, name in pairs(TextureList.textureByPath) do
		local thisName = name
		local unknownCount = 1
		while (textures[thisName]) do
			thisName = name .. unknownCount
			unknownCount = unknownCount + 1
		end
		
		textures[thisName] = path
	end
	
	TextureList.textureByName = textures
end

--
-- Texture data
--

TextureList.textureByPath = {
		["Data/\\UI\\ability_icons\\alchemy1a.dds"] = "Endless Overgrowth Oil",
		["Data/\\UI\\ability_icons\\alchemy1b.dds"] = "Vial of Liquid Shadows",
		["Data/\\UI\\ability_icons\\anthem_of_competence.dds"] = "Anthem of Competence",
		["Data/\\UI\\ability_icons\\anthem_of_defiance.dds"] = "Anthem of Defiance",
		["Data/\\UI\\ability_icons\\anthem_of_fervor.dds"] = "Anthem of Fervor",
		["Data/\\UI\\ability_icons\\anthem_of_glory.dds"] = "Anthem of Glory",
		["Data/\\UI\\ability_icons\\arcaneshield2.dds"] = "Elemental Armor",
		["Data/\\UI\\ability_icons\\arcaneshield3.dds"] = "Ice Shield",
		["Data/\\UI\\ability_icons\\arcaneshield4.dds"] = "Heart of the Fire Mountain",
		["Data/\\UI\\ability_icons\\ardent_doctrine_01_a.dds"] = "Healer's Covenant",
		["Data/\\UI\\ability_icons\\arkanxshield1.dds"] = "Heart of the Frozen Sea",
		["Data/\\UI\\ability_icons\\auto_attack_1.dds"] = "PvP Turning Off",
		["Data/\\UI\\ability_icons\\awareness_01.dds"] = "Spotter's Order",
		["Data/\\UI\\ability_icons\\baneful_touch_01.dds"] = "Baneful Touch",
		["Data/\\UI\\ability_icons\\bash1.dds"] = "Bash",
		["Data/\\UI\\ability_icons\\beastmaster-greater_corporeal_manifestation.dds"] = "Summon Greater Primal Companion",
		["Data/\\UI\\ability_icons\\beastmaster-lesser_corporeal_manifestation_a.dds"] = "Summon Lesser Primal Companion",
		["Data/\\UI\\ability_icons\\beastmaster-natural_instincts.dds"] = "Bond of Feline Grace",
		["Data/\\UI\\ability_icons\\beastmaster_bond_of_might_a.dds"] = "Bond of Might",
		["Data/\\UI\\ability_icons\\beastmaster_enraged_companion_01.dds"] = "Enraged Companion",
		["Data/\\UI\\ability_icons\\beastmaster_enraged_companion_aura_01.dds"] = "Companion's Rage",
		["Data/\\UI\\ability_icons\\beastmaster_primal_recovery_01_a.dds"] = "Primal Recovery",
		["Data/\\UI\\ability_icons\\beastmaster_protective_companion_01.dds"] = "Protective Companion",
		["Data/\\UI\\ability_icons\\beastmaster_protective_companion_aura_01_b.dds"] = "Companion's Protection",
		["Data/\\UI\\ability_icons\\berserk1b.dds"] = "Sergeant's Order",
		["Data/\\UI\\ability_icons\\berserk3.dds"] = "Soldier's Bearing",
		["Data/\\UI\\ability_icons\\binding_of_affliction_01_a.dds"] = "Binding of Affliction",
		["Data/\\UI\\ability_icons\\binding_of_atrophy_01.dds"] = "Binding of Atrophy",
		["Data/\\UI\\ability_icons\\blazeofglory3.dds"] = "Aegis of Salvation",
		["Data/\\UI\\ability_icons\\bleeding_aa.dds"] = "Searing Vitality",
		["Data/\\UI\\ability_icons\\bloodfire1.dds"] = "Melted Skin",
		["Data/\\UI\\ability_icons\\bolt_of_radiance_01_a.dds"] = "Bolt of Radiance",
		["Data/\\UI\\ability_icons\\burning.dds"] = "Flicker",
		["Data/\\UI\\ability_icons\\cadence_a.dds"] = "Cadence",
		["Data/\\UI\\ability_icons\\castigation1b.dds"] = "Armor of Treachery",
		["Data/\\UI\\ability_icons\\castigation1c.dds"] = "Armor of Devotion",
		["Data/\\UI\\ability_icons\\castigation3.dds"] = "Armor of Awakening",
		["Data/\\UI\\ability_icons\\cautious_stalker.dds"] = "Rift Barrier",
		["Data/\\UI\\ability_icons\\cavalier_01.dds"] = "Cavalier",
		["Data/\\UI\\ability_icons\\censure_01.dds"] = "Censure",
		["Data/\\UI\\ability_icons\\chainlighting1.dds"] = "Forked Lightning",
		["Data/\\UI\\ability_icons\\chainlighting2b.dds"] = "Stormtouched",
		["Data/\\UI\\ability_icons\\chainlighting4.dds"] = "Electrocute",
		["Data/\\UI\\ability_icons\\chainlighting5a.dds"] = "Storm Burst",
		["Data/\\UI\\ability_icons\\champion-bull_rush.dds"] = "Bull Rush",
		["Data/\\UI\\ability_icons\\channelhealth3.dds"] = "Neddra's Strength",
		["Data/\\UI\\ability_icons\\charge1.dds"] = "Ravaging Strike",
		["Data/\\UI\\ability_icons\\charge3a.dds"] = "Punishing Blow",
		["Data/\\UI\\ability_icons\\charge4z.dds"] = "Power Strike",
		["Data/\\UI\\ability_icons\\chloromancer_void_life_01_a.dds"] = "Void Life",
		["Data/\\UI\\ability_icons\\cleave1.dds"] = "Rising Waterfall",
		["Data/\\UI\\ability_icons\\cleave1a.dds"] = "Frenzied Strike",
		["Data/\\UI\\ability_icons\\coalesceheat1.dds"] = "Cinder Burst",
		["Data/\\UI\\ability_icons\\coda_of_cowardice.dds"] = "Coda of Cowardice",
		["Data/\\UI\\ability_icons\\coda_of_distress.dds"] = "Coda of Distress",
		["Data/\\UI\\ability_icons\\coda_of_fury.dds"] = "Coda of Fury",
		["Data/\\UI\\ability_icons\\coda_of_restoration_a.dds"] = "Coda of Restoration",
		["Data/\\UI\\ability_icons\\coda_of_wrath_a.dds"] = "Coda of Wrath",
		["Data/\\UI\\ability_icons\\combat_pose.dds"] = "Combat Pose",
		["Data/\\UI\\ability_icons\\combat_pose1_a.dds"] = "Combat Preparation",
		["Data/\\UI\\ability_icons\\combat_survival.dds"] = "Motivational Roar",
		["Data/\\UI\\ability_icons\\concussion_01_a.dds"] = "Concussion",
		["Data/\\UI\\ability_icons\\concussion_01_b.dds"] = "Pillaging Stone",
		["Data/\\UI\\ability_icons\\concussive_blast_01.dds"] = "Concussive Blast",
		["Data/\\UI\\ability_icons\\conflagration1.dds"] = "Fire Storm",
		["Data/\\UI\\ability_icons\\conflagration2.dds"] = "Burning Ground",
		["Data/\\UI\\ability_icons\\confound1.dds"] = "Trepidation",
		["Data/\\UI\\ability_icons\\confound2.dds"] = "Bewilder",
		["Data/\\UI\\ability_icons\\conviction_01.dds"] = "Conviction",
		["Data/\\UI\\ability_icons\\cripple1.dds"] = "Maim",
		["Data/\\UI\\ability_icons\\cripple1a.dds"] = "Leg Sweep",
		["Data/\\UI\\ability_icons\\crippling_shot_01.dds"] = "Crippling Shot",
		["Data/\\UI\\ability_icons\\cripplingshot1.dds"] = "Fork",
		["Data/\\UI\\ability_icons\\critical2.dds"] = "Pin Target",
		["Data/\\UI\\ability_icons\\critical2a.dds"] = "Flamespear",
		["Data/\\UI\\ability_icons\\critical3.dds"] = "Deathblow",
		["Data/\\UI\\ability_icons\\critical4.dds"] = "Flesh Rip",
		["Data/\\UI\\ability_icons\\criticalstrike1.dds"] = "Debilitating Strike",
		["Data/\\UI\\ability_icons\\criticalsweep1b.dds"] = "Rift Disturbance",
		["Data/\\UI\\ability_icons\\criticalsweep2.dds"] = "Bloodthirst",
		["Data/\\UI\\ability_icons\\criticalsweep3c.dds"] = "Bladefury",
		["Data/\\UI\\ability_icons\\cruel_vengeance_01.dds"] = "Cruel Vengence",
		["Data/\\UI\\ability_icons\\dancing_steel.dds"] = "Promise of Steel",
		["Data/\\UI\\ability_icons\\dark_harbor.dds"] = "Dark Harbor",
		["Data/\\UI\\ability_icons\\darkball1.dds"] = "Warlock Armor",
		["Data/\\UI\\ability_icons\\darkball2.dds"] = "Dark Armor",
		["Data/\\UI\\ability_icons\\darkball2z.dds"] = "Dark Touch",
		["Data/\\UI\\ability_icons\\debilitating_poison_01.dds"] = "Vicious Cleave",
		["Data/\\UI\\ability_icons\\debilitating_poison_01_a.dds"] = "Debilitating Poison",
		["Data/\\UI\\ability_icons\\debilitating_poison_01_b.dds"] = "Leeching Poison",
		["Data/\\UI\\ability_icons\\defensivestrike1.dds"] = "Shield Throw",
		["Data/\\UI\\ability_icons\\deflect1c.dds"] = "Burn",
		["Data/\\UI\\ability_icons\\deflection2.dds"] = "Bend Like the Reed",
		["Data/\\UI\\ability_icons\\deflection2a.dds"] = "Retaliation",
		["Data/\\UI\\ability_icons\\deflection4c.dds"] = "Turn the Blade",
		["Data/\\UI\\ability_icons\\defstrike2.dds"] = "Aggressive Block",
		["Data/\\UI\\ability_icons\\devouring_mist.dds"] = "Decay",
		["Data/\\UI\\ability_icons\\devouring_mist_a.dds"] = "Lurking Decay",
		["Data/\\UI\\ability_icons\\devout_conviction_01.dds"] = "Radiant Spores",
		["Data/\\UI\\ability_icons\\dirtytricks1c.dds"] = "Glyphs of Power",
		["Data/\\UI\\ability_icons\\disabling2.dds"] = "Planar Strike",
		["Data/\\UI\\ability_icons\\discombobulate2a.dds"] = "Burning Intellect",
		["Data/\\UI\\ability_icons\\divert_rage_01.dds"] = "Divert Rage",
		["Data/\\UI\\ability_icons\\doctrine_of_bliss_01.dds"] = "Cloudburst",
		["Data/\\UI\\ability_icons\\doctrine_of_bliss_01_b.dds"] = "Synergize",
		["Data/\\UI\\ability_icons\\doctrine_of_bliss_01_c.dds"] = "Judgment",
		["Data/\\UI\\ability_icons\\double_cross_01.dds"] = "Dual Strike",
		["Data/\\UI\\ability_icons\\drainlife1.dds"] = "Life Leech",
		["Data/\\UI\\ability_icons\\drowning2.dds"] = "Drown",
		["Data/\\UI\\ability_icons\\druid-hidden_path.dds"] = "Ruin",
		["Data/\\UI\\ability_icons\\druid-spiteful_spirit_a.dds"] = "Track Death Creatures",
		["Data/\\UI\\ability_icons\\druid-stone_strike.dds"] = "Track Earth Creatures",
		["Data/\\UI\\ability_icons\\ebon_terror_01.dds"] = "Ebon Terror",
		["Data/\\UI\\ability_icons\\elementalres2b.dds"] = "Intensify Elements",
		["Data/\\UI\\ability_icons\\elementalward2a.dds"] = "Aspect of the Elements",
		["Data/\\UI\\ability_icons\\elementalward3.dds"] = "Revitalize",
		["Data/\\UI\\ability_icons\\elemres4.dds"] = "Static Flux",
		["Data/\\UI\\ability_icons\\elemresist1.dds"] = "Elemental Forces",
		["Data/\\UI\\ability_icons\\elemresistance3.dds"] = "Exposure",
		["Data/\\UI\\ability_icons\\elemward1.dds"] = "Fire Armor",
		["Data/\\UI\\ability_icons\\elemward1a.dds"] = "Smoldering Power",
		["Data/\\UI\\ability_icons\\enchant1.dds"] = "Pact Conversion",
		["Data/\\UI\\ability_icons\\energybeam1.dds"] = "Flame Bolt",
		["Data/\\UI\\ability_icons\\energyburst5.dds"] = "Static Discharge",
		["Data/\\UI\\ability_icons\\escape_artist_01.dds"] = "Escape Artist",
		["Data/\\UI\\ability_icons\\even_justice_01.dds"] = "Even Justice",
		["Data/\\UI\\ability_icons\\excommunicate.dds"] = "Excommunicate",
		["Data/\\UI\\ability_icons\\exorcize2.dds"] = "Circle of Oblivion",
		["Data/\\UI\\ability_icons\\exorcize3.dds"] = "Impede",
		["Data/\\UI\\ability_icons\\exposeweak1b.dds"] = "Slayer's Bearing",
		["Data/\\UI\\ability_icons\\exposeweakness1.dds"] = "Mark of Inevitability",
		["Data/\\UI\\ability_icons\\exposeweakness2a.dds"] = "Puncture",
		["Data/\\UI\\ability_icons\\faithful_doctrine_01.dds"] = "Aegis of Vitality",
		["Data/\\UI\\ability_icons\\faithful_doctrine_01_a.dds"] = "Righteous Mandate",
		["Data/\\UI\\ability_icons\\fanaticism.dds"] = "Battlefield Awareness",
		["Data/\\UI\\ability_icons\\fanfare_of_knowledge.dds"] = "Fanfare of Knowledge",
		["Data/\\UI\\ability_icons\\fanfare_of_power.dds"] = "Fanfare of Power",
		["Data/\\UI\\ability_icons\\fanfare_of_vigor.dds"] = "Fanfare of Vigor",
		["Data/\\UI\\ability_icons\\fiery_spike_01.dds"] = "Fiery Spike",
		["Data/\\UI\\ability_icons\\finalstrike2.dds"] = "Mark of Extermination",
		["Data/\\UI\\ability_icons\\fire_and_forget.dds"] = "Deaden",
		["Data/\\UI\\ability_icons\\fireball.dds"] = "Fireball",
		["Data/\\UI\\ability_icons\\flame_speaker_a.dds"] = "Healing Breath",
		["Data/\\UI\\ability_icons\\flamejet1.dds"] = "Flame Jet",
		["Data/\\UI\\ability_icons\\flare_of_light_01.dds"] = "Righteous Blow",
		["Data/\\UI\\ability_icons\\fleeting_instincts_02_b.dds"] = "Fleeting Instincts",
		["Data/\\UI\\ability_icons\\frost_hammer1.dds"] = "Crushing Blow",
		["Data/\\UI\\ability_icons\\generic_ability_005.dds"] = "Path of the Hurricane",
		["Data/\\UI\\ability_icons\\generic_ability_011.dds"] = "Lifebound Veil",
		["Data/\\UI\\ability_icons\\generic_ability_012_a.dds"] = "Increased Fire Power",
		["Data/\\UI\\ability_icons\\generic_ability_016.dds"] = "Shield of the Chosen",
		["Data/\\UI\\ability_icons\\generic_ability_022.dds"] = "Riftwalk",
		["Data/\\UI\\ability_icons\\generic_ability_024_a.dds"] = "Purpose",
		["Data/\\UI\\ability_icons\\generic_ability_025.dds"] = "Summon: Greater Air Elemental",
		["Data/\\UI\\ability_icons\\generic_ability_025_a.dds"] = "Avatar of Wind",
		["Data/\\UI\\ability_icons\\generic_ability_026_a.dds"] = "Abyssal Madness",
		["Data/\\UI\\ability_icons\\generic_ability_030.dds"] = "Planar Lure",
		["Data/\\UI\\ability_icons\\generic_ability_031.dds"] = "Rift Storm",
		["Data/\\UI\\ability_icons\\generic_ability_031_2.dds"] = "Fast Summon",
		["Data/\\UI\\ability_icons\\guardian_phase.dds"] = "Guardian Phase",
		["Data/\\UI\\ability_icons\\hasted_time.dds"] = "Hasted Time",
		["Data/\\UI\\ability_icons\\headshot1b.dds"] = "Deadeye Shot",
		["Data/\\UI\\ability_icons\\headshot2.dds"] = "Bull's Eye",
		["Data/\\UI\\ability_icons\\headshot3.dds"] = "Head Shot",
		["Data/\\UI\\ability_icons\\heal1.dds"] = "Life of the Era",
		["Data/\\UI\\ability_icons\\heal2z.dds"] = "Healing Invocation",
		["Data/\\UI\\ability_icons\\heal3.dds"] = "Healing Communion",
		["Data/\\UI\\ability_icons\\healing_flare_a.dds"] = "Empowering Light",
		["Data/\\UI\\ability_icons\\healthchannel1g.dds"] = "Harsh Discipline",
		["Data/\\UI\\ability_icons\\healthchannel3.dds"] = "Quicken Elements",
		["Data/\\UI\\ability_icons\\heavybleeding.dds"] = "Lust for Blood",
		["Data/\\UI\\ability_icons\\hellfire_blades_01.dds"] = "Hellfire Blades",
		["Data/\\UI\\ability_icons\\heroicsurge1.dds"] = "Flametouched",
		["Data/\\UI\\ability_icons\\hidden_veil_01.dds"] = "Hidden Veil",
		["Data/\\UI\\ability_icons\\holysmite1.dds"] = "Life's Vengeance",
		["Data/\\UI\\ability_icons\\holystrength2.dds"] = "Serendipity",
		["Data/\\UI\\ability_icons\\holystrength3c.dds"] = "Noble Blessing",
		["Data/\\UI\\ability_icons\\holysweep2a.dds"] = "Inescapable Fury",
		["Data/\\UI\\ability_icons\\holysweep3.dds"] = "Corporal Punishment",
		["Data/\\UI\\ability_icons\\holysweep3a.dds"] = "Blazing Strike",
		["Data/\\UI\\ability_icons\\humbling_blow_01.dds"] = "Humility",
		["Data/\\UI\\ability_icons\\icebolt1.dds"] = "Icicle",
		["Data/\\UI\\ability_icons\\icebolt3.dds"] = "Flash Freeze",
		["Data/\\UI\\ability_icons\\icebolt4aa.dds"] = "Arctic Blast",
		["Data/\\UI\\ability_icons\\icestormdark.dds"] = "Hailstorm",
		["Data/\\UI\\ability_icons\\icy_heart1.dds"] = "Ageless Ice",
		["Data/\\UI\\ability_icons\\incite_01.dds"] = "Deathly Calling",
		["Data/\\UI\\ability_icons\\increased_explosive_power.dds"] = "Track Fire Creatures",
		["Data/\\UI\\ability_icons\\infestation_01.dds"] = "Infestation",
		["Data/\\UI\\ability_icons\\inspiration1.dds"] = "Natural Awareness",
		["Data/\\UI\\ability_icons\\interdict_01.dds"] = "Interdict",
		["Data/\\UI\\ability_icons\\internalizeheat1.dds"] = "Inferno",
		["Data/\\UI\\ability_icons\\internalizeheat3.dds"] = "Heat Retention",
		["Data/\\UI\\ability_icons\\intervene2.dds"] = "Duality of Mind",
		["Data/\\UI\\ability_icons\\intimidatingweapon1a.dds"] = "Fated Blow",
		["Data/\\UI\\ability_icons\\intimidatingweapon2.dds"] = "Strike Like Iron",
		["Data/\\UI\\ability_icons\\ironskin1.dds"] = "Summon: Lesser Earth Elemental",
		["Data/\\UI\\ability_icons\\ironwill2.dds"] = "Courage of the Eagle",
		["Data/\\UI\\ability_icons\\kick4.dds"] = "Fleet of Foot",
		["Data/\\UI\\ability_icons\\killer_instinct.dds"] = "Splinter Shot",
		["Data/\\UI\\ability_icons\\lance1.dds"] = "Electrified Munitions",
		["Data/\\UI\\ability_icons\\latent_blaze.dds"] = "Latent Blaze",
		["Data/\\UI\\ability_icons\\latent_blaze_a.dds"] = "Angelic Flight",
		["Data/\\UI\\ability_icons\\layinghands1x.dds"] = "Reprieve",
		["Data/\\UI\\ability_icons\\layonhands2z.dds"] = "Healing Spray",
		["Data/\\UI\\ability_icons\\layonhands5.dds"] = "Touch of Life",
		["Data/\\UI\\ability_icons\\life-drain5.dds"] = "Grim Lure",
		["Data/\\UI\\ability_icons\\life_and_death_concord_b.dds"] = "Life and Death Concord",
		["Data/\\UI\\ability_icons\\lighting1.dds"] = "Thunderbolt",
		["Data/\\UI\\ability_icons\\lighting2a.dds"] = "Lightning Strike",
		["Data/\\UI\\ability_icons\\lighting_fury.dds"] = "Lightning Fury",
		["Data/\\UI\\ability_icons\\lightingbolt4.dds"] = "Lightning Field",
		["Data/\\UI\\ability_icons\\maelstrom_a.dds"] = "Building Storm",
		["Data/\\UI\\ability_icons\\magicresist3.dds"] = "Synthesis",
		["Data/\\UI\\ability_icons\\magicward1.dds"] = "Spiritual Protection",
		["Data/\\UI\\ability_icons\\malicious_strike_01_a.dds"] = "Slashing Strike",
		["Data/\\UI\\ability_icons\\manachannel1.dds"] = "Aggressive Renewal",
		["Data/\\UI\\ability_icons\\master_of_the_abyss_01.dds"] = "Master of the Abyss",
		["Data/\\UI\\ability_icons\\menacingroar2.dds"] = "Feral Instincts",
		["Data/\\UI\\ability_icons\\menacingroar4.dds"] = "Battlefield Distraction",
		["Data/\\UI\\ability_icons\\menacingroar4b.dds"] = "Spiritual Deficiency",
		["Data/\\UI\\ability_icons\\mortalshot1.dds"] = "Vampiric Munitions",
		["Data/\\UI\\ability_icons\\mortalshot1b.dds"] = "Silver Tip Munitions",
		["Data/\\UI\\ability_icons\\mortalshot2.dds"] = "Barbed Shot",
		["Data/\\UI\\ability_icons\\motif_of_bravery.dds"] = "Motif of Bravery",
		["Data/\\UI\\ability_icons\\motif_of_encouragement.dds"] = "Motif of Encouragement",
		["Data/\\UI\\ability_icons\\motif_of_focus.dds"] = "Motif of Focus",
		["Data/\\UI\\ability_icons\\motif_of_grandeur.dds"] = "Motif of Grandeur",
		["Data/\\UI\\ability_icons\\motif_of_regeneration.dds"] = "Motif of Renewal",
		["Data/\\UI\\ability_icons\\motif_of_regeneration_a.dds"] = "Motif of Regeneration",
		["Data/\\UI\\ability_icons\\motif_of_tenacity.dds"] = "Motif of Tenacity",
		["Data/\\UI\\ability_icons\\necropotence2.dds"] = "Soul Sickness",
		["Data/\\UI\\ability_icons\\necropotence4a.dds"] = "Necrotic Wounds",
		["Data/\\UI\\ability_icons\\nysyrs_rebuke.dds"] = "Nysyr's Rebuke",
		["Data/\\UI\\ability_icons\\opportunity_01_a.dds"] = "Living Energy",
		["Data/\\UI\\ability_icons\\overwhelming_brilliance_01.dds"] = "Mien of Honor",
		["Data/\\UI\\ability_icons\\paladin_face_slam.dds"] = "Face Slam",
		["Data/\\UI\\ability_icons\\paragon-way_of_mountain_a.dds"] = "Way of the Mountain",
		["Data/\\UI\\ability_icons\\paragon_flowing_strikes.dds"] = "Flowing Strikes",
		["Data/\\UI\\ability_icons\\paragon_shifting_blades_01.dds"] = "Shifting Blades",
		["Data/\\UI\\ability_icons\\paragon_way_of_the_river_01.dds"] = "Way of the River",
		["Data/\\UI\\ability_icons\\paralyzing_strike_01.dds"] = "Devouring Blow",
		["Data/\\UI\\ability_icons\\perseverance.dds"] = "Perseverance",
		["Data/\\UI\\ability_icons\\piercing_shot_01.dds"] = "Rending Munitions",
		["Data/\\UI\\ability_icons\\piety2a.dds"] = "Provoke",
		["Data/\\UI\\ability_icons\\plague_01_a.dds"] = "Plague Bringer",
		["Data/\\UI\\ability_icons\\poisonone2a.dds"] = "Lethal Poison",
		["Data/\\UI\\ability_icons\\poisonone2b.dds"] = "Virulent Poison",
		["Data/\\UI\\ability_icons\\poisontwo2.dds"] = "Venom Strike",
		["Data/\\UI\\ability_icons\\poisontwo4.dds"] = "Poison Malice",
		["Data/\\UI\\ability_icons\\power_chord.dds"] = "Power Chord",
		["Data/\\UI\\ability_icons\\primal_death_01.dds"] = "Looming Demise",
		["Data/\\UI\\ability_icons\\purge_a.dds"] = "Purge",
		["Data/\\UI\\ability_icons\\pyroclasty1.dds"] = "Withering Flames",
		["Data/\\UI\\ability_icons\\pyroclasty2.dds"] = "Countdown",
		["Data/\\UI\\ability_icons\\quick_reload_a.dds"] = "Quick Reload",
		["Data/\\UI\\ability_icons\\rainofarrows1.dds"] = "Fan Out",
		["Data/\\UI\\ability_icons\\rainofarrows2.dds"] = "Rain of Arrows",
		["Data/\\UI\\ability_icons\\rainofarrows4a.dds"] = "Crossfire",
		["Data/\\UI\\ability_icons\\rally1.dds"] = "Cleansing Waters",
		["Data/\\UI\\ability_icons\\rampage1.dds"] = "Empowering Strike",
		["Data/\\UI\\ability_icons\\rampage2.dds"] = "Leader's Mark",
		["Data/\\UI\\ability_icons\\rapidshot1.dds"] = "Quick Shot",
		["Data/\\UI\\ability_icons\\rapidshot1a.dds"] = "Rapid Fire Shot",
		["Data/\\UI\\ability_icons\\rapidshot2.dds"] = "Shadow Fire",
		["Data/\\UI\\ability_icons\\rapidshot3.dds"] = "Hasted Shot",
		["Data/\\UI\\ability_icons\\rapidshot4.dds"] = "Swift Shot",
		["Data/\\UI\\ability_icons\\rapidshot4a.dds"] = "Empowered Shot",
		["Data/\\UI\\ability_icons\\redirect1a.dds"] = "Dismiss Pet",
		["Data/\\UI\\ability_icons\\redirect4a.dds"] = "Touch the Light",
		["Data/\\UI\\ability_icons\\releaseheat2.dds"] = "Flashfire",
		["Data/\\UI\\ability_icons\\releaseheat3a.dds"] = "Burning Shield",
		["Data/\\UI\\ability_icons\\releaseheat4b.dds"] = "Internalize Charge",
		["Data/\\UI\\ability_icons\\releaseheat5.dds"] = "Track Elementals",
		["Data/\\UI\\ability_icons\\remote_clipping.dds"] = "Unstable Attunement",
		["Data/\\UI\\ability_icons\\removeheat1.dds"] = "Icy Gale",
		["Data/\\UI\\ability_icons\\reparation_01.dds"] = "Reparation",
		["Data/\\UI\\ability_icons\\requital_01_a.dds"] = "Mien of Leadership",
		["Data/\\UI\\ability_icons\\resolve1.dds"] = "Planar Protection",
		["Data/\\UI\\ability_icons\\resonance.dds"] = "Resonance",
		["Data/\\UI\\ability_icons\\ressurect4.dds"] = "Aspect of the Fallen Hero",
		["Data/\\UI\\ability_icons\\ressurect4bb.dds"] = "Well of Life",
		["Data/\\UI\\ability_icons\\ressurect4c.dds"] = "Breath of Life",
		["Data/\\UI\\ability_icons\\ressurect4y.dds"] = "Life's Return",
		["Data/\\UI\\ability_icons\\ressurection1za.dds"] = "Life's Grace",
		["Data/\\UI\\ability_icons\\ressurection3bb.dds"] = "Righteous Imperative",
		["Data/\\UI\\ability_icons\\restore2b.dds"] = "Healing Grace",
		["Data/\\UI\\ability_icons\\retribution1.dds"] = "Grim Satisfaction",
		["Data/\\UI\\ability_icons\\revitalize1c.dds"] = "Wardstone Attunement",
		["Data/\\UI\\ability_icons\\revitalize1u.dds"] = "Wardstone Aura",
		["Data/\\UI\\ability_icons\\ricochet1.dds"] = "Trick Shot",
		["Data/\\UI\\ability_icons\\riff.dds"] = "Riff",
		["Data/\\UI\\ability_icons\\rift_affinity.dds"] = "Planebound Resilience",
		["Data/\\UI\\ability_icons\\riftblade-avatar_of_water.dds"] = "Summon: Greater Water Elemental",
		["Data/\\UI\\ability_icons\\riftblade-avatar_of_water_a.dds"] = "Avatar of Water",
		["Data/\\UI\\ability_icons\\riftblade-earth_burst_a.dds"] = "Earth Burst",
		["Data/\\UI\\ability_icons\\riftblade-surging_energy_a.dds"] = "Surging Energy",
		["Data/\\UI\\ability_icons\\riftblade_frost_strike_01_a.dds"] = "Frost Strike",
		["Data/\\UI\\ability_icons\\riftblade_stone_shield_01_b.dds"] = "Stoneshield",
		["Data/\\UI\\ability_icons\\riftblade_thunder_strike_01.dds"] = "Thunder Strike",
		["Data/\\UI\\ability_icons\\righteous_healing_01.dds"] = "Doctrine of Righteousness",
		["Data/\\UI\\ability_icons\\righteous_invocation_01.dds"] = "Doctrine of Loyalty",
		["Data/\\UI\\ability_icons\\roguestrike1a.dds"] = "Dire Blow",
		["Data/\\UI\\ability_icons\\roguestrike2a.dds"] = "Flinching Strike",
		["Data/\\UI\\ability_icons\\rudeawake1.dds"] = "Rested",
		["Data/\\UI\\ability_icons\\ruthless_stalker.dds"] = "Ruthless Stalker",
		["Data/\\UI\\ability_icons\\sacrificepet1.dds"] = "Ignite",
		["Data/\\UI\\ability_icons\\sacrificepet4a.dds"] = "Fell Blades",
		["Data/\\UI\\ability_icons\\salvation_01.dds"] = "Salvation",
		["Data/\\UI\\ability_icons\\sanction_heretic_a.dds"] = "Sanction Heretic",
		["Data/\\UI\\ability_icons\\scourge_of_darkness_01_a.dds"] = "Scourge of Darkness",
		["Data/\\UI\\ability_icons\\secondwind2a.dds"] = "Way of the Wind",
		["Data/\\UI\\ability_icons\\sentinel-quietus.dds"] = "Quietus",
		["Data/\\UI\\ability_icons\\sentinel-refreshing_communion.dds"] = "Healer's Haste",
		["Data/\\UI\\ability_icons\\sentinel-shards_of_light_b.dds"] = "Shards of Light",
		["Data/\\UI\\ability_icons\\shadow_shift_b.dds"] = "Shadow Shift",
		["Data/\\UI\\ability_icons\\shaman-courage_of_the_bear.dds"] = "Courage of the Bear",
		["Data/\\UI\\ability_icons\\shaman-courage_of_the_jaguar.dds"] = "Courage of the Jaguar",
		["Data/\\UI\\ability_icons\\shaman-glacial_shield.dds"] = "Glacial Shield",
		["Data/\\UI\\ability_icons\\shaman-glory_of_the_chosen_a.dds"] = "Glory of the Chosen",
		["Data/\\UI\\ability_icons\\shaman-lighting_hammer.dds"] = "Lightning Hammer",
		["Data/\\UI\\ability_icons\\shaman-vengeance_of_frozen_earth.dds"] = "Vengeance of the Frozen Earth",
		["Data/\\UI\\ability_icons\\shaman-vengeance_of_winter_storm.dds"] = "Vengeance of the Winter Storm",
		["Data/\\UI\\ability_icons\\shatter1.dds"] = "Enraged Essence",
		["Data/\\UI\\ability_icons\\shatter3.dds"] = "Furious Rage",
		["Data/\\UI\\ability_icons\\shockwave1.dds"] = "Searing Strike",
		["Data/\\UI\\ability_icons\\shockwave2a.dds"] = "Massive Blow",
		["Data/\\UI\\ability_icons\\shockwave3.dds"] = "Storm Blade",
		["Data/\\UI\\ability_icons\\shockwave3d.dds"] = "Planar Blade",
		["Data/\\UI\\ability_icons\\shreddingtrap2.dds"] = "Encase",
		["Data/\\UI\\ability_icons\\shroud_of_agony_a.dds"] = "Shroud Of Agony",
		["Data/\\UI\\ability_icons\\shroudofpenitence1a.dds"] = "Divine Pardon",
		["Data/\\UI\\ability_icons\\slam1.dds"] = "Battle Charge",
		["Data/\\UI\\ability_icons\\slam2.dds"] = "Avalanche Strike",
		["Data/\\UI\\ability_icons\\slam4a.dds"] = "Fiery Burst",
		["Data/\\UI\\ability_icons\\smite5.dds"] = "Focus of Strength",
		["Data/\\UI\\ability_icons\\soulbind.dds"] = "Soul Recall",
		["Data/\\UI\\ability_icons\\souldrain1.dds"] = "Raging Storm",
		["Data/\\UI\\ability_icons\\souldrain4.dds"] = "Soul Drain",
		["Data/\\UI\\ability_icons\\spiritblast2.dds"] = "Healing Current",
		["Data/\\UI\\ability_icons\\spiritblast3.dds"] = "Spark",
		["Data/\\UI\\ability_icons\\spiritblast4b.dds"] = "Bolt of Depravity",
		["Data/\\UI\\ability_icons\\spiritblast4bc.dds"] = "Crystalline Missiles",
		["Data/\\UI\\ability_icons\\spiritbolt1.dds"] = "Light Bolt",
		["Data/\\UI\\ability_icons\\spiritofcrusader2.dds"] = "Rage of the North",
		["Data/\\UI\\ability_icons\\spiritofcrusader3.dds"] = "Wardstone Blessing",
		["Data/\\UI\\ability_icons\\spiritoflion3.dds"] = "Predatory Instincts",
		["Data/\\UI\\ability_icons\\spiritofsaints1.dds"] = "Divine Call",
		["Data/\\UI\\ability_icons\\spiritofsaints3.dds"] = "Aid Command",
		["Data/\\UI\\ability_icons\\spiritoftree1.dds"] = "Track Life Creatures",
		["Data/\\UI\\ability_icons\\spiritoftree1a.dds"] = "Wild Growth",
		["Data/\\UI\\ability_icons\\spiritstrike5.dds"] = "Pacifying Strike",
		["Data/\\UI\\ability_icons\\spiritual_will.dds"] = "Cleansing Prayer",
		["Data/\\UI\\ability_icons\\staggeringshot1.dds"] = "Serrated Blades",
		["Data/\\UI\\ability_icons\\stalking_phase_a.dds"] = "Stalker Phase",
		["Data/\\UI\\ability_icons\\stealth_z.dds"] = "Conceal",
		["Data/\\UI\\ability_icons\\steelskin2.dds"] = "Summon: Greater Earth Elemental",
		["Data/\\UI\\ability_icons\\steelskin3.dds"] = "Intercept",
		["Data/\\UI\\ability_icons\\stormcaller_lightning_storm_02_b.dds"] = "Lightning Storm",
		["Data/\\UI\\ability_icons\\strafe_a.dds"] = "Strafe",
		["Data/\\UI\\ability_icons\\strike_of_judgement_01.dds"] = "Strike of Judgment",
		["Data/\\UI\\ability_icons\\strong_constitution_01_a.dds"] = "Focus of Body",
		["Data/\\UI\\ability_icons\\survival1.dds"] = "Track Beasts",
		["Data/\\UI\\ability_icons\\sweep1.dds"] = "Backhanded Blow",
		["Data/\\UI\\ability_icons\\sweep3c.dds"] = "Primal Strike",
		["Data/\\UI\\ability_icons\\sweep4a.dds"] = "Fierce Strike",
		["Data/\\UI\\ability_icons\\sweep4b.dds"] = "Mighty Blow",
		["Data/\\UI\\ability_icons\\swiftstrike1a.dds"] = "Reckless Strike",
		["Data/\\UI\\ability_icons\\triage3.dds"] = "Rejuvenate",
		["Data/\\UI\\ability_icons\\triumphant_spirit_a.dds"] = "Verse of Rebirth",
		["Data/\\UI\\ability_icons\\tumble1.dds"] = "Thread the Trees",
		["Data/\\UI\\ability_icons\\twilight_force_01.dds"] = "Ice Shear",
		["Data/\\UI\\ability_icons\\twin_strike_a.dds"] = "Reaping Harvest",
		["Data/\\UI\\ability_icons\\underwaterbreath2.dds"] = "Blessing of the Sea",
		["Data/\\UI\\ability_icons\\vex.dds"] = "Vex",
		["Data/\\UI\\ability_icons\\virtuoso_b.dds"] = "Track Water Creatures",
		["Data/\\UI\\ability_icons\\virtuous_conviction_01.dds"] = "Sovereignty",
		["Data/\\UI\\ability_icons\\vitalizingshield1b.dds"] = "Lifegiving Veil",
		["Data/\\UI\\ability_icons\\void_knight-pact.dds"] = "Pact",
		["Data/\\UI\\ability_icons\\void_knight-ravenous_strength_b.dds"] = "Ravenous Strength",
		["Data/\\UI\\ability_icons\\void_knight-void_a.dds"] = "Void",
		["Data/\\UI\\ability_icons\\voidknight_ravenous_defence_01.dds"] = "Ravenous Defense",
		["Data/\\UI\\ability_icons\\warcry.dds"] = "Call to Battle",
		["Data/\\UI\\ability_icons\\ward1z.dds"] = "Circle of Life",
		["Data/\\UI\\ability_icons\\ward1zz.dds"] = "Orbs of the Stream",
		["Data/\\UI\\ability_icons\\ward4.dds"] = "Stone Armor",
		["Data/\\UI\\ability_icons\\warden-crushing_wave.dds"] = "Crushing Wave",
		["Data/\\UI\\ability_icons\\warden-deluge.dds"] = "Deluge",
		["Data/\\UI\\ability_icons\\warden-healing_flood.dds"] = "Healing Flood",
		["Data/\\UI\\ability_icons\\warden-tidal_surge.dds"] = "Tidal Surge",
		["Data/\\UI\\ability_icons\\warden-waterjet.dds"] = "Waterjet",
		["Data/\\UI\\ability_icons\\warlord-cracking_skulls_a.dds"] = "Cracking Skulls",
		["Data/\\UI\\ability_icons\\warlord-cutting_distraction.dds"] = "Cutting Distraction",
		["Data/\\UI\\ability_icons\\weakening_essence_01.dds"] = "Weakening Essence",
		["Data/\\UI\\ability_icons\\weaponsmith1a.dds"] = "Double Jeopardy",
		["Data/\\UI\\ability_icons\\whirlingblades1.dds"] = "Ripple",
		["Data/\\UI\\ability_icons\\whirlwind3.dds"] = "Feral Sweep",
		["Data/\\UI\\item_icons\\1h_mace_air_epic_b.dds"] = "Lure of the Eye",
		["Data/\\UI\\item_icons\\1h_shield_054a.dds"] = "Shield of the Hero",
		["Data/\\UI\\item_icons\\2h_polearm_019_a.dds"] = "Stonespear",
		["Data/\\UI\\item_icons\\barnacled_shell.dds"] = "Breath of the Abyssal",
		["Data/\\UI\\item_icons\\food_11a.dds"] = "Icewatch Cupcake",
		["Data/\\UI\\item_icons\\heroic_potion_spherical_6.dds"] = "Veteran's Experience Vial",
		["Data/\\UI\\item_icons\\pick4.dds"] = "Track Ore",
		["Data/\\UI\\item_icons\\plant1.dds"] = "Track Plants",
		["Data/\\UI\\item_icons\\plant39a.dds"] = "Omen Sight",
		["Data/\\UI\\item_icons\\plate_114a_chest_a.dds"] = "Freeze Armor",
		["Data/\\UI\\item_icons\\rune_5_d_red_a.dds"] = "Blazing Rune",
		["Data/\\UI\\item_icons\\rune_5_e_pink.dds"] = "Windshielding Rune",
		["Data/\\UI\\item_icons\\scroll2a.dds"] = "Exalted Scroll of Strength",
		["Data/\\UI\\item_icons\\trinket4.dds"] = "Proper Timing",
		["Data/\\UI\\item_icons\\trinket4a.dds"] = "PvP Invulnerability",
		["Data/\\UI\\item_icons\\wood3.dds"] = "Track Wood"
	}
