-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TexturePicker = {}
TexturePicker.name = "TexturePicker"
TexturePicker.version = "v0.1"

--
-- Main functions
--

--
-- UI functions
--

function TexturePicker.Create(parentFrame, callback)
	local txtPicker = {}
	txtPicker.parentFrame = parentFrame
	txtPicker.callback = callback
	txtPicker.textureArray = {}
	txtPicker.page = 1
	txtPicker.maxPage = 1
	
	txtPicker.textFrame = UI.CreateFrame("RiftWindow", "TextureFrame", parentFrame)
	txtPicker.textFrame:SetPoint("CENTER", UIParent, "CENTER")
	txtPicker.textFrame:SetWidth(1188)
	txtPicker.textFrame:SetHeight(628)
	txtPicker.textFrame:SetTitle("Pick Texture")
	txtPicker.textFrame:SetLayer(4)
	txtPicker.textFrame:SetVisible(false)

	local xMargin = 20
	local yMargin = 20
	txtPicker.okButton = UI.CreateFrame("RiftButton", "OKButton", txtPicker.textFrame)
	txtPicker.okButton:SetPoint("BOTTOMLEFT", txtPicker.textFrame, "BOTTOMLEFT", xMargin, -yMargin)
	txtPicker.okButton:SetText("OK")
	txtPicker.okButton.Event.LeftPress = function() TexturePicker.WindowClose(txtPicker, true) end	

	txtPicker.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtPicker.textFrame)
	txtPicker.cancelButton:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	txtPicker.cancelButton:SetText("Cancel")
	txtPicker.cancelButton.Event.LeftPress = function() TexturePicker.WindowClose(txtPicker, false) end	

	txtPicker.pathLabel = UI.CreateFrame("Text", "PathLabel", txtPicker.textFrame)
	txtPicker.pathLabel:SetPoint("BOTTOMLEFT", txtPicker.okButton, "TOPLEFT", 0, -yMargin)
	txtPicker.pathLabel:SetText("Texture Name:")
	txtPicker.pathLabel:SetText("Texture Path:")

	txtPicker.pathText = UI.CreateFrame("RiftTextfield", "PathText", txtPicker.textFrame)
	txtPicker.pathText:SetPoint("BOTTOMRIGHT", txtPicker.cancelButton, "TOPRIGHT", 0, -yMargin)
	txtPicker.pathText:SetPoint("BOTTOMLEFT", txtPicker.pathLabel, "BOTTOMRIGHT", xMargin, 0)
	txtPicker.pathText:SetPoint("TOPLEFT", txtPicker.pathLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.pathText:SetText("")

	txtPicker.nameLabel = UI.CreateFrame("Text", "NameLabel", txtPicker.textFrame)
	txtPicker.nameLabel:SetPoint("BOTTOMLEFT", txtPicker.pathLabel, "TOPLEFT", 0, -yMargin)
	txtPicker.nameLabel:SetText("Texture Name:")

	txtPicker.nameText = UI.CreateFrame("RiftTextfield", "NameText", txtPicker.textFrame)
	txtPicker.nameText:SetPoint("BOTTOMRIGHT", txtPicker.pathText, "TOPRIGHT", 0, -yMargin)
	txtPicker.nameText:SetPoint("BOTTOMLEFT", txtPicker.pathText, "TOPLEFT", 0, -yMargin)
	txtPicker.nameText:SetPoint("TOPLEFT", txtPicker.nameLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.nameText:SetText("")

	local frameMiddle = math.floor(txtPicker.textFrame:GetWidth() / 2)
	txtPicker.prevButton = UI.CreateFrame("RiftButton", "PrevButton", txtPicker.textFrame)
	txtPicker.prevButton:SetPoint("BOTTOMRIGHT", txtPicker.nameLabel, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)
	txtPicker.prevButton:SetText("<<")
	txtPicker.prevButton.Event.LeftPress = function() TexturePicker.PageTurn(txtPicker, -1) end	

	txtPicker.nextButton = UI.CreateFrame("RiftButton", "NextButton", txtPicker.textFrame)
	txtPicker.nextButton:SetPoint("BOTTOMLEFT", txtPicker.nameText, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.nextButton:SetText(">>")
	txtPicker.nextButton.Event.LeftPress = function() TexturePicker.PageTurn(txtPicker, 1) end
	
	txtPicker.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtPicker.textFrame)
	txtPicker.gridFrame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT", xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("TOPRIGHT", txtPicker.textFrame, "TOPRIGHT", -xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("BOTTOMLEFT", txtPicker.prevButton, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.gridFrame:SetPoint("BOTTOMRIGHT", txtPicker.nextButton, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)

	local margin = 4
	local textureSize = 48
	local cols = math.floor((txtPicker.gridFrame:GetWidth() + margin) / (textureSize + margin))
	local rows = math.floor((txtPicker.gridFrame:GetHeight() + margin) / (textureSize + margin))
	txtPicker.rowLayout = RowLayout.Create(txtPicker.gridFrame, rows, cols, textureSize, textureSize, margin, margin)
	
	for row = 1, rows do
		for col = 1, cols do
			local txtFrame = RowLayout.CreateFrame(txtPicker.rowLayout, row, col, "Texture")
		end
	end

	TexturePicker.Redraw(txtPicker)
	
	RowLayout.Redraw(txtPicker.rowLayout)

	return txtPicker
end

function TexturePicker.Show(txtPicker, textureArray)
	local cols = RowLayout.GetNumCols(txtPicker.rowLayout)
	local rows = RowLayout.GetNumRows(txtPicker.rowLayout)
	txtPicker.maxPage = math.ceil(#textureArray / (cols * rows))
	txtPicker.nameText:SetText("")
	txtPicker.pathText:SetText("")
	txtPicker.textureArray = textureArray
	TexturePicker.Redraw(txtPicker)
	txtPicker.textFrame:SetVisible(true)
end

function TexturePicker.WindowClose(txtPicker, okPressed)
	txtPicker.pathText:SetKeyFocus(false)
	txtPicker.nameText:SetKeyFocus(false)
	txtPicker.textFrame:SetVisible(false)
	
	if okPressed then
		txtPicker.callback(txtPicker, txtPicker.nameText:GetText(), txtPicker.pathText:GetText())
	else
		txtPicker.callback(txtPicker)
	end
end

function TexturePicker.Redraw(txtPicker)
	txtPicker.prevButton:SetEnabled(txtPicker.page ~= 1)
	txtPicker.nextButton:SetEnabled(txtPicker.page ~= txtPicker.maxPage)
	
	local rows = RowLayout.GetNumRows(txtPicker.rowLayout)
	local cols = RowLayout.GetNumCols(txtPicker.rowLayout)
	for row = 1, rows do
		for col = 1, cols do
			local txtFrame = RowLayout.GetCellFrame(txtPicker.rowLayout, row, col)
			local textureIndex = (rows * cols * (txtPicker.page - 1)) + (cols * (row - 1)) + col
			if textureIndex <= #txtPicker.textureArray then
				txtFrame:SetTexture("Rift", txtPicker.textureArray[textureIndex].path)
				txtFrame.Event.LeftClick = function() TexturePicker.ChooseTexture(txtPicker, textureIndex) end
				txtFrame:SetVisible(true)
			else
				txtFrame:SetVisible(false)
				txtFrame.Event.LeftClick = function() TexturePicker.ChooseTexture(nil, nil) end
			end
		end
	end
end

function TexturePicker.ChooseTexture(txtPicker, textureIndex)
	if not txtPicker then
		txtPicker.nameText:SetText("")
		txtPicker.pathText:SetText("")
	else
		txtPicker.nameText:SetText(txtPicker.textureArray[textureIndex].name)
		txtPicker.pathText:SetText(txtPicker.textureArray[textureIndex].path)
	end
end

function TexturePicker.PageTurn(txtPicker, pageIncrement)
	txtPicker.page = txtPicker.page + pageIncrement
	if txtPicker.page < 1 then
		txtPicker.page = 1
	end
	if txtPicker.page > txtPicker.maxPage then
		txtPicker.page = txtPicker.maxPage
	end
	
	TexturePicker.Redraw(txtPicker)
end
