-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextureWarning = {}
TextureWarning.name = "TextureWarning"
TextureWarning.version = "v0.1"

--
-- Main functions
--

function TextureWarning.Create(name, parentFrame, addon, icon, xPct, yPct, scale, duration, alpha, flashFrequency, flashAlpha)
	local txtWarn = {}
	txtWarn.name = name
	txtWarn.addon = addon
	txtWarn.icon = icon
	txtWarn.xPct = xPct
	txtWarn.yPct = yPct
	txtWarn.fontSize = fontSize
	txtWarn.scale = Utilities.GetValue(scale, 100)
	txtWarn.duration = duration
	txtWarn.alpha = alpha
	txtWarn.flashFrequency = flashFrequency
	txtWarn.flashAlpha = flashAlpha
	txtWarn.flashTimer = nil
	txtWarn.hideTimer = nil
	txtWarn.delayTimer = nil
	txtWarn.countdownFrame = nil
	txtWarn.countdownTimer = nil

	txtWarn.frame = UI.CreateFrame("Texture", "TextureWarning", parentFrame)
	txtWarn.frame:SetVisible(false)
	TextureWarning.SetTexture(txtWarn, txtWarn.addon, txtWarn.icon)
	
	return txtWarn
end

function TextureWarning.SetTexture(txtWarn, addon, icon)
	txtWarn.addon = addon
	txtWarn.icon = icon
	txtWarn.frame:SetTexture(addon, icon)
	local desiredHeight = txtWarn.frame:GetTextureHeight()
	local desiredWidth = txtWarn.frame:GetTextureWidth()
	if txtWarn.scale and txtWarn.scale > 0 then
		desiredHeight = math.floor(desiredHeight * txtWarn.scale / 100)
		desiredWidth = math.floor(desiredWidth * txtWarn.scale / 100)
	end

	txtWarn.frame:SetHeight(desiredHeight)
	txtWarn.frame:SetWidth(desiredWidth)
	TextureWarning.SetPosition(txtWarn)
end

function TextureWarning.SetCountdown(txtWarn, fontColour, highlightDuration, highlightColour)
	if txtWarn.countdownTimer then
		Timer.RemoveCallback(txtWarn.countdownTimer)
		txtWarn.countdownTimer = nil
	end
	
	txtWarn.countdownFrame = UI.CreateFrame("Text", "TextureCountdown", txtWarn.frame)
	txtWarn.countdownFrame:SetFontSize(math.floor(txtWarn.frame:GetWidth() / 2))
	txtWarn.countdownFrame:SetPoint("CENTER", txtWarn.frame, "CENTER")
	txtWarn.countdownFontColour = fontColour
	txtWarn.countdownHighlightDuration = highlightDuration
	txtWarn.countdownHighlightColour = highlightColour
end

function TextureWarning.GetPosition(txtWarn)
	return txtWarn.xPct, txtWarn.yPct
end

function TextureWarning.SetPosition(txtWarn)
	local width = txtWarn.frame:GetWidth()
	local height = txtWarn.frame:GetHeight()
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	
	local xPct = txtWarn.xPct
	if xPct < 0 then
		xPct = 0
	end
	if xPct > 100 then
		xPct = 100
	end
	
	local yPct = txtWarn.yPct
	if yPct < 0 then
		yPct = 0
	end
	if yPct > 100 then
		yPct = 100
	end
	
	local x = math.floor(totalWidth * xPct / 100 - width / 2)
	if x < 0 then
		x = 0
	end
	if x + width > totalWidth then
		x = totalWidth - width
	end
	
	local y = math.floor(totalHeight * yPct / 100 - height / 2)
	if y < 0 then
		y = 0
	end
	if y + height > totalHeight then
		y = totalHeight - height
	end
	
	txtWarn.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
end

function TextureWarning.Show(txtWarn)
	if txtWarn.duration > 0 then
		if txtWarn.hideTimer then
			Timer.RemoveCallback(txtWarn.hideTimer)
			txtWarn.hideTimer = nil
		end
		
		txtWarn.hideTimer = Timer.AddCallback("TextureWarnHide-" .. txtWarn.name, txtWarn.duration, function() TextureWarning.Hide(txtWarn) end)
	end

	if txtWarn.flashFrequency > 0 then
		if txtWarn.flashTimer then
			Timer.RemoveCallback(txtWarn.flashTimer)
			txtWarn.flashTimer = nil
		end
				
		local initialFlashDuration = txtWarn.flashFrequency - Timer.GetCurrentTime() % txtWarn.flashFrequency
		if initialFlashDuration <= 0 then
			txtWarn.flashTimer = Timer.AddCallback("TextureWarnFlash-" .. txtWarn.name, txtWarn.flashFrequency, function(timeVal) TextureWarning.FlashCallback(txtWarn, timeVal) end, true)
		else
			txtWarn.flashTimer = Timer.AddCallback("TextureWarnInitFlash-" .. txtWarn.name, initialFlashDuration, function(timeVal) TextureWarning.InitialFlashCallback(txtWarn, timeVal) end, false)
		end
	end
	
	if txtWarn.countdownFrame then
		if txtWarn.countdownTimer then
			Timer.RemoveCallback(txtWarn.countdownTimer)
			txtWarn.countdownTimer = nil
		end
		
		txtWarn.countdownTimer = Timer.AddCallback("TextureWarnCountdown-" .. txtWarn.name, 1, function() TextureWarning.CountdownCallback(txtWarn) end, true)
		txtWarn.countdownValue = txtWarn.duration
		TextureWarning.CountdownCallback(txtWarn)
	end
	
	txtWarn.frame:SetAlpha(txtWarn.alpha)
	txtWarn.frame:SetVisible(true)
end

function TextureWarning.Hide(txtWarn)
	if txtWarn.flashTimer then
		Timer.RemoveCallback(txtWarn.flashTimer)
		txtWarn.flashTimer = nil
	end
	
	if txtWarn.flashTimer then
		Timer.RemoveCallback(txtWarn.flashTimer)
		txtWarn.flashTimer = nil
	end
	
	if txtWarn.countdownTimer then
		Timer.RemoveCallback(txtWarn.countdownTimer)
		txtWarn.countdownTimer = nil
	end

	txtWarn.frame:SetVisible(false)
end

function TextureWarning.SetVisible(txtWarn, visible)
	if visible == true then
		TextureWarning.Show(txtWarn)
	else
		TextureWarning.Hide(txtWarn)
	end
end

function TextureWarning.Dragged(txtWarn, xIn, yIn)
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	local widthOffset = txtWarn.frame:GetWidth() / (2 * totalWidth)
	local heightOffset = txtWarn.frame:GetHeight() / (2 * totalHeight)
	local x = Utilities.Round((xIn / UIParent:GetWidth() + widthOffset) * 100, 2)
	local y = Utilities.Round((yIn / UIParent:GetHeight() + heightOffset) * 100, 2)
	if x < 0 then x = 0 end
	if x > 100 then x = 100 end
	if y < 0 then y = 0 end
	if y > 100 then y = 100 end
	txtWarn.xPct = x
	txtWarn.yPct = y
	TextureWarning.SetPosition(txtWarn)
end

function TextureWarning.SetDragWindowVisible(txtWarn, visible)
	if txtWarn.countdownFrame then
		txtWarn.countdownFrame:SetVisible(false)
	end
	
	if visible == true then
		if not txtWarn.dragFrame then
			txtWarn.dragFrame = DragFrame.Create(txtWarn.frame, txtWarn.frame:GetWidth(), txtWarn.frame:GetHeight(), function(dragFrame) TextureWarning.Dragged(txtWarn, dragFrame.x, dragFrame.y) end)
		end
		txtWarn.frame:SetVisible(true)
	else
		txtWarn.frame:SetVisible(false)
	end
end


--
-- Event Handlers
--

function TextureWarning.InitialFlashCallback(txtWarn, timeVal)
	if txtWarn.flashTimer then
		Timer.RemoveCallback(txtWarn.flashTimer)
		txtWarn.flashTimer = nil
	end
	txtWarn.flashTimer = Timer.AddCallback("TextureWarnFlash-" .. txtWarn.name, txtWarn.flashFrequency, function(timeVal) TextureWarning.FlashCallback(txtWarn, timeVal) end, true)
	
	TextureWarning.FlashCallback(txtWarn, timeVal)
end

function TextureWarning.FlashCallback(txtWarn, timeVal)
	local isFlashing = math.floor(timeVal / txtWarn.flashFrequency) % 2
	if isFlashing == 0 then
		if txtWarn.alpha == 0 then
			txtWarn.frame:SetVisible(false)
		else
			txtWarn.frame:SetVisible(true)
		end
		txtWarn.frame:SetAlpha(txtWarn.alpha)
	else
		if txtWarn.flashAlpha == 0 then
			txtWarn.frame:SetVisible(false)
		else
			txtWarn.frame:SetVisible(true)
		end
		txtWarn.frame:SetAlpha(txtWarn.flashAlpha)
	end
end

function TextureWarning.CountdownCallback(txtWarn)
	if txtWarn.countdownValue < 1 then
		if txtWarn.countdownTimer then
			Timer.RemoveCallback(txtWarn.countdownTimer)
			txtWarn.countdownTimer = nil
		end
		
		txtWarn.countdownFrame:SetVisible(false)
	else
		txtWarn.countdownFrame:SetText(tostring(txtWarn.countdownValue))
		if txtWarn.countdownValue > txtWarn.countdownHighlightDuration then
			txtWarn.countdownFrame:SetFontColor(txtWarn.countdownFontColour.red, txtWarn.countdownFontColour.green, txtWarn.countdownFontColour.blue)
		else
			txtWarn.countdownFrame:SetFontColor(txtWarn.countdownHighlightColour.red, txtWarn.countdownHighlightColour.green, txtWarn.countdownHighlightColour.blue)
		end
		txtWarn.countdownFrame:SetVisible(true)
		txtWarn.countdownValue = txtWarn.countdownValue - 1
	end
end

--
-- Register events
--

