-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TextureWarningConfig = {}
TextureWarningConfig.name = "TextureWarningConfig"
TextureWarningConfig.version = "$Id$"

--
-- Main functions
--
local function AbilityDetail(abilityID)
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.Detail(abilityID)
	else
		return Inspect.Ability.Detail(abilityID)
	end
end

local function AbilityList()
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.List()
	else
		return Inspect.Ability.List()
	end
end

--
-- UI functions
--

function TextureWarningConfig.Create(parentFrame, callback)
	local txtWarnConfig = {}
	txtWarnConfig.parentFrame = parentFrame
	txtWarnConfig.callback = callback
	txtWarnConfig.warning = TextureWarningConfig.GetDefaultWarning()
	
	txtWarnConfig.windowFrame = UI.CreateFrame("RiftWindow", "TxtWarningSelector", parentFrame)
	txtWarnConfig.windowFrame:SetTitle("Texture Warning Editor")
	txtWarnConfig.windowFrame:SetWidth(720)
	txtWarnConfig.windowFrame:SetHeight(720)
	txtWarnConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	txtWarnConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", txtWarnConfig.windowFrame)
	txtWarnConfig.okButton:SetPoint("BOTTOMLEFT", txtWarnConfig.windowFrame, "BOTTOMLEFT", 20, -20)
	txtWarnConfig.okButton:SetText("OK")
	txtWarnConfig.okButton.Event.LeftPress = function() TextureWarningConfig.WindowClose(txtWarnConfig, true) end	

	txtWarnConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtWarnConfig.windowFrame)
	txtWarnConfig.cancelButton:SetPoint("BOTTOMRIGHT", txtWarnConfig.windowFrame, "BOTTOMRIGHT", -20, -20)
	txtWarnConfig.cancelButton:SetText("Cancel")
	txtWarnConfig.cancelButton.Event.LeftPress = function() TextureWarningConfig.WindowClose(txtWarnConfig, false) end	

	local margin = 4
	local height = 40
	txtWarnConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtWarnConfig.windowFrame)
	txtWarnConfig.gridFrame:SetPoint("TOPLEFT", txtWarnConfig.windowFrame, "TOPLEFT", 40, 80)
	txtWarnConfig.rowLayout = RowLayout.Create(txtWarnConfig.gridFrame, 13, 3, height, 350, margin, margin)
	RowLayout.SetColWidth(txtWarnConfig.rowLayout, 1, 150)

	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the warning must be unique")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Addon")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The addon holding the texture.  Rift for game textures")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Texture")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("x%,y%")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Texture scaling%")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 5, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The size of the texture.  100 is normal size")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Display duration (secs)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 6, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Duration of 0 means show until event ends")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("Alpha Level (0-1)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 7, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Sets the transparency of the warning")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 8, 1, "Text")
	txtFrame:SetText("Flash frequency (secs)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 8, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("How often the warning flashes, 0 means never")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 9, 1, "Text")
	txtFrame:SetText("Flash Alpha level")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 9, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Set the transparency of the flashed text")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 1, "Text")
	txtFrame:SetText("Countdown")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Shows a numerical countdown on texture")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 11, 1, "Text")
	txtFrame:SetText("Countdown colour")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 12, 1, "Text")
	txtFrame:SetText("Countdown Highlight (secs)")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 12, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Shows the countdown in highlight colour")
	local txtFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 13, 1, "Text")
	txtFrame:SetText("Highlight colour")

	txtWarnConfig.nameFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 1, 2, "RiftTextfield")
	txtWarnConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	txtWarnConfig.addonFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 2, 2, "RiftTextfield")
	txtWarnConfig.addonFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	txtWarnConfig.iconFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 2, "RiftTextfield")
	txtWarnConfig.iconFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	txtWarnConfig.countdownFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 10, 2, "RiftCheckbox", RowLayout.centerFrame)

	txtWarnConfig.colourFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 11, 2, "Frame")
	txtWarnConfig.highlightColourFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 13, 2, "Frame")
	
	local xyFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 4, 2, "Frame")
	local xyFrame1 = UI.CreateFrame("Frame", "xyFrame1", xyFrame)
	xyFrame1:SetPoint("TOPLEFT", xyFrame, "TOPLEFT")
	xyFrame1:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMLEFT", math.floor((RowLayout.GetColWidth(txtWarnConfig.rowLayout, 2) - margin) / 2), 0)	
	txtWarnConfig.xSlider = NumberSlider.Create(xyFrame1, 0, 100, 0.01, 0)
	local xyFrame2 = UI.CreateFrame("Frame", "xyFrame2", xyFrame)
	xyFrame2:SetPoint("TOPLEFT", xyFrame1, "TOPRIGHT", margin, 0)
	xyFrame2:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMRIGHT")
	txtWarnConfig.ySlider = NumberSlider.Create(xyFrame2, 0, 100, 0.01, 0)

	txtWarnConfig.scaleSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 5, 2, 10, 1000, 10)
	txtWarnConfig.durationSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 6, 2, 0, 300, 1)
	txtWarnConfig.alphaSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 7, 2, 0, 1, 0.05)
	txtWarnConfig.flashDurationSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 8, 2, 0, 30, 1)
	txtWarnConfig.flashAlphaSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 9, 2, 0, 1, 0.05)
	txtWarnConfig.highlightSlider = TextureWarningConfig.CreateSlider(txtWarnConfig, 12, 2, 0, 20, 1)
	
	txtWarnConfig.xyButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 4, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.xyButton:SetText("Choose")
	txtWarnConfig.xyButton.Event.LeftPress = function() TextureWarningConfig.ShowXY(txtWarnConfig) end
	txtWarnConfig.textureButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 3, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.textureButton:SetText("Choose")
	txtWarnConfig.textureButton.Event.LeftPress = function() TextureWarningConfig.ShowTextureChooser(txtWarnConfig) end
	txtWarnConfig.colourButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 11, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.colourButton:SetText("Choose")
	txtWarnConfig.colourButton.Event.LeftPress = function() TextureWarningConfig.ShowChooseColour(txtWarnConfig) end
	txtWarnConfig.flashColourButton = RowLayout.CreateFrame(txtWarnConfig.rowLayout, 13, 3, "RiftButton", RowLayout.centerFrame)
	txtWarnConfig.flashColourButton:SetText("Choose")
	txtWarnConfig.flashColourButton.Event.LeftPress = function() TextureWarningConfig.ShowChooseHighlightColour(txtWarnConfig) end
	RowLayout.SetColWidth(txtWarnConfig.rowLayout, 3, txtWarnConfig.xyButton:GetWidth())

	RowLayout.Redraw(txtWarnConfig.rowLayout)
	
	txtWarnConfig.clrChooser = ColourChooser.Create(txtWarnConfig.windowFrame, txtWarnConfig.warning.countdownFontColour)
	txtWarnConfig.xySelector = XYSelector.Create(txtWarnConfig.parentFrame, function(x,y) TextureWarningConfig.SetXY(txtWarnConfig, x, y) end)
	txtWarnConfig.txtPicker = TexturePicker.Create(txtWarnConfig.windowFrame, function(a, name, path) TextureWarningConfig.TextureChosen(txtWarnConfig, name, path) end)
	txtWarnConfig.wrnWindow = WarningWindow.CreateWarningWindow(txtWarnConfig.parentFrame, txtWarnConfig.windowFrame)
	
	TextureWarningConfig.Show(txtWarnConfig)
	txtWarnConfig.windowFrame:SetVisible(false)
	
	return txtWarnConfig
end

function TextureWarningConfig.WindowClose(txtWarnConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = txtWarnConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this warning\n"
		end
		local addon = txtWarnConfig.addonFrame:GetText()
		if not addon or string.len(addon) < 1 then
			message = message .. "You must specify an addon string for this warning\n"
		end
		local icon = txtWarnConfig.iconFrame:GetText()
		if not icon or string.len(icon) < 1 then
			message = message .. "You must specify a texture string for this warning\n"
		end
		
		if string.len(message) > 0 then
			TextureWarningConfig.SetEnabled(txtWarnConfig, false)
			WarningWindow.ShowWarningWindow(txtWarnConfig.wrnWindow, message, function() TextureWarningConfig.SetEnabled(txtWarnConfig, true) end)
			return
		end
		
		TextureWarningConfig.FillWarning(txtWarnConfig)
	end
	
	txtWarnConfig.nameFrame:SetKeyFocus(false)
	txtWarnConfig.addonFrame:SetKeyFocus(false)
	txtWarnConfig.iconFrame:SetKeyFocus(false)
	
	txtWarnConfig.windowFrame:SetVisible(false)
	if txtWarnConfig.callback then
		if okPressed == true then
			txtWarnConfig.callback(txtWarnConfig, txtWarnConfig.warning)
		else
			txtWarnConfig.callback(txtWarnConfig)
		end
	end
end

function TextureWarningConfig.Show(txtWarnConfig, inWarning)
	if inWarning then
		txtWarnConfig.warning = TextureWarningConfig.CopyWarning(inWarning)
	end
	
	local warning = txtWarnConfig.warning
	if warning.countdownDuration and warning.countdownDuration > 0 then
		txtWarnConfig.countdownFrame:SetChecked(true)
		NumberSlider.SetValue(txtWarnConfig.highlightSlider, warning.countdownHighlightDuration)
	else
		txtWarnConfig.countdownFrame:SetChecked(false)
	end
	txtWarnConfig.colourFrame:SetBackgroundColor(warning.countdownFontColour.red, warning.countdownFontColour.green, warning.countdownFontColour.blue)
	txtWarnConfig.highlightColourFrame:SetBackgroundColor(warning.countdownHighlightColour.red, warning.countdownHighlightColour.green, warning.countdownHighlightColour.blue)
	
	txtWarnConfig.nameFrame:SetText(warning.name)
	txtWarnConfig.addonFrame:SetText(warning.addon)
	txtWarnConfig.iconFrame:SetText(warning.icon)
	NumberSlider.SetValue(txtWarnConfig.xSlider, warning.xPct)
	NumberSlider.SetValue(txtWarnConfig.ySlider, warning.yPct)
	NumberSlider.SetValue(txtWarnConfig.scaleSlider, warning.scale)
	NumberSlider.SetValue(txtWarnConfig.durationSlider, warning.duration)
	NumberSlider.SetValue(txtWarnConfig.alphaSlider, warning.alpha)
	NumberSlider.SetValue(txtWarnConfig.flashDurationSlider, warning.flashFrequency)
	NumberSlider.SetValue(txtWarnConfig.flashAlphaSlider, warning.flashAlpha)
	txtWarnConfig.windowFrame:SetVisible(true)
end

function TextureWarningConfig.FillWarning(txtWarnConfig)
	txtWarnConfig.warning.name = txtWarnConfig.nameFrame:GetText()
	txtWarnConfig.warning.addon = txtWarnConfig.addonFrame:GetText()
	txtWarnConfig.warning.icon = txtWarnConfig.iconFrame:GetText()
	txtWarnConfig.warning.xPct = NumberSlider.GetValue(txtWarnConfig.xSlider)
	txtWarnConfig.warning.yPct = NumberSlider.GetValue(txtWarnConfig.ySlider)
	txtWarnConfig.warning.scale = NumberSlider.GetValue(txtWarnConfig.scaleSlider)
	txtWarnConfig.warning.duration = NumberSlider.GetValue(txtWarnConfig.durationSlider)
	txtWarnConfig.warning.alpha = NumberSlider.GetValue(txtWarnConfig.alphaSlider)
	txtWarnConfig.warning.flashFrequency = NumberSlider.GetValue(txtWarnConfig.flashDurationSlider)
	txtWarnConfig.warning.flashAlpha = NumberSlider.GetValue(txtWarnConfig.flashAlphaSlider)
	if txtWarnConfig.countdownFrame:GetChecked() == true then
		txtWarnConfig.warning.countdownDuration = txtWarnConfig.warning.duration
		txtWarnConfig.warning.countdownHighlightDuration = NumberSlider.GetValue(txtWarnConfig.highlightSlider)
	else
		txtWarnConfig.warning.countdownDuration = 0
		txtWarnConfig.warning.countdownHighlightDuration = 0
	end
end

function TextureWarningConfig.SetEnabled(txtWarnConfig, enabled)
	--txtWarnConfig.nameFrame:SetEnabled(enabled)
	--txtWarnConfig.strFrame:SetEnabled(enabled)
	txtWarnConfig.okButton:SetEnabled(enabled)
	txtWarnConfig.cancelButton:SetEnabled(enabled)
	txtWarnConfig.xyButton:SetEnabled(enabled)
	txtWarnConfig.textureButton:SetEnabled(enabled)
	NumberSlider.SetEnabled(txtWarnConfig.xSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.ySlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.scaleSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.durationSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.alphaSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.flashDurationSlider, enabled)
	NumberSlider.SetEnabled(txtWarnConfig.flashAlphaSlider, enabled)
end

function TextureWarningConfig.CreateSlider(txtWarnConfig, row, col, minValue, maxValue, stepSize)
	local xyFrame = RowLayout.CreateFrame(txtWarnConfig.rowLayout, row, col, "Frame")
	return NumberSlider.Create(xyFrame, minValue, maxValue, stepSize, minValue)
end

function TextureWarningConfig.ShowXY(txtWarnConfig)
	txtWarnConfig.windowFrame:SetVisible(false)
	XYSelector.SetVisible(txtWarnConfig.xySelector, true)
end

function TextureWarningConfig.SetXY(txtWarnConfig, x, y)
	txtWarnConfig.windowFrame:SetVisible(true)
	TextureWarningConfig.FillWarning(txtWarnConfig)
	txtWarnConfig.warning.xPct = x
	txtWarnConfig.warning.yPct = y
	TextureWarningConfig.Show(txtWarnConfig)
end

function TextureWarningConfig.CopyWarning(warning2)
	local warning = {}
	warning.warningType = WarnMe.warningTypeTexture
	warning.name = warning2.name
	warning.str = warning2.str
	warning.xPct = warning2.xPct
	warning.yPct = warning2.yPct
	warning.addon = warning2.addon
	warning.icon = warning2.icon
	warning.scale = Utilities.GetValue(warning2.scale, 100)
	warning.duration = warning2.duration
	warning.alpha = warning2.alpha
	warning.flashFrequency = warning2.flashFrequency
	warning.flashAlpha = warning2.flashAlpha
	if warning2.countdownDuration and warning2.countdownDuration > 0 then
		warning.countdownDuration = warning2.countdownDuration
		warning.countdownFontColour = warning2.countdownFontColour
		warning.countdownHighlightDuration = warning2.countdownHighlightDuration
		warning.countdownHighlightColour = warning2.countdownHighlightColour
	else
		warning.countdownDuration = 0
		warning.countdownFontColour = { red = 1, green = 1, blue = 1 }
		warning.countdownHighlightDuration = 0
		warning.countdownHighlightColour = { red = 1, green = 0, blue = 0 }
	end
	
	return warning
end

function TextureWarningConfig.GetDefaultWarning()
	local warning = {}
	warning.warningType = WarnMe.warningTypeTexture
	warning.name = ""
	warning.str = ""
	warning.xPct = 50
	warning.yPct = 50
	warning.addon = "Rift"
	warning.icon = ""
	warning.scale = 100
	warning.duration = 5
	warning.alpha = 1
	warning.flashFrequency = 0
	warning.flashAlpha = 0.2
	warning.countdownDuration = 0
	warning.countdownFontColour = { red = 1, green = 1, blue = 1 }
	warning.countdownHighlightDuration = 0
	warning.countdownHighlightColour = { red = 1, green = 0, blue = 0 }
	
	return warning
end

function TextureWarningConfig.TextureChosen(txtWarnConfig, name, path)
	TextureWarningConfig.SetEnabled(txtWarnConfig, true)
	TextureWarningConfig.FillWarning(txtWarnConfig)
	if name and path then
		if not txtWarnConfig.warning.name or string.len(txtWarnConfig.warning.name) < 1 then
			txtWarnConfig.warning.name = name
		end
		txtWarnConfig.warning.addon = "Rift"
		txtWarnConfig.warning.icon = path
	end
	TextureWarningConfig.Show(txtWarnConfig)
end

function TextureWarningConfig.ShowTextureChooser(txtWarnConfig)
	local playerTextures = TextureWarningConfig.GetPlayerTextures()
	local mergedTexures = TextureWarningConfig.MergeArrays(playerTextures)
	TextureWarningConfig.SetEnabled(txtWarnConfig, false)
	TexturePicker.Show(txtWarnConfig.txtPicker, mergedTexures)
end

function TextureWarningConfig.GetPlayerTextures()
	local playerID = Inspect.Unit.Lookup("player")
	local buffDetails = Inspect.Buff.Detail(playerID, Inspect.Buff.List(playerID))
	
	local textures = {}
	for _, buffDetail in pairs(buffDetails) do
		textures[buffDetail.name] = buffDetails.icon
	end
	
	local abilityDetails = AbilityDetail(AbilityList())
	for _, abilityDetail in pairs(abilityDetails) do
		textures[abilityDetail.name] = abilityDetail.icon
	end
	
	return textures
end

function TextureWarningConfig.MergeArrays(inArray)
	local textureArray = {}

	local sortedNames = {}
	for name, _ in pairs(inArray) do
		table.insert(sortedNames, name)
	end
	
	local textureByName = TextureList.GetAllTextureNames()
	for name, _ in pairs(textureByName) do
		if not inArray[name] then
			table.insert(sortedNames, name)
		end
	end

	table.sort(sortedNames)

	for _, name in ipairs(sortedNames) do
		local textureEntry = {}
		textureEntry.name = name
		if inArray[name] then
			textureEntry.path = inArray[name]
		else
			textureEntry.path = textureByName[name]
		end
		table.insert(textureArray, textureEntry)
	end
	
	return textureArray
end

function TextureWarningConfig.ShowChooseColour(txtWarnConfig)
	TextureWarningConfig.SetEnabled(txtWarnConfig, false)
	ColourChooser.Show(txtWarnConfig.clrChooser, txtWarnConfig.warning.countdownFontColour, function(x, colour) TextureWarningConfig.ChooseColour(txtWarnConfig, colour) end)
end

function TextureWarningConfig.ChooseColour(txtWarnConfig, colour)
	TextureWarningConfig.SetEnabled(txtWarnConfig, true)
	if colour then
		txtWarnConfig.warning.countdownFontColour = colour
		TextureWarningConfig.FillWarning(txtWarnConfig)
		TextureWarningConfig.Show(txtWarnConfig)
	end
end

function TextureWarningConfig.ShowChooseHighlightColour(txtWarnConfig)
	TextureWarningConfig.SetEnabled(txtWarnConfig, false)
	ColourChooser.Show(txtWarnConfig.clrChooser, txtWarnConfig.warning.countdownHighlightColour, function(x, colour) TextureWarningConfig.ChooseHightlightColour(txtWarnConfig, colour) end)
end

function TextureWarningConfig.ChooseHightlightColour(txtWarnConfig, colour)
	TextureWarningConfig.SetEnabled(txtWarnConfig, true)
	if colour then
		txtWarnConfig.warning.countdownHighlightColour = colour
		TextureWarningConfig.FillWarning(txtWarnConfig)
		TextureWarningConfig.Show(txtWarnConfig)
	end
end

