-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

Timer = {}
Timer.name = "Timer"
Timer.version = "v0.1"
Timer.eventsByTime = {}
Timer.eventsByID = {}
Timer.eventID = 0
Timer.currentTime = 0

--
-- Main functions
--

function Timer.Initialise(addonName)
	table.insert(Event.System.Update.Begin, { Timer.Refresh, addonName, "Timer" })
end

function Timer.Clear()
	Timer.eventsByTime = {}
	Timer.eventsByID = {}
end

function Timer.AddCallback(name, duration, callback, repeatIt)
	if Timer.currentTime == 0 then
		Timer.currentTime = Timer.InternalGetTime()
	end
	
	local eventID = Timer.InternalGetID()
	Timer.InternalAdd(eventID, name, duration, callback, repeatIt)
	return eventID
end

function Timer.RemoveCallback(timerId)
	local thisTimer = Timer.eventsByID[timerId]
	if thisTimer then
		local dueTimeKey = Timer.GetTimeKey(thisTimer.dueTime)
		Timer.eventsByID[timerId] = nil
		Timer.eventsByTime[dueTimeKey][timerId] = nil
		
		local deleteTable = true
		for i, j in pairs(Timer.eventsByTime[dueTimeKey]) do
			deleteTable = false
		end
		
		if deleteTable == true then
			Timer.eventsByTime[dueTimeKey] = nil
		end
	end
end

function Timer.InternalAdd(id, name, duration, callback, repeatIt)
	local dueTime = Timer.currentTime + (duration * 10)
	
	local thisTimer = {}
	thisTimer.id = id
	thisTimer.name = name
	thisTimer.duration = duration
	thisTimer.callback = callback
	thisTimer.dueTime = dueTime
	if (repeatIt == true) then
		thisTimer.repeatTimer = true
	else
		thisTimer.repeatTimer = false
	end
	
	local dueTimeKey = Timer.GetTimeKey(dueTime)
	if not Timer.eventsByTime[dueTimeKey] then
		Timer.eventsByTime[dueTimeKey] = {}
	end

	Timer.eventsByTime[dueTimeKey][id] = thisTimer
	Timer.eventsByID[id] = thisTimer
end

function Timer.InternalGetTime()
	return math.floor(Inspect.Time.Frame() * 10)
end

function Timer.GetTime(currentTime)
	return currentTime / 10
end

function Timer.GetCurrentTime()
	return Timer.GetTime(Timer.currentTime)
end

function Timer.InternalGetID()
	Timer.eventID = Timer.eventID + 1
	return "#" .. Timer.eventID
end

function Timer.GetTimeKey(thisTime)
	return "@" .. thisTime
end


--
-- Event Handlers
--

function Timer.Refresh()
	local newTime = Timer.InternalGetTime()
	
	if newTime ~= Timer.currentTime then
		local nextTimerTime = Timer.currentTime + 1
		for currentTime = nextTimerTime, newTime do
			Timer.currentTime = currentTime
			local timeKey = Timer.GetTimeKey(Timer.currentTime)
			if Timer.eventsByTime[timeKey] then
				for id, thisTimer in pairs(Timer.eventsByTime[timeKey]) do
					if thisTimer.repeatTimer == true then
						Timer.InternalAdd(thisTimer.id, thisTimer.name, thisTimer.duration, thisTimer.callback, thisTimer.repeatTimer)
					else
						Timer.eventsByID[thisTimer.id] = nil
					end

					thisTimer.callback(Timer.GetTime(currentTime))
				end

				Timer.eventsByTime[timeKey] = nil
			end
		end
	end
end

