-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- This file provide a numeric label coupled with a RiftSlider to allow a user to select a number
-- within a given range
--

--
-- Global variables
--

TimerSlider = {}
TimerSlider.version = "$Id$"
TimerSlider.name = "TimerSlider"
TimerSlider.CountdownTemplate = "{Countdown}"

--
-- Main functions
--

function TimerSlider.Create(parentFrame, name, duration, frequency, x, y, width, fontSize, colour, fontColour, text)
	if not(frequency == 2 or frequency == 5) then
		print(string.format("%s: frequency must be 2 or 5", TimerSlider.name))
		return
	end

	local timerSlider = {}
	timerSlider.parentFrame = parentFrame
	timerSlider.name = name
	timerSlider.duration = duration
	timerSlider.frequency = frequency
	timerSlider.xPct = x
	timerSlider.yPct = y
	timerSlider.width = width
	timerSlider.fontSize = fontSize
	timerSlider.colour = colour
	timerSlider.fontColour = fontColour
	
	local position = string.find(text, TimerSlider.CountdownTemplate)
	if position then
		timerSlider.countdownPrefix = string.sub(text, 1, position - 1)
		timerSlider.countdownSuffix = string.sub(text, position + string.len(TimerSlider.CountdownTemplate))
		timerSlider.isCountdown = true
		timerSlider.countdownValue = duration
	else
		timerSlider.text = text
	end

	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	local xPos = math.floor(totalWidth * x / 100)
	local yPos = math.floor(totalHeight * y / 100)
	local widthVal = math.floor(totalWidth * width / 100)
	local margin = 3
	timerSlider.margin = margin
	
	timerSlider.windowFrame = UI.CreateFrame("Frame", "TimerSliderBackground", parentFrame)
	timerSlider.windowFrame:SetVisible(false)
	timerSlider.topBorderFrame = UI.CreateFrame("Frame", "TimerSliderTopBorder", timerSlider.windowFrame)
	timerSlider.leftBorderFrame = UI.CreateFrame("Frame", "TimerSliderTopLeft", timerSlider.windowFrame)
	timerSlider.rightBorderFrame = UI.CreateFrame("Frame", "TimerSliderTopRight", timerSlider.windowFrame)
	timerSlider.bottomBorderFrame = UI.CreateFrame("Frame", "TimerSliderTopBottom", timerSlider.windowFrame)
	timerSlider.innerFrame = UI.CreateFrame("Frame", "TimerSliderInner", timerSlider.windowFrame)
	timerSlider.textFrame = UI.CreateFrame("Text", "TimerSliderText", timerSlider.windowFrame)

	timerSlider.textFrame:SetFontSize(fontSize)
	timerSlider.textFrame:SetFontColor(fontColour.red, fontColour.green, fontColour.blue)
	timerSlider.textFrame:SetText(text)
	timerSlider.textFrame:SetLayer(timerSlider.innerFrame:GetLayer() + 1)
	local heightVal = timerSlider.textFrame:GetHeight()
	
	TimerSlider.SetPosition(timerSlider, x, y)
	timerSlider.windowFrame:SetWidth(widthVal)
	timerSlider.windowFrame:SetHeight(heightVal + margin * 4)

	timerSlider.topBorderFrame:SetPoint("TOPLEFT", timerSlider.windowFrame, "TOPLEFT")
	timerSlider.topBorderFrame:SetPoint("BOTTOMRIGHT", timerSlider.windowFrame, "TOPRIGHT", 0, 1)
	timerSlider.leftBorderFrame:SetPoint("TOPLEFT", timerSlider.windowFrame, "TOPLEFT")
	timerSlider.leftBorderFrame:SetPoint("BOTTOMRIGHT", timerSlider.windowFrame, "BOTTOMLEFT", 1, 0)
	timerSlider.rightBorderFrame:SetPoint("TOPRIGHT", timerSlider.windowFrame, "TOPRIGHT")
	timerSlider.rightBorderFrame:SetPoint("BOTTOMLEFT", timerSlider.windowFrame, "BOTTOMRIGHT", -1, 0)
	timerSlider.bottomBorderFrame:SetPoint("TOPLEFT", timerSlider.windowFrame, "BOTTOMLEFT", 0, -1)
	timerSlider.bottomBorderFrame:SetPoint("BOTTOMRIGHT", timerSlider.windowFrame, "BOTTOMRIGHT")

	timerSlider.innerFrame:SetPoint("TOPLEFT", timerSlider.windowFrame, "TOPLEFT", margin, margin)
	timerSlider.innerFrame:SetHeight(heightVal + margin * 2)
	timerSlider.innerFrame:SetWidth(1)

	timerSlider.topBorderFrame:SetBackgroundColor(colour.red, colour.green, colour.blue)
	timerSlider.leftBorderFrame:SetBackgroundColor(colour.red, colour.green, colour.blue)
	timerSlider.rightBorderFrame:SetBackgroundColor(colour.red, colour.green, colour.blue)
	timerSlider.bottomBorderFrame:SetBackgroundColor(colour.red, colour.green, colour.blue)
	timerSlider.innerFrame:SetBackgroundColor(colour.red, colour.green, colour.blue)
	
	timerSlider.textFrame:SetPoint("TOPLEFT", timerSlider.windowFrame, "TOPLEFT", margin * 2, margin * 2)

	return timerSlider
end

function TimerSlider.GetPosition(timerSlider)
	return timerSlider.xPct, timerSlider.yPct
end

function TimerSlider.SetPosition(timerSlider)
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	local xPos = math.floor(totalWidth * timerSlider.xPct / 100)
	local yPos = math.floor(totalHeight * timerSlider.yPct / 100)
	timerSlider.windowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", xPos, yPos)
end

function TimerSlider.SetVisible(timerSlider, visible, duration)
	if not timerSlider then
		print(string.format("%s: cannot SetVisible on a nil slider", TimerSlider.name))
		return
	end
	
	if duration then
		timerSlider.duration = duration
	end
	
	if timerSlider.barTimer then
		Timer.RemoveCallback(timerSlider.barTimer)
		timerSlider.barTimer = nil
	end
	
	if timerSlider.textTimer then
		Timer.RemoveCallback(timerSlider.textTimer)
		timerSlider.textTimer = nil
	end

	if visible == true then
		timerSlider.countdownValue = timerSlider.duration
		timerSlider.updateCount = timerSlider.frequency
		
		TimerSlider.RedrawText(timerSlider)
		TimerSlider.RedrawBar(timerSlider)

		if timerSlider.duration > 0 then
			timerSlider.textTimer = Timer.AddCallback("TimerSliderTextCountdown" .. timerSlider.name, 1, function() TimerSlider.TextCallback(timerSlider) end, true)
			timerSlider.barTimer = Timer.AddCallback("TimerSliderBarCountdown" .. timerSlider.name, 1 / timerSlider.frequency, function() TimerSlider.BarCallback(timerSlider) end, true)
		else
			timerSlider.countdownValue = nil
			timerSlider.updateCount = nil
		end
	end
	
	timerSlider.windowFrame:SetVisible(visible)
end

function TimerSlider.Dragged(timerSlider, xIn, yIn)
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	local widthOffset = timerSlider.windowFrame:GetWidth() / (2 * totalWidth)
	local heightOffset = timerSlider.windowFrame:GetHeight() / (2 * totalHeight)
	local x = Utilities.Round((xIn / UIParent:GetWidth()) * 100, 2)
	local y = Utilities.Round((yIn / UIParent:GetHeight()) * 100, 2)
	if x < 0 then x = 0 end
	if x > 100 then x = 100 end
	if y < 0 then y = 0 end
	if y > 100 then y = 100 end
	timerSlider.xPct = x
	timerSlider.yPct = y
	TimerSlider.SetPosition(timerSlider)
end

function TimerSlider.SetDragWindowVisible(timerSlider, visible)
	if visible == true then
		if not timerSlider.dragFrame then
			timerSlider.dragFrame = DragFrame.Create(timerSlider.windowFrame, timerSlider.windowFrame:GetWidth(), timerSlider.windowFrame:GetHeight(), function(dragFrame) TimerSlider.Dragged(timerSlider, dragFrame.x, dragFrame.y) end)
		end
		timerSlider.windowFrame:SetVisible(true)
	else
		timerSlider.windowFrame:SetVisible(false)
	end
end

function TimerSlider.RedrawText(timerSlider)
	if timerSlider.isCountdown == true then
		timerSlider.textFrame:SetText(timerSlider.countdownPrefix .. timerSlider.countdownValue .. timerSlider.countdownSuffix)
	end
end

function TimerSlider.RedrawBar(timerSlider)
	local pctNum = (timerSlider.duration * timerSlider.frequency - timerSlider.countdownValue * timerSlider.frequency + (timerSlider.frequency - timerSlider.updateCount))
	local pctDenom = (timerSlider.duration * timerSlider.frequency)
	local pctVisible = pctNum / pctDenom
	local width = Utilities.Round((timerSlider.windowFrame:GetWidth() - 2 * timerSlider.margin) * pctVisible, 0)
	timerSlider.innerFrame:SetWidth(width)

	--print(pctNum .. " / "	.. pctDenom .. " = " .. width)
end

function TimerSlider.BarCallback(timerSlider)
	if timerSlider.updateCount then
		timerSlider.updateCount = timerSlider.updateCount - 1
				
		if timerSlider.updateCount <= 1 then
			if timerSlider.barTimer then
				Timer.RemoveCallback(timerSlider.barTimer)
				timerSlider.barTimer = nil
			end
		end
		
		TimerSlider.RedrawBar(timerSlider)
		TimerSlider.RedrawText(timerSlider)
	end
end

function TimerSlider.TextCallback(timerSlider)
	timerSlider.updateCount = timerSlider.frequency

	if timerSlider.countdownValue and timerSlider.countdownValue > 0 then
		timerSlider.countdownValue = timerSlider.countdownValue - 1
		timerSlider.barTimer = Timer.AddCallback("TimerSliderBarCountdown" .. timerSlider.name, 1 / timerSlider.frequency, function() TimerSlider.BarCallback(timerSlider) end, true)
	else
		if timerSlider.textTimer then
			Timer.RemoveCallback(timerSlider.textTimer)
			timerSlider.textTimer = nil
		end
	end

	TimerSlider.RedrawBar(timerSlider)
	TimerSlider.RedrawText(timerSlider)
end

