-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TimerWarning = {}
TimerWarning.name = "TimerWarning"
TimerWarning.version = "$Id$"

--
-- Main functions
--

function TimerWarning.Create(name, parentFrame, warning, initialDelay, duration, repeats)
	local timerWarn = {}
	timerWarn.parentFrame = parentFrame
	timerWarn.name = name
	timerWarn.duration = duration
	if not initialDelay or initialDelay < 1 then
		timerWarn.initialDelay = duration
	else
		timerWarn.initialDelay = initialDelay
	end
	timerWarn.repeats = repeats
	timerWarn.internalRepeats = repeats
	timerWarn.warning = warning
	
	return timerWarn
end

function TimerWarning.SetSlider(timerWarn, x, y, width, fontSize, colour, fontColour, text)
	if timerWarn.slider then
		TimerSlider.SetVisible(timerWarn.slider, false)
	end
	
	timerWarn.slider = TimerSlider.Create(timerWarn.parentFrame, timerWarn.name, timerWarn.duration, 5, x, y, width, fontSize, colour, fontColour, text)
end

function TimerWarning.Show(timerWarn)
	timerWarn.internalRepeats = timerWarn.repeats

	if timerWarn.initialDelay and timerWarn.initialDelay > 0 then
		if timerWarn.initialTimer then
			Timer.RemoveCallback(timerWarn.initialTimer)
			timerWarn.initialTimer = nil
		end
		
		timerWarn.initialTimer = Timer.AddCallback("TextWarnInitial-" .. timerWarn.name, timerWarn.initialDelay, function() TimerWarning.InternalShow(timerWarn) end)
		
		if timerWarn.slider then
			TimerSlider.SetVisible(timerWarn.slider, true, timerWarn.initialDelay)
		end
	end
end

function TimerWarning.Hide(timerWarn)
	if timerWarn.initialTimer then
		Timer.RemoveCallback(timerWarn.initialTimer)
		timerWarn.initialTimer = nil
	end

	if timerWarn.repeatTimer then
		Timer.RemoveCallback(timerWarn.repeatTimer)
		timerWarn.repeatTimer = nil
	end

	if timerWarn.slider then
		TimerSlider.SetVisible(timerWarn.slider, false)
	end
	
	if timerWarn.warning then
		timerWarn.warning.setVisible(false)
	end
end

function TimerWarning.SetVisible(timerWarn, visible)
	if visible == true then
		TimerWarning.Show(timerWarn)
	else
		TimerWarning.Hide(timerWarn)
	end
end

function TimerWarning.GetPosition(timerWarn)
	if timerWarn.slider then
		local xPct, yPct = TimerSlider.GetPosition(timerWarn.slider)
		return xPct, yPct
	end
end

function TimerWarning.SetDragWindowVisible(timerWarn, visible)
	if timerWarn.slider then
		TimerSlider.SetDragWindowVisible(timerWarn.slider, visible)
	end
end

function TimerWarning.InternalShow(timerWarn)
	if timerWarn.repeatTimer then
		Timer.RemoveCallback(timerWarn.repeatTimer)
		timerWarn.repeatTimer = nil
	end
	
	if timerWarn.internalRepeats > 0 then
		timerWarn.internalRepeats = timerWarn.internalRepeats - 1
		timerWarn.repeatTimer = Timer.AddCallback("TimerWarnRepeat-" .. timerWarn.name, timerWarn.duration, function() TimerWarning.InternalShow(timerWarn) end)

		if timerWarn.slider then
			TimerSlider.SetVisible(timerWarn.slider, true, timerWarn.duration)
		end
	else
		if timerWarn.slider then
			TimerSlider.SetVisible(timerWarn.slider, false)
		end
	end

	if timerWarn.warning then
		timerWarn.warning.setVisible(true)	
	end
end
