-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

TimerWarningConfig = {}
TimerWarningConfig.name = "TimerWarningConfig"
TimerWarningConfig.version = "$Id$"

--
-- Main functions
--

--
-- UI functions
--

function TimerWarningConfig.Create(wmeConfig, parentFrame, callback)
	local timerWarnConfig = {}
	timerWarnConfig.parentFrame = parentFrame
	timerWarnConfig.callback = callback
	timerWarnConfig.warning = TimerWarningConfig.GetDefaultWarning()
	timerWarnConfig.txtPicker = wmeConfig.txtPicker
	
	timerWarnConfig.windowFrame = UI.CreateFrame("RiftWindow", "TimerWarningSelector", parentFrame)
	timerWarnConfig.windowFrame:SetTitle("Timer Warning Editor")
	timerWarnConfig.windowFrame:SetWidth(700)
	timerWarnConfig.windowFrame:SetHeight(720)
	timerWarnConfig.windowFrame:SetPoint("CENTER", UIParent, "CENTER")

	timerWarnConfig.xySelector = XYSelector.Create(timerWarnConfig.windowFrame, function(x,y) TimerWarningConfig.SetXY(timerWarnConfig, x, y) end)
	
	timerWarnConfig.okButton = UI.CreateFrame("RiftButton", "OKButton", timerWarnConfig.windowFrame)
	timerWarnConfig.okButton:SetPoint("BOTTOMLEFT", timerWarnConfig.windowFrame, "BOTTOMLEFT", 20, -20)
	timerWarnConfig.okButton:SetText("OK")
	timerWarnConfig.okButton.Event.LeftPress = function() TimerWarningConfig.WindowClose(timerWarnConfig, true) end	

	timerWarnConfig.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", timerWarnConfig.windowFrame)
	timerWarnConfig.cancelButton:SetPoint("BOTTOMRIGHT", timerWarnConfig.windowFrame, "BOTTOMRIGHT", -20, -20)
	timerWarnConfig.cancelButton:SetText("Cancel")
	timerWarnConfig.cancelButton.Event.LeftPress = function() TimerWarningConfig.WindowClose(timerWarnConfig, false) end	

	local margin = 4
	local height = 40
	timerWarnConfig.gridFrame = UI.CreateFrame("Frame", "GridFrame", timerWarnConfig.windowFrame)
	timerWarnConfig.gridFrame:SetPoint("TOPLEFT", timerWarnConfig.windowFrame, "TOPLEFT", 40, 80)
	timerWarnConfig.rowLayout = RowLayout.Create(timerWarnConfig.gridFrame, 12, 3, height, 200, margin, margin)

	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 1, 1, "Text")
	txtFrame:SetText("Name")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 1, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("The name of the warning must be unique")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 2, 1, "Text")
	txtFrame:SetText("Warning")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 3, 1, "Text")
	txtFrame:SetText("Initial delay (secs)")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 3, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Number of seconds to pause before showing warning")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 4, 1, "Text")
	txtFrame:SetText("Timer duration (secs)")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 4, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Duration before the timer repeats")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 5, 1, "Text")
	txtFrame:SetText("Repeats")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 5, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Number of times the timer repeats")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 6, 1, "Text")
	txtFrame:SetText("Show Bar")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 6, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Show a countdown bar")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 7, 1, "Text")
	txtFrame:SetText("x%,y%")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 8, 1, "Text")
	txtFrame:SetText("width%")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 8, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Width of the bar as a % of screen width")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 9, 1, "Text")
	txtFrame:SetText("Font Size")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 10, 1, "Text")
	txtFrame:SetText("Text colour")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 11, 1, "Text")
	txtFrame:SetText("Bar colour")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 12, 1, "Text")
	txtFrame:SetText("Bar text")
	local txtFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 12, 3, "Text")
	txtFrame:SetWordwrap(true)
	txtFrame:SetText("Include {Countdown} for the time remaining")

	timerWarnConfig.fontColourFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 10, 2, "Frame")
	timerWarnConfig.colourFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 11, 2, "Frame")

	timerWarnConfig.warningFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 2, 2, "Text")
	timerWarnConfig.warningFrame:SetText("")

	timerWarnConfig.nameFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 1, 2, "RiftTextfield")
	timerWarnConfig.nameFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	timerWarnConfig.barTextFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 12, 2, "RiftTextfield")
	timerWarnConfig.barTextFrame:SetBackgroundColor(0.15, 0.15, 0.15)
	
	timerWarnConfig.barCheckFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 6, 2, "RiftCheckbox", RowLayout.centerFrame)
	
	timerWarnConfig.delaySlider = TimerWarningConfig.CreateSlider(timerWarnConfig, 3, 2, 0, 300, 1)
	timerWarnConfig.durationSlider = TimerWarningConfig.CreateSlider(timerWarnConfig, 4, 2, 0, 300, 1)
	timerWarnConfig.repeatSlider = TimerWarningConfig.CreateSlider(timerWarnConfig, 5, 2, 0, 20, 1)
	timerWarnConfig.widthSlider = TimerWarningConfig.CreateSlider(timerWarnConfig, 8, 2, 5, 100, 1)
	timerWarnConfig.fontSizeSlider = TimerWarningConfig.CreateSlider(timerWarnConfig, 9, 2, 8, 150, 1)
	
	local xyFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 7, 2, "Frame")
	local xyFrame1 = UI.CreateFrame("Frame", "xyFrame1", xyFrame)
	xyFrame1:SetPoint("TOPLEFT", xyFrame, "TOPLEFT")
	xyFrame1:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMLEFT", math.floor((RowLayout.GetColWidth(timerWarnConfig.rowLayout, 2) - margin) / 2), 0)	
	timerWarnConfig.xSlider = NumberSlider.Create(xyFrame1, 0, 100, 0.01, 0)
	local xyFrame2 = UI.CreateFrame("Frame", "xyFrame2", xyFrame)
	xyFrame2:SetPoint("TOPLEFT", xyFrame1, "TOPRIGHT", margin, 0)
	xyFrame2:SetPoint("BOTTOMRIGHT", xyFrame, "BOTTOMRIGHT")
	timerWarnConfig.ySlider = NumberSlider.Create(xyFrame2, 0, 100, 0.01, 0)

	timerWarnConfig.warningButton = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 2, 3, "RiftButton", RowLayout.centerFrame)
	timerWarnConfig.warningButton:SetText("Choose")
	timerWarnConfig.warningButton.Event.LeftPress = function() WarnMeConfig.ChooseWarning(timerWarnConfig, WarnMe.warningTypeAll, function(a, warningName) TimerWarningConfig.ChooseWarningCallback(timerWarnConfig, warningName) end) end
	timerWarnConfig.xyButton = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 7, 3, "RiftButton", RowLayout.centerFrame)
	timerWarnConfig.xyButton:SetText("Choose")
	timerWarnConfig.xyButton.Event.LeftPress = function() TimerWarningConfig.ShowXY(timerWarnConfig) end
	timerWarnConfig.fontColourButton = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 10, 3, "RiftButton", RowLayout.centerFrame)
	timerWarnConfig.fontColourButton:SetText("Choose")
	timerWarnConfig.fontColourButton.Event.LeftPress = function() TimerWarningConfig.ShowChooseFontColour(timerWarnConfig) end
	timerWarnConfig.colourButton = RowLayout.CreateFrame(timerWarnConfig.rowLayout, 11, 3, "RiftButton", RowLayout.centerFrame)
	timerWarnConfig.colourButton:SetText("Choose")
	timerWarnConfig.colourButton.Event.LeftPress = function() TimerWarningConfig.ShowChooseColour(timerWarnConfig) end
	RowLayout.SetColWidth(timerWarnConfig.rowLayout, 3, timerWarnConfig.warningButton:GetWidth())

	RowLayout.Redraw(timerWarnConfig.rowLayout)

	timerWarnConfig.xySelector = XYSelector.Create(timerWarnConfig.parentFrame, function(x,y) TimerWarningConfig.SetXY(timerWarnConfig, x, y) end)
	timerWarnConfig.clrChooser = ColourChooser.Create(timerWarnConfig.windowFrame, timerWarnConfig.warning.slider.fontColour)
	timerWarnConfig.wrnWindow = WarningWindow.CreateWarningWindow(timerWarnConfig.parentFrame, timerWarnConfig.windowFrame)
	
	TimerWarningConfig.Show(timerWarnConfig)
	timerWarnConfig.windowFrame:SetVisible(false)
	
	return timerWarnConfig
end

function TimerWarningConfig.WindowClose(timerWarnConfig, okPressed)
	if okPressed == true then
		local message = ""
		local name = timerWarnConfig.nameFrame:GetText()
		if not name or string.len(name) < 1 then
			message = message .. "You must specify a name for this warning\n"
		end
		
		if string.len(message) > 0 then
			TimerWarningConfig.SetEnabled(timerWarnConfig, false)
			WarningWindow.ShowWarningWindow(timerWarnConfig.wrnWindow, message, function() TimerWarningConfig.SetEnabled(timerWarnConfig, true) end)
			return
		end
		
		TimerWarningConfig.FillWarning(timerWarnConfig)
	end
	
	timerWarnConfig.nameFrame:SetKeyFocus(false)
	timerWarnConfig.barTextFrame:SetKeyFocus(false)
	
	timerWarnConfig.windowFrame:SetVisible(false)
	if timerWarnConfig.callback then
		if okPressed == true then
			timerWarnConfig.callback(timerWarnConfig, timerWarnConfig.warning)
		else
			timerWarnConfig.callback(timerWarnConfig)
		end
	end
end

function TimerWarningConfig.Show(timerWarnConfig, inWarning)
	if inWarning then
		timerWarnConfig.warning = TimerWarningConfig.CopyWarning(inWarning)
	end
	
	local warning = timerWarnConfig.warning
	timerWarnConfig.nameFrame:SetText(warning.name)
	if warning.warningName then
		timerWarnConfig.warningFrame:SetText(warning.warningName)
	else
		timerWarnConfig.warningFrame:SetText("")
	end
	NumberSlider.SetValue(timerWarnConfig.delaySlider, warning.initialDelay)
	NumberSlider.SetValue(timerWarnConfig.durationSlider, warning.duration)
	NumberSlider.SetValue(timerWarnConfig.repeatSlider, warning.repeats)
	
	if warning.slider and warning.slider.enabled == true then
		timerWarnConfig.barCheckFrame:SetChecked(true)
		NumberSlider.SetValue(timerWarnConfig.xSlider, warning.slider.xPct)
		NumberSlider.SetValue(timerWarnConfig.ySlider, warning.slider.yPct)
		NumberSlider.SetValue(timerWarnConfig.widthSlider, warning.slider.widthPct)
		NumberSlider.SetValue(timerWarnConfig.fontSizeSlider, warning.slider.fontSize)
	else
		timerWarnConfig.barCheckFrame:SetChecked(false)
	end
	
	timerWarnConfig.colourFrame:SetBackgroundColor(warning.slider.colour.red, warning.slider.colour.green, warning.slider.colour.blue)
	timerWarnConfig.fontColourFrame:SetBackgroundColor(warning.slider.fontColour.red, warning.slider.fontColour.green, warning.slider.fontColour.blue)

	timerWarnConfig.windowFrame:SetVisible(true)
end

function TimerWarningConfig.FillWarning(timerWarnConfig)
	timerWarnConfig.warning.name = timerWarnConfig.nameFrame:GetText()
	timerWarnConfig.warning.warningName = timerWarnConfig.warningFrame:GetText()
	timerWarnConfig.warning.initialDelay = NumberSlider.GetValue(timerWarnConfig.delaySlider)
	timerWarnConfig.warning.duration = NumberSlider.GetValue(timerWarnConfig.durationSlider)
	timerWarnConfig.warning.repeats = NumberSlider.GetValue(timerWarnConfig.repeatSlider)
	
	if timerWarnConfig.barCheckFrame:GetChecked() == true then
		timerWarnConfig.warning.slider.enabled = true
		timerWarnConfig.warning.slider.xPct = NumberSlider.GetValue(timerWarnConfig.xSlider)
		timerWarnConfig.warning.slider.yPct = NumberSlider.GetValue(timerWarnConfig.ySlider)
		timerWarnConfig.warning.slider.widthPct = NumberSlider.GetValue(timerWarnConfig.widthSlider)
		timerWarnConfig.warning.slider.fontSize = NumberSlider.GetValue(timerWarnConfig.fontSizeSlider)
	else
		timerWarnConfig.warning.slider.enabled = false
	end
end

function TimerWarningConfig.SetEnabled(timerWarnConfig, enabled)
	--timerWarnConfig.nameFrame:SetEnabled(enabled)
	--timerWarnConfig.strFrame:SetEnabled(enabled)
	timerWarnConfig.okButton:SetEnabled(enabled)
	timerWarnConfig.cancelButton:SetEnabled(enabled)
	timerWarnConfig.warningButton:SetEnabled(enabled)
	NumberSlider.SetEnabled(timerWarnConfig.delaySlider, enabled)
	NumberSlider.SetEnabled(timerWarnConfig.durationSlider, enabled)
	NumberSlider.SetEnabled(timerWarnConfig.repeatSlider, enabled)
end

function TimerWarningConfig.CreateSlider(timerWarnConfig, row, col, minValue, maxValue, stepSize)
	local xyFrame = RowLayout.CreateFrame(timerWarnConfig.rowLayout, row, col, "Frame")
	return NumberSlider.Create(xyFrame, minValue, maxValue, stepSize, minValue)
end

function TimerWarningConfig.ChooseWarningCallback(timerWarnConfig, warningName)
	TimerWarningConfig.FillWarning(timerWarnConfig)
	timerWarnConfig.warning.warningName = warningName
	TimerWarningConfig.Show(timerWarnConfig)
	
	timerWarnConfig.windowFrame:SetVisible(true)
end

function TimerWarningConfig.ShowXY(timerWarnConfig)
	timerWarnConfig.windowFrame:SetVisible(false)
	XYSelector.SetVisible(timerWarnConfig.xySelector, true)
end

function TimerWarningConfig.SetXY(timerWarnConfig, x, y)
	timerWarnConfig.windowFrame:SetVisible(true)
	TimerWarningConfig.FillWarning(timerWarnConfig)
	timerWarnConfig.warning.slider.xPct = x
	timerWarnConfig.warning.slider.yPct = y
	TimerWarningConfig.Show(timerWarnConfig)
end

function TimerWarningConfig.ShowChooseColour(timerWarnConfig)
	TimerWarningConfig.SetEnabled(timerWarnConfig, false)
	ColourChooser.Show(timerWarnConfig.clrChooser, timerWarnConfig.warning.slider.colour, function(x, colour) TimerWarningConfig.ChooseColour(timerWarnConfig, colour) end)
end

function TimerWarningConfig.ChooseColour(timerWarnConfig, colour)
	TimerWarningConfig.SetEnabled(timerWarnConfig, true)
	if colour then
		timerWarnConfig.warning.slider.colour = colour
		TimerWarningConfig.FillWarning(timerWarnConfig)
		TimerWarningConfig.Show(timerWarnConfig)
	end
end

function TimerWarningConfig.ShowChooseFontColour(timerWarnConfig)
	TimerWarningConfig.SetEnabled(timerWarnConfig, false)
	ColourChooser.Show(timerWarnConfig.clrChooser, timerWarnConfig.warning.slider.fontColour, function(x, colour) TimerWarningConfig.ChooseFontColour(timerWarnConfig, colour) end)
end

function TimerWarningConfig.ChooseFontColour(timerWarnConfig, colour)
	TimerWarningConfig.SetEnabled(timerWarnConfig, true)
	if colour then
		timerWarnConfig.warning.slider.fontColour = colour
		TimerWarningConfig.FillWarning(timerWarnConfig)
		TimerWarningConfig.Show(timerWarnConfig)
	end
end

function TimerWarningConfig.CopyWarning(warning2)
	local warning = {}
	warning.warningType = WarnMe.warningTypeTimer
	warning.name = warning2.name
	warning.warningName = warning2.warningName
	warning.initialDelay = warning2.initialDelay
	warning.duration = warning2.duration
	warning.repeats = warning2.repeats
	
	warning.slider = {
		enabled = false,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "{Countdown}"
	}
	
	if warning2.slider and warning2.slider.enabled == true then
		warning.slider.enabled = warning2.slider.enabled
		warning.slider.xPct = warning2.slider.xPct
		warning.slider.yPct = warning2.slider.yPct
		warning.slider.widthPct = warning2.slider.widthPct
		warning.slider.fontSize = warning2.slider.fontSize
		warning.slider.text = warning2.slider.text
		warning.slider.colour = warning2.slider.colour
		warning.slider.fontColour = warning2.slider.fontColour
	end
	
	return warning
end

function TimerWarningConfig.GetDefaultWarning()
	local warning = {}
	warning.warningType = WarnMe.warningTypeTimer
	warning.name = ""
	warning.warningName = ""
	warning.initialDelay = 0
	warning.duration = 5
	warning.repeats = 0
	warning.slider = {
		enabled = false,
		xPct = 5,
		yPct = 10,
		widthPct = 15,
		fontSize = 16,
		colour = {
			red = 0.25,
			green = 1,
			blue = 0.25
		},
		fontColour = {
			red = 1,
			green = 1,
			blue = 1
		},
		text = "{Countdown}"
	}

	return warning
end

