-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

WarnMe = {}
WarnMe.name = "WarnMe"
WarnMe.slashName = "warnme"
WarnMe.version = "v1.9.3"
WarnMe.debug = false
WarnMe.x = 90
WarnMe.y = 90
WarnMe.timeOffset = 0
WarnMe.verbose = false
WarnMe.raidThreshold = 8
WarnMe.addonOff = false
WarnMe.inParty = false
WarnMe.inRaid = false
WarnMe.inCrossShardRaid = false
WarnMe.playerName = nil
WarnMe.playerID = nil
WarnMe.currentRole = "0"
WarnMe.currentZone = ""
WarnMe.inCombat = false
WarnMe.combatStartTime = 0
WarnMe.combatTimer = nil
WarnMe.combatEvent = false
WarnMe.unitIDMap = {}
WarnMe.unitID2NameMap = {}
WarnMe.unitName2IDMap = {}
WarnMe.events = {}
WarnMe.events.emotes = {}
WarnMe.events.npcChat = {}
WarnMe.events.buffs = {}
WarnMe.events.multiBuffs = {}
WarnMe.events.castbar = {}
WarnMe.events.health = {}
WarnMe.events.mana = {}
WarnMe.events.charge = {}
WarnMe.events.energy = {}
WarnMe.events.ability = {}
WarnMe.warnings = {}
WarnMe.enabledCustomWarnings = {}
WarnMe.totalEvents = {}
WarnMe.totalWarnings = {}
WarnMe.savedRoles = {}
WarnMe.warningTypeAll = 0
WarnMe.warningTypeText = 1
WarnMe.warningTypeTexture = 2
WarnMe.warningTypeTimer = 3
WarnMe.warningTypeScreenFlash = 4
WarnMe.warningTypeComposite = 5
WarnMe.warningTypeCancelBuff = 6
WarnMe.eventTypeBuffExists = 1
WarnMe.eventTypeBuffNotExists = 2
WarnMe.eventTypeBuffStackGreater = 3
WarnMe.eventTypeBuffStackLess = 4
WarnMe.eventTypeEmote = 5
WarnMe.eventTypeNpcChat = 6
WarnMe.eventTypeCastbar = 7
WarnMe.eventTypeStat = 8
WarnMe.eventTypeAbility = 9
WarnMe.statTypeHealth = 1
WarnMe.statTypeEnergy = 2
WarnMe.statTypeMana = 3
WarnMe.statTypeCharge = 4
WarnMe.abilityTypeBegin = 1
WarnMe.abilityTypeEnd = 2
WarnMe.abilityTypeAvailable = 3
WarnMe.abilityTypeNotAvailable = 4
WarnMe.raidZoneMap = {
	["Warboss Drak"] = "Infernal Dawn",
	["Ituziel"] = "Infernal Dawn",
	["Ember Conclave"] = "Infernal Dawn",
	["Maklamos the Scryer"] = "Infernal Dawn",
	["Rusila Dreadblade"] = "Infernal Dawn",
	["Murdantix"] = "Hammerknell Fortress",
	["Matron Zamira"] = "Hammerknell Fortress",
	["Soulrender Zilas"] = "Hammerknell Fortress",
	["Vladmal Prime"] = "Hammerknell Fortress",
	["Sicaron"] = "Hammerknell Fortress",
	["King Molinar"] = "Hammerknell Fortress",
	["Prince Dollin"] = "Hammerknell Fortress",
	["Estrode"] = "Hammerknell Fortress",
	["Grugonim"] = "Hammerknell Fortress",
	["Inquisitor Garau"] = "Hammerknell Fortress",
	["Aqualix"] = "Hammerknell Fortress",
	["Denizar"] = "Hammerknell Fortress",
	["Inwar Darktide"] = "Hammerknell Fortress",
	["Jornaru"] = "Hammerknell Fortress",
	["Akylios"] = "Hammerknell Fortress",
	["Ereandorn"] = "Rise of the Phoenix",
	["Beruhast"] = "Rise of the Phoenix",
	["General Silgen"] = "Rise of the Phoenix",
	["High Priest Arakhurn"] = "Rise of the Phoenix",
	["Dark Focus"] = "River of Souls",
	["Warmaster Galenir"] = "River of Souls",
	["Plutonus the Immortal"] = "River of Souls",
	["Herald Gaurath"] = "River of Souls",
	["Alsbeth the Discordant"] = "River of Souls",
	["Duke Letareus"] = "Greenscale's Blight",
	["Infiltrator Johlen"] = "Greenscale's Blight",
	["Oracle Aleria"] = "Greenscale's Blight",
	["Prince Hylas"] = "Greenscale's Blight",
	["Lord Greenscale"] = "Greenscale's Blight",
	["Anrak the Foul"] = "Gilded Prophecy",
	["Guurloth"] = "Gilded Prophecy",
	["Thalguur"] = "Gilded Prophecy",
	["Uruluuk"] = "Gilded Prophecy",
	["Assault Commander Jorb"] = "Drowned Halls",
	["Joloral Ragetide"] = "Drowned Halls",
	["Isskal"] = "Drowned Halls",
	["High Priestess Hydriss"] = "Drowned Halls"
}

--
-- Main functions
--

function WarnMe.Dump()
	print("WarnMe version=" .. WarnMe.version)
	print("inParty=" .. tostring(WarnMe.inParty) .. ", inRaid=" .. tostring(WarnMe.inRaid) .. ", inCrossShardRaid=" .. tostring(WarnMe.inCrossShardRaid))
	print("PlayerName=" .. WarnMe.playerName .. ", currentRole=" .. WarnMe.currentRole .. ", inCombat=" .. tostring(WarnMe.inCombat))
	print("CurrentZone=" .. WarnMe.currentZone)
end

function WarnMe.Debug(str)
	if WarnMe.debug == true then
		print(str)
	end
end

function WarnMe.VerboseMessage(str)
	if WarnMe.verbose == true then
		local timeVal = Inspect.Time.Frame()
		local message = WarnMe.GetTimeString(timeVal)
		if WarnMe.inCombat == true then
			offsetStr = WarnMe.GetTimeString(timeVal - WarnMe.combatStartTime, 0)
			message = message .. " " .. offsetStr
		end
		
		print(message .. " - " .. str)
	end
end

function WarnMe.SplitArgs(str)
	local result = {}
	local count = 1
	for token in string.gmatch(str, "[^%s]+") do
		result[count] = token
		count = count + 1
	end
	return result
end

function WarnMe.SetTimeOffset(timeStr)
	if timeStr then
		local timeParts = Utilities.Split(timeStr, ':')
		if #timeParts < 2 or #timeParts > 3 then
			print("Ignoring settime " .. timeStr .. " String must be in format hh:mm or hh:mm:ss")
			return
		end
		
		local hours = tonumber(timeParts[1])
		if not hours or hours < 0 or hours > 23 then
			print("Ignoring settime " .. timeStr .. " Hours must be in range 0-23")
			return
		end
		
		local mins = tonumber(timeParts[2])
		if not mins or mins < 0 or mins > 59 then
			print("Ignoring settime " .. timeStr .. " Minutes must be in range 0-59")
			return
		end
		
		local secs = 0
		if #timeParts == 3 then
			secs = tonumber(timeParts[3])
			if not secs or secs < 0 or secs > 59 then
				print("Ignoring settime " .. timeStr .. " Seconds must be in range 0-59")
				return
			end
		end
		
		WarnMe.timeOffset = (hours * 3600 + mins * 60 + secs) - Inspect.Time.Frame()
	else
		WarnMe.timeOffset = 0
	end
end

function WarnMe.GetTimeString(timeVal, inOffset)
	local offset = WarnMe.timeOffset
	if inOffset then
		offset = inOffset
	end
	
	local totalSeconds = timeVal + offset
	local hours = math.floor(totalSeconds / 3600)
	local mins = math.floor(totalSeconds % 3600 / 60)
	local secs = math.floor(totalSeconds % 60)
	return string.format("%02d:%02d:%02d", hours, mins, secs)
end

function WarnMe.CheckInCombatScope(spec)
	if spec.scope.inCombat ~= nil then
		if spec.scope.inCombat ~= WarnMe.inCombat then
			return false
		end
	end
	
	return true
end

function WarnMe.ChangeZone(zoneName, dontReset)
	if zoneName then
		local trimmedZoneName = ""
		if type(zoneName) == "string" then
			trimmedZoneName = Utilities.Trim(zoneName)
		else
			local thisZone = ""
			for i = 2, #zoneName do
				thisZone = thisZone .. " " .. zoneName[i]
			end

			trimmedZoneName = Utilities.Trim(thisZone)
		end
		
		if WarnMe.currentZone ~= trimmedZoneName then
			WarnMe.currentZone = trimmedZoneName
			if string.len(WarnMe.currentZone) > 0 then
				print("Zone changed to be " .. trimmedZoneName)
			else
				print("Zone cleared")
			end
			
			if dontReset ~= true then
				WarnMe.Reset()
			end
		end
	end
end

function WarnMe.LoadUnknownUnits(unitIDTable)
	local unknownUnits = {}
	
	if not WarnMe.unitName2IDMap["player"] then
		local playerUnit = Inspect.Unit.Lookup("player")
		unknownUnits[playerUnit] = 1
		WarnMe.unitName2IDMap["player"] = playerUnit
	end
	
	for id, val in pairs(unitIDTable) do
		if val ~= false and id ~= false then
			if not WarnMe.unitID2NameMap[id] then
				unknownUnits[id] = 1
			end
		end
	end
	
	local details = Inspect.Unit.Detail(unknownUnits)
	for id, detail in pairs(details) do
		WarnMe.unitID2NameMap[id] = detail.name
		WarnMe.unitName2IDMap[detail.name] = id
		
		if WarnMe.inRaid == true then
			if WarnMe.raidZoneMap[detail.name] then
				WarnMe.ChangeZone(WarnMe.raidZoneMap[detail.name])
			end
		end
	end
end

function WarnMe.ParseEmote(emote)
	local event = WarnMe.events.emotes[emote]
	if event then
		WarnMe.SetWarningVisible(event, true)
	end
end

function WarnMe.ParseNpcChat(fromName, message)
	if WarnMe.events.npcChat[fromName] then
		local event = WarnMe.events.npcChat[fromName][message]
		if event then
			WarnMe.SetWarningVisible(event, true)
		end
	end
end

function WarnMe.SetCombatStatus(inCombat)
	if inCombat == true then
		if WarnMe.inCombat == false then
			WarnMe.VerboseMessage("Start Combat")
			WarnMe.inCombat = true
			WarnMe.combatStartTime = Inspect.Time.Frame()
			WarnMe.button:SetAlpha(0.2)
			WarnMe.CombatChanged()
		end
	elseif inCombat == false then
		if WarnMe.inCombat == true then
			WarnMe.VerboseMessage("End Combat")

			-- Empty the name map
			WarnMe.unitID2NameMap = {}
			WarnMe.unitName2IDMap = {}

			WarnMe.button:SetAlpha(1)
			WarnMe.inCombat = false
			WarnMe.CombatChanged()
			WarnMe.HideAllInCombatWarnings()
		end
	end
end

function WarnMe.CalculateGroupSize()
	local playerCount = 0
	local crossShard = false

	local inCombat = false
	for id = 1, 20 do
		local unitSpecifier = string.format("group%02d", id)
		unitdetail = Inspect.Unit.Detail(unitSpecifier)
		if unitdetail then
			if string.find(unitdetail.name, "@") then
				crossShard = true
			end
			
			if unitdetail.combat == true then
				inCombat = true
			end
			
			playerCount = playerCount + 1
		end
	end
	

	local inParty = false
	local inRaid = false
	local inCrossShardRaid = false
	if playerCount > 1 and playerCount < WarnMe.raidThreshold then
		inParty = true
	end
	
	if playerCount >= WarnMe.raidThreshold then
		if crossShard then
			inCrossShardRaid = true
		else
			inRaid = true
		end
	end
	
	local ret = false
	if inParty ~= WarnMe.inParty or inRaid ~= WarnMe.inRaid or inCrossShardRaid ~= WarnMe.inCrossShardRaid then
		WarnMe.inParty = inParty
		WarnMe.inRaid = inRaid
		WarnMe.inCrossShardRaid = inCrossShardRaid		
		ret = true
	end
	
	if ret == true and WarnMe.inRaid ~= true then
		WarnMe.ChangeZone("", true)
	end
	
	--WarnMe.SetCombatStatus(inCombat)
	
	return ret
end

function WarnMe.IsInScope(event)
	if event.scope.all == true then
		return true
	end
	
	if event.scope.player then
		local found = false
		for _, playerStr in ipairs(Utilities.Split(event.scope.player, ',')) do
			local thisPlayer = Utilities.Trim(playerStr)
			if thisPlayer == WarnMe.playerName then
				found = true
			end
		end
		
		if found == false then
			return false
		end
	end
	
	if event.scope.roles and WarnMe.currentRole ~= "0" then
		if not string.find(event.scope.roles, WarnMe.currentRole) then
			return false
		end
	end
	
	if event.scope.zone and next(event.scope.zone) ~= nil then
		if not WarnMe.currentZone or WarnMe.currentZone == "" then
				return false
		end
		
		if not event.scope.zone[WarnMe.currentZone] then
			return false
		end
	end
	
	if event.scope.inParty == true and WarnMe.inParty == true then
		return true
	end
	
	if event.scope.inRaid == true and WarnMe.inRaid == true then
		return true
	end
	
	if event.scope.inCrossShardRaid == true and WarnMe.inCrossShardRaid == true then
		return true
	end
	
	if WarnMe.inParty == false and WarnMe.inRaid == false and WarnMe.inCrossShardRaid == false and event.scope.isSolo == true then
		return true
	end
	
	return false
end

function WarnMe.CopyScope(scope2)
	local scope = {}

	if scope2.all and scope2.all == true then
		scope.all = true
	else
		if scope2.player and string.len(scope2.player) > 0 then
			scope.player = scope2.player
		end

		if scope2.roles and string.len(scope2.roles) > 0 then
			scope.roles = scope2.roles
		end

		if scope2.isSolo == true then
			scope.isSolo = scope2.isSolo
		end

		if scope2.inParty == true then
			scope.inParty = scope2.inParty
		end

		if scope2.inRaid == true then
			scope.inRaid = scope2.inRaid
		end

		if scope2.inCrossShardRaid == true then
			scope.inCrossShardRaid = scope2.inCrossShardRaid
		end
		
		if scope2.inCombat ~= nil then
			scope.inCombat = scope2.inCombat
		end
		
		if scope2.zone then
			scope.zone = {}
			for name, value in pairs(scope2.zone) do
				if name ~= "" then
					scope.zone[name] = value
				end
			end
		end
	end
	
	return scope
end

function WarnMe.HideAllWarnings()
	WarnMe.VerboseMessage("Hide all warnings")

	for name, warning in pairs(WarnMe.warnings) do
		warning.setVisible(false)
	end
end

function WarnMe.HideAllInCombatWarnings()
	WarnMe.VerboseMessage("Hide all inCombat warnings")

	for name, warning in pairs(WarnMe.warnings) do
		if warning.shownInCombat == true then
			warning.setVisible(false)
		end
	end
end

function WarnMe.SetMultiEventWarningVisible(event, buffName, visible)
	event.buffVisible[buffName] = visible
	local totalVisible = true
	for _, isVisible in pairs(event.buffVisible) do
		totalVisible = totalVisible and (isVisible == true)
	end
	
	WarnMe.SetWarningVisible(event, totalVisible)
end

function WarnMe.SetWarningVisible(event, visible)
	local initialVisible = visible
	if visible == true and WarnMe.CheckInCombatScope(event.spec) == false then
		visible = false
	end
	
	local shownInCombat = false
	if visible == true and event.spec.scope.inCombat == true then
		shownInCombat = true
	end
	
	local warning = WarnMe.warnings[event.spec.warningName]
	if warning then
		warning.events[event.spec.name] = visible
		
		local isVisible = false
		if warning.spec.showIfAllVisible == true then
			isVisible = true
		end
		for name, val in pairs(warning.events) do
			if warning.spec.showIfAllVisible == true then
				isVisible = isVisible and val
			else
				isVisible = isVisible or val
			end
		end
		
		if isVisible == true and shownInCombat == true then
			warning.shownInCombat = true
		else
			warning.shownInCombat = nil
		end
		
		warning.setVisible(isVisible)
	end

	if WarnMe.verbose == true then
		local message = "Set event " .. event.spec.name .. " warning " .. event.spec.warningName .. " visible=" .. tostring(visible) .. " iv=" .. tostring(initialVisible) .. " sic=" .. tostring(shownInCombat)
		WarnMe.VerboseMessage(message)
	end
end

function WarnMe.ClearEvents()
	WarnMe.warnings = {}
	WarnMe.events = {}
	WarnMe.events.emotes = {}
	WarnMe.events.npcChat = {}
	WarnMe.events.buffs = {}
	WarnMe.events.multiBuffs = {}
	WarnMe.events.castbar = {}
	WarnMe.events.health = {}
	WarnMe.events.energy = {}
	WarnMe.events.charge = {}
	WarnMe.events.mana = {}
	WarnMe.events.ability = {}
end

function WarnMe.InitialiseCompositeWarning(warnings, warning)
	warning.compWarn = CompositeWarning.Create(warning.spec.name, warning.spec.warningNames, warnings)
	warning.setVisible = function(visible) CompositeWarning.SetVisible(warning.compWarn, visible) end
	warning.getPosition = nil
	warning.setDragFrameVisible = nil
end

function WarnMe.InitialiseScreenFlashWarning(warning, parentFrame)
	warning.scrnFlashWarn = ScreenFlashWarning.Create(warning.spec.name, parentFrame, warning.spec.screenPct,
						warning.spec.duration, warning.spec.alpha, warning.spec.colour,
						warning.spec.flashFrequency, warning.spec.flashAlpha, warning.spec.flashColour)
	warning.setVisible = function(visible) ScreenFlashWarning.SetVisible(warning.scrnFlashWarn, visible) end
	warning.getPosition = nil
	warning.setDragFrameVisible = nil
end

function WarnMe.InitialiseTextWarning(warning, parentFrame)
	warning.txtWarn = TextWarning.Create(warning.spec.name, parentFrame, warning.spec.str, warning.spec.xPct, warning.spec.yPct,
						warning.spec.fontSize, warning.spec.duration, warning.spec.alpha, warning.spec.colour,
						warning.spec.flashFrequency, warning.spec.flashAlpha, warning.spec.flashColour)
	warning.setVisible = function(visible) TextWarning.SetVisible(warning.txtWarn, visible) end
	warning.getPosition = function() return TextWarning.GetPosition(warning.txtWarn) end
	warning.setDragFrameVisible = function(visible) TextWarning.SetDragWindowVisible(warning.txtWarn, visible) end
end

function WarnMe.InitialiseTextureWarning(warning, parentFrame)
	warning.txtWarn = TextureWarning.Create(warning.spec.name, parentFrame, warning.spec.addon, warning.spec.icon, warning.spec.xPct, warning.spec.yPct,
						warning.spec.scale, warning.spec.duration, warning.spec.alpha, warning.spec.flashFrequency, warning.spec.flashAlpha)
	if warning.spec.countdownDuration and warning.spec.countdownDuration > 0 then
		TextureWarning.SetCountdown(warning.txtWarn, warning.spec.countdownFontColour, warning.spec.countdownHighlightDuration, warning.spec.countdownHighlightColour)
	end
	warning.setVisible = function(visible) TextureWarning.SetVisible(warning.txtWarn, visible) end
	warning.getPosition = function() return TextureWarning.GetPosition(warning.txtWarn) end
	warning.setDragFrameVisible = function(visible) TextureWarning.SetDragWindowVisible(warning.txtWarn, visible) end
end

function WarnMe.InitialiseCancelBuffWarning(warning, parentFrame)
	warning.cancWarn = CancelBuffWarning.Create(warning.spec.name, warning.spec.buff, warning.spec.target)
	warning.setVisible = function(visible) CancelBuffWarning.SetVisible(warning.cancWarn, visible) end
	warning.getPosition = nil
	warning.setDragFrameVisible = nil
end

function WarnMe.InitialiseTimerWarning(event, warning, warnings, totalWarnings, parentFrame)
	if warning.spec.warningName and string.len(warning.spec.warningName) > 0 then
		WarnMe.CreateEmptyWarning(warnings, totalWarnings, event, warning.spec.warningName)
	end
	
	warning.txtWarn = TimerWarning.Create(warning.spec.name, parentFrame, warnings[warning.spec.warningName], warning.spec.initialDelay, warning.spec.duration, warning.spec.repeats)
	warning.setVisible = function(visible) TimerWarning.SetVisible(warning.txtWarn, visible) end
	
	if warning.spec.slider and warning.spec.slider.enabled == true then
		TimerWarning.SetSlider(warning.txtWarn, warning.spec.slider.xPct, warning.spec.slider.yPct, warning.spec.slider.widthPct, warning.spec.slider.fontSize, warning.spec.slider.colour, warning.spec.slider.fontColour, warning.spec.slider.text)
		warning.getPosition = function() return TimerWarning.GetPosition(warning.txtWarn) end
		warning.setDragFrameVisible = function(visible) TimerWarning.SetDragWindowVisible(warning.txtWarn, visible) end
	else
		warning.getPosition = nil
		warning.setDragFrameVisible = nil
	end
end

function WarnMe.CreateEmptyWarning(warnings, totalWarnings, event, warningName, parentFrame)
	if not warnings[warningName] then
		if not totalWarnings[warningName] then
			return
		end
		
		local warning = {}
		warning.spec = totalWarnings[warningName]
		warning.events = {}
		if warning.spec.warningType == WarnMe.warningTypeTimer then
			WarnMe.InitialiseTimerWarning(event, warning, warnings, totalWarnings, parentFrame)
		end
		
		if warning.spec.warningType == WarnMe.warningTypeComposite then
			for _, embeddedName in ipairs(warning.spec.warningNames) do
				WarnMe.CreateEmptyWarning(warnings, totalWarnings, event, embeddedName, parentFrame)
			end
		end

		warnings[warningName] = warning
	end

	local warning = warnings[warningName]
	if not warning.events[event.name] then
		warning.events[event.name] = false
	end
end

function WarnMe.CreateRelevantEvents(warnings, totalEvents, totalWarnings)
	local loadedEvents = ""
	for name, event in pairs(totalEvents) do
		if WarnMe.IsInScope(event) then
			if not totalWarnings[event.warningName] then
				print("Event " .. name .. " ignored due to missing warning " .. event.warningName)
			else
				local thisEvent = {}
				thisEvent.spec = event
				if event.eventType == WarnMe.eventTypeEmote then
					local str = WarnMe.GetPlayerSpecificString(event.emote.message)
					WarnMe.events.emotes[str] = thisEvent
				elseif event.eventType == WarnMe.eventTypeNpcChat then
					if not WarnMe.events.npcChat[event.NpcChat.name] then
						WarnMe.events.npcChat[event.NpcChat.name] = {}
					end
					
					local str = WarnMe.GetPlayerSpecificString(event.NpcChat.message)
					WarnMe.events.npcChat[event.NpcChat.name][str] = thisEvent
				elseif event.eventType == WarnMe.eventTypeCastbar then
					for _, name in ipairs(event.castbar.unitSpecs) do
						if not WarnMe.events.castbar[event.castbar.abilityName] then
							WarnMe.events.castbar[event.castbar.abilityName] = {}
							WarnMe.events.castbar[event.castbar.abilityName].unitSpec = {}
							WarnMe.events.castbar[event.castbar.abilityName].all = {}
							WarnMe.events.castbar[event.castbar.abilityName].named = {}
						end


						if name == "named" then
							Utilities.ArrayInsert(WarnMe.events.castbar[event.castbar.abilityName].named, event.castbar.targetName, thisEvent)
						elseif name == "all" then
							Utilities.ArrayInsert(WarnMe.events.castbar[event.castbar.abilityName], "all", thisEvent)
						else
							Utilities.ArrayInsert(WarnMe.events.castbar[event.castbar.abilityName].unitSpec, name, thisEvent)
						end
					end
				elseif event.eventType == WarnMe.eventTypeStat then
					if event.statType == WarnMe.statTypeHealth then
						for _, name in ipairs(event.stat.unitSpecs) do
							if not WarnMe.events.health.unitSpec then
								WarnMe.events.health.unitSpec = {}
								WarnMe.events.health.named = {}
							end

							if name == "named" then
								Utilities.ArrayInsert(WarnMe.events.health.named, event.stat.targetName, thisEvent)
							else
								Utilities.ArrayInsert(WarnMe.events.health.unitSpec, name, thisEvent)
							end
						end
					elseif event.statType == WarnMe.statTypeMana then
						for _, name in ipairs(event.stat.unitSpecs) do
							if not WarnMe.events.mana.unitSpec then
								WarnMe.events.mana.unitSpec = {}
								WarnMe.events.mana.named = {}
							end

							if name == "named" then
								Utilities.ArrayInsert(WarnMe.events.mana.named, event.stat.targetName, thisEvent)
							else
								Utilities.ArrayInsert(WarnMe.events.mana.unitSpec, name, thisEvent)
							end
						end
					elseif event.statType == WarnMe.statTypeEnergy then
						for _, name in ipairs(event.stat.unitSpecs) do
							if not WarnMe.events.energy.unitSpec then
								WarnMe.events.energy.unitSpec = {}
								WarnMe.events.energy.named = {}
							end

							if name == "named" then
								Utilities.ArrayInsert(WarnMe.events.energy.named, event.stat.targetName, thisEvent)
							else
								Utilities.ArrayInsert(WarnMe.events.energy.unitSpec, name, thisEvent)
							end
						end
					elseif event.statType == WarnMe.statTypeCharge then
						for _, name in ipairs(event.stat.unitSpecs) do
							if not WarnMe.events.charge.unitSpec then
								WarnMe.events.charge.unitSpec = {}
								WarnMe.events.charge.named = {}
							end

							if name == "named" then
								Utilities.ArrayInsert(WarnMe.events.charge.named, event.stat.targetName, thisEvent)
							else
								Utilities.ArrayInsert(WarnMe.events.charge.unitSpec, name, thisEvent)
							end
						end
					end
				elseif event.eventType == WarnMe.eventTypeAbility then
					WarnMe.events.ability[event.name] = thisEvent
				elseif string.find(event.buff.name, ",") then
					WarnMe.events.multiBuffs[event.name] = thisEvent
				else
					WarnMe.events.buffs[event.name] = thisEvent
				end
				
				WarnMe.CreateEmptyWarning(warnings, totalWarnings, event, event.warningName, WarnMe.parentFrame)
				
				if string.len(loadedEvents) > 0 then
					loadedEvents = loadedEvents .. ", "
				end
				loadedEvents = loadedEvents .. event.name
			end
		end
	end
	
	return loadedEvents
end

function WarnMe.InitialiseWarnings(warnings, parentFrame)
	for warningName, warning in pairs(warnings) do
		if warning.spec.warningType == WarnMe.warningTypeScreenFlash then
			WarnMe.InitialiseScreenFlashWarning(warning, parentFrame)
		end
	end
	
	for warningName, warning in pairs(warnings) do
		if warning.spec.warningType == WarnMe.warningTypeTexture then
			WarnMe.InitialiseTextureWarning(warning, parentFrame)
		end
	end
	
	for warningName, warning in pairs(warnings) do
		if warning.spec.warningType == WarnMe.warningTypeText then
			WarnMe.InitialiseTextWarning(warning, parentFrame)
		end
	end
	
	for warningName, warning in pairs(warnings) do
		if warning.spec.warningType == WarnMe.warningTypeCancelBuff then
			WarnMe.InitialiseCancelBuffWarning(warning, parentFrame)
		end
	end

	for warningName, warning in pairs(warnings) do
		if warning.spec.warningType == WarnMe.warningTypeComposite then
			WarnMe.InitialiseCompositeWarning(warnings, warning, parentFrame)
		end
	end
end

function WarnMe.InitialiseEvents()
	for name, event in pairs(WarnMe.events.ability) do
		if event.spec.ability.abilityType == WarnMe.abilityTypeBegin then
			AbilityManager.AddAbilityBeginCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, true) end)
		elseif event.spec.ability.abilityType == WarnMe.abilityTypeEnd then
			AbilityManager.AddAbilityEndCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, true) end)
		elseif event.spec.ability.abilityType == WarnMe.abilityTypeAvailable then
			AbilityManager.AddAbilityAvailableCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, true) end)
			AbilityManager.AddAbilityNotAvailableCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, false) end)
		elseif event.spec.ability.abilityType == WarnMe.abilityTypeNotAvailable then
			AbilityManager.AddAbilityAvailableCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, false) end)
			AbilityManager.AddAbilityNotAvailableCallback(event.spec.ability.name, function(a) WarnMe.SetWarningVisible(event, true) end)
		else
			if event.spec.ability.abilityType then
				print("Ignoring ability " .. event.spec.name .. " with unknown type " .. event.spec.eventType)
			else
				print("Ignoring ability " .. event.spec.name .. " with nil type ")
			end
		end
	end
	
	for name, event in pairs(WarnMe.events.buffs) do
		if event.spec.eventType == WarnMe.eventTypeBuffExists then
			BuffManager.AddBuffExistsCallback(event.spec.buff.unitSpec, event.spec.buff.name, event.spec.buff.playersOwn, function(a,b,c,visible) WarnMe.SetWarningVisible(event, visible) end)
		elseif event.spec.eventType == WarnMe.eventTypeBuffNotExists then
			BuffManager.AddBuffNotExistsCallback(event.spec.buff.unitSpec, event.spec.buff.name, event.spec.buff.playersOwn, function(a,b,c,visible) WarnMe.SetWarningVisible(event, visible) end)
		elseif event.spec.eventType == WarnMe.eventTypeBuffStackGreater then
			BuffManager.AddStacksGreaterEqualToCallback(event.spec.buff.unitSpec, event.spec.buff.name, event.spec.buff.playersOwn, event.spec.buff.stacks, function(a,b,c,visible) WarnMe.SetWarningVisible(event, visible) end)
		elseif event.spec.eventType == WarnMe.eventTypeBuffStackLess then
			BuffManager.AddStacksLessEqualToCallback(event.spec.buff.unitSpec, event.spec.buff.name, event.spec.buff.playersOwn, event.spec.buff.stacks, function(a,b,c,visible) WarnMe.SetWarningVisible(event, visible) end)
		else
			if event.spec.eventType then
				print("Ignoring buff " .. event.spec.name .. " with unknown type " .. event.spec.eventType)
			else
				print("Ignoring buff " .. event.spec.name .. " with nil type ")
			end
		end
	end
	
	for name, event in pairs(WarnMe.events.multiBuffs) do
		if event.spec.eventType == WarnMe.eventTypeBuffNotExists then
			event.buffVisible = {}
			for _, buffStr in ipairs(Utilities.Split(event.spec.buff.name, ',')) do
				local buffName = Utilities.Trim(buffStr)
				event.buffVisible[buffName] = false
				BuffManager.AddBuffNotExistsCallback(event.spec.buff.unitSpec, buffName, event.spec.buff.playersOwn, function(a,bName,c,visible) WarnMe.SetMultiEventWarningVisible(event, bName, visible) end)
			end
		else
			if event.spec.eventType then
				print("Ignoring multi-buff " .. event.spec.name .. " with unknown type " .. event.spec.eventType)
			else
				print("Ignoring multi-buff " .. event.spec.name .. " with nil type ")
			end
		end
	end
end

function WarnMe.LoadRelevantEvents()
	local eventSpecs, warningSpecs = CustomEvents.GetCustomEvents(WarnMe.enabledCustomWarnings)

	local totalEvents = {}
	for name, eventSpec in pairs(WarnMe.totalEvents) do
		totalEvents[name] = eventSpec
	end

	for name, eventSpec in pairs(eventSpecs) do
		if not totalEvents[name] then
			totalEvents[name] = eventSpec
		else
			WarnMe.VerboseMessage("Ignoring overwritten event " .. name)
		end
	end

	local totalWarnings = {}
	for name, warningSpec in pairs(WarnMe.totalWarnings) do
		totalWarnings[name] = warningSpec
	end
	
	for name, warningSpec in pairs(warningSpecs) do
		if not totalWarnings[name] then 
			totalWarnings[name] = warningSpec
		else
			WarnMe.VerboseMessage("Ignoring overwritten warning " .. name)
		end
	end
	
	local loadedEvents = WarnMe.CreateRelevantEvents(WarnMe.warnings, totalEvents, totalWarnings)
		
	if string.len(loadedEvents) < 1  then
		print("No events loaded!")
	else
		print("Loaded events " .. loadedEvents)
		WarnMe.InitialiseWarnings(WarnMe.warnings, WarnMe.parentFrame)
		WarnMe.InitialiseEvents()
	end
end

function WarnMe.SetPlayerNameAndID()
	local playerID = Inspect.Unit.Lookup("player")
	if playerID then
		WarnMe.playerID = playerID
		
		local playerDetail = Inspect.Unit.Detail(playerID)
		if playerDetail and playerDetail.name then
			WarnMe.playerName = playerDetail.name
			WarnMe.currentRole = WarnMe.GetRoleNumber()
		end
	end
end

function WarnMe.GetPlayerSpecificString(str)
	return string.gsub(str, "{Player}", WarnMe.playerName)
end

function WarnMe.RefreshBuffs()
	BuffManager.Refresh("player")
	BuffManager.Refresh("player.pet")
	BuffManager.Refresh("player.target")
	BuffManager.Refresh("focus")
end

function WarnMe.CheckStat(spec, statPct)
	local fireEvent = false
	
	if statPct then
		if WarnMe.CheckInCombatScope(spec) == true then
			if spec.stat.pct2 then
				if statPct > spec.stat.pct and statPct < spec.stat.pct2 then
					fireEvent = true
				end
			else
				if spec.stat.isLess == true then
					if statPct <= spec.stat.pct then
						fireEvent = true
					end
				else
					if statPct >= spec.stat.pct then
						fireEvent = true
					end
				end
			end
		end
	end
	
	return fireEvent
end

function WarnMe.Stat(eventTable, statTable, pctCallback)
	WarnMe.LoadUnknownUnits(eventTable)

	if not statTable.unitSpec then
		return
	end
	
	local unitSpecsEmpty = true
	local unitSpecs = {}
	for name, _ in pairs(statTable.unitSpec) do
		unitSpecsEmpty = false
		unitSpecs[name] = 1
	end
	
	local foundUnit = false
	local unitLookup = {}
	local detailsLookup = {}
	if unitSpecsEmpty == false then
		local unitIDs = Inspect.Unit.Lookup(unitSpecs)
		for unitSpec, unitID in pairs(unitIDs) do
			unitLookup[unitID] = unitSpec
			if eventTable[unitID] then
				foundUnit = true
				detailsLookup[unitID] = 1
			end
		end
	end
	
	for unitID, _ in pairs(eventTable) do
		local name = WarnMe.unitID2NameMap[unitID]
		if name and statTable.named[name] then
			foundUnit = true
			detailsLookup[unitID] = 1
		end
	end
	
	if foundUnit == true then		
		local unitDetails = Inspect.Unit.Detail(detailsLookup)
		for unitID, details in pairs(unitDetails) do
			if details.name then
				local events = {}
				if unitLookup[unitID] then
					for _, thisEvent in ipairs(statTable.unitSpec[unitLookup[unitID]]) do
						table.insert(events, thisEvent)
					end
				end
				
				if statTable.named[details.name] then
					for _, thisEvent in ipairs(statTable.named[details.name]) do
						table.insert(events, thisEvent)
					end
				end
				
				for _, thisEvent in ipairs(events) do
					if not thisEvent.stat then
						thisEvent.stat = {}
					end

					local reset = false
					if thisEvent.stat.unitID and thisEvent.stat.unitID ~= unitID then
						thisEvent.stat.unitID = nil
						thisEvent.stat.prevPct = nil
						reset = true
					end
					
					local pct = pctCallback(details)
					if pct then
						local fireEvent = WarnMe.CheckStat(thisEvent.spec, pct)
						local prevFireEvent = WarnMe.CheckStat(thisEvent.spec, thisEvent.stat.prevPct)

						if fireEvent == true and prevFireEvent == false then
							thisEvent.stat.unitID = unitID
							thisEvent.stat.prevPct = pct
							WarnMe.SetWarningVisible(thisEvent, true)
						elseif fireEvent == false and prevFireEvent == true then
							thisEvent.stat.unitID = unitID
							thisEvent.stat.prevPct = pct
							WarnMe.SetWarningVisible(thisEvent, false)
						end
					end
				end
			end
		end
	end
end

function WarnMe.ResetStat(statTable)
	for _, eventTable in pairs(statTable.named) do
		for _, event in ipairs(eventTable) do
			if event.stat and event.stat.prevPct and event.spec.scope.inCombat ~= nil then
				event.stat.prevPct = nil
				event.stat.unitID = nil
			end
		end
	end
	for _, eventTable in pairs(statTable.unitSpec) do
		for _, event in ipairs(eventTable) do
			if event.stat and event.stat.prevPct and event.spec.scope.inCombat ~= nil then
				event.stat.prevPct = nil
				event.stat.unitID = nil
			end
		end
	end
end

function WarnMe.RefreshHealth()
	if not WarnMe.events.health.named then
		return
	end
	
	WarnMe.ResetStat(WarnMe.events.health)

	local unitSpecsEmpty = true
	local unitSpecs = {}
	for name, _ in pairs(WarnMe.events.health.unitSpec) do
		unitSpecsEmpty = false
		unitSpecs[name] = 1
	end
	
	local eventTableEmpty = true
	local eventTable = {}
	if unitSpecsEmpty == false then
		local unitIDs = Inspect.Unit.Lookup(unitSpecs)
		for spec, id in pairs(unitIDs) do
			eventTableEmpty = false
			eventTable[id] = spec
		end
	end
	
	for name, _ in pairs(WarnMe.events.health.named) do
		local unitID = WarnMe.unitName2IDMap[name]
		if unitID and not eventTable[unitID] then
			eventTableEmpty = false
			eventTable[unitID] = 1
		end
	end

	if eventTableEmpty == false then	
		WarnMe.Health(eventTable)
	end
end

function WarnMe.GetHealthPct(details)
	if not details.health then
		return nil
	end

	local healthMax = details.healthMax
	if details.healthCap and details.healthCap > 0 and details.healthCap < details.healthMax then
		healthMax = details.healthCap
	end

	local healthPct = details.health / healthMax * 100
	return healthPct
end

function WarnMe.RefreshMana()
	if not WarnMe.events.mana.named then
		return
	end
	
	WarnMe.ResetStat(WarnMe.events.mana)

	local unitSpecsEmpty = true
	local unitSpecs = {}
	for name, _ in pairs(WarnMe.events.mana.unitSpec) do
		unitSpecsEmpty = false
		unitSpecs[name] = 1
	end
	
	local eventTableEmpty = true
	local eventTable = {}
	if unitSpecsEmpty == false then
		local unitIDs = Inspect.Unit.Lookup(unitSpecs)
		for spec, id in pairs(unitIDs) do
			eventTableEmpty = false
			eventTable[id] = spec
		end
	end
	
	for name, _ in pairs(WarnMe.events.mana.named) do
		local unitID = WarnMe.unitName2IDMap[name]
		if unitID and not eventTable[unitID] then
			eventTableEmpty = false
			eventTable[unitID] = 1
		end
	end

	if eventTableEmpty == false then	
		WarnMe.Mana(eventTable)
	end
end

function WarnMe.GetManaPct(details)
	if not details.mana then
		return nil
	end

	local manaMax = details.manaMax
	local manaPct = details.mana / manaMax * 100
	return manaPct
end

function WarnMe.RefreshCharge()
	if not WarnMe.events.charge.named then
		return
	end
	
	WarnMe.ResetStat(WarnMe.events.charge)

	local unitSpecsEmpty = true
	local unitSpecs = {}
	for name, _ in pairs(WarnMe.events.charge.unitSpec) do
		unitSpecsEmpty = false
		unitSpecs[name] = 1
	end
	
	local eventTableEmpty = true
	local eventTable = {}
	if unitSpecsEmpty == false then
		local unitIDs = Inspect.Unit.Lookup(unitSpecs)
		for spec, id in pairs(unitIDs) do
			eventTableEmpty = false
			eventTable[id] = spec
		end
	end
	
	for name, _ in pairs(WarnMe.events.charge.named) do
		local unitID = WarnMe.unitName2IDMap[name]
		if unitID and not eventTable[unitID] then
			eventTableEmpty = false
			eventTable[unitID] = 1
		end
	end

	if eventTableEmpty == false then	
		WarnMe.Charge(eventTable)
	end
end

function WarnMe.GetChargePct(details)
	if not details.charge then
		return nil
	end

	local chargePct = details.charge
	return chargePct
end

function WarnMe.RefreshEnergy()
	if not WarnMe.events.energy.named then
		return
	end
	
	WarnMe.ResetStat(WarnMe.events.energy)

	local unitSpecsEmpty = true
	local unitSpecs = {}
	for name, _ in pairs(WarnMe.events.energy.unitSpec) do
		unitSpecsEmpty = false
		unitSpecs[name] = 1
	end
	
	local eventTableEmpty = true
	local eventTable = {}
	if unitSpecsEmpty == false then
		local unitIDs = Inspect.Unit.Lookup(unitSpecs)
		for spec, id in pairs(unitIDs) do
			eventTableEmpty = false
			eventTable[id] = spec
		end
	end
	
	for name, _ in pairs(WarnMe.events.energy.named) do
		local unitID = WarnMe.unitName2IDMap[name]
		if unitID and not eventTable[unitID] then
			eventTableEmpty = false
			eventTable[unitID] = 1
		end
	end

	if eventTableEmpty == false then	
		WarnMe.Energy(eventTable)
	end
end

function WarnMe.GetEnergyPct(details)
	if not details.energy then
		return nil
	end

	local energyMax = details.energyMax
	local energyPct = details.energy / energyMax * 100
	return energyPct
end

function WarnMe.Reset()
	-- Go through and hide all existing warnings
	WarnMe.HideAllWarnings()
	
	-- Remove all timers and buff callbacks
	local restartCombatTimer = false
	if WarnMe.combatTimer then
		restartCombatTimer = true
	end
	
	BuffManager.Clear()
	Timer.Clear()
	AbilityManager.Clear()

	WarnMe.unitIDMap = {}
	WarnMe.unitID2NameMap = {}
	WarnMe.unitName2IDMap = {}

	WarnMe.CalculateGroupSize()
	
	WarnMe.ClearEvents()
	
	if WarnMe.addonOff == false then	
		-- Extract the relevant events
		WarnMe.LoadRelevantEvents()
	end
	
	if restartCombatTimer == true then
		WarnMe.StartCombatTimer()
	end
	
	WarnMe.CombatChanged()
end

function WarnMe.GetRoleNumber()
	if Inspect.TEMPORARY ~= nil and Inspect.TEMPORARY.Role ~= nil then
		local ret = Inspect.TEMPORARY.Role()
		if ret ~= nil then
			return tostring(ret)
		else
			return "0"
		end
	elseif WarnMe.currentRole ~= nil then
		return WarnMe.currentRole
	elseif WarnMe.savedRoles[WarnMe.playerName] then
		return WarnMe.savedRoles[WarnMe.playerName]
	else
		return "0"
	end
end

function WarnMe.SetRole(givenRole)
	local roleNumber = tonumber(givenRole)
	if not roleNumber then
		print("Must supply a valid number to set the role: " .. givenRole)
		return
	end
	
	if roleNumber < 0 or roleNumber > 6 then
		print("Must supply a number between 0-6 to set the role: " .. givenRole)
		return
	end
	
	local newRole = tostring(roleNumber)
	WarnMe.currentRole = tostring(roleNumber)
	WarnMe.Reset()
end

function WarnMe.CombatChanged()
	WarnMe.RefreshBuffs()
	WarnMe.RefreshHealth()
	WarnMe.RefreshMana()
	WarnMe.RefreshCharge()
	WarnMe.RefreshEnergy()
	AbilityManager.Refresh()
end

function WarnMe.Startup()
	WarnMe.SetPlayerNameAndID()
	WarnMe.CreateButton()
	WarnMe.RegisterPostStartupEvents()
	WarnMe.Reset()
	
	local secure = Inspect.System.Secure()
	if secure == true then
		WarnMe.SetInCombat()
	end
end


--
-- User Interface (UI)
--

function WarnMe.WarningCreated(txtWarnConfig, warning)
	if warning then
		WarnMe.totalWarnings[warning.name] = warning
	end
end

function WarnMe.CreateUI()
	local parent = UI.CreateContext("WarnMeContext")
	WarnMe.context = parent
	WarnMe.parentFrame = UI.CreateFrame("Frame", "WarnMeParent", parent)
	WarnMe.parentFrame:SetPoint("CENTER", UIParent, "CENTER")	
	WarnMe.config = WarnMeConfig.Create(WarnMe.parentFrame, WarnMe.Reset)
end

function WarnMe.CreateButton()
	-- create the button	
	local x = math.floor(WarnMe.x / 100 * UIParent:GetWidth())
	local y = math.floor(WarnMe.y / 100 * UIParent:GetHeight())
	WarnMe.button = UI.CreateFrame("Texture", "WarnMeButton", WarnMe.context)
	WarnMe.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	WarnMe.button:SetTexture("WarnMe", "WarnMeIcon.png")
	WarnMe.button.Event.LeftClick = function() WarnMeConfig.SetVisible(WarnMe.config, not WarnMeConfig.GetVisible(WarnMe.config)) end
	DragFrame.Create(WarnMe.button, WarnMe.button:GetWidth(), WarnMe.button:GetHeight(), function(dragFrame) WarnMe.SetCoords(dragFrame.x, dragFrame.y) end)
end

function WarnMe.SetCoords(x, y)
	WarnMe.x = x / UIParent:GetWidth() * 100
	WarnMe.y = y / UIParent:GetHeight() * 100
	WarnMe.button:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
end

--
-- Event Handlers
--

function WarnMe.Emote(info)
	if info and info.message then
		WarnMe.ParseEmote(info.message)
	end
end

function WarnMe.NpcChat(info)
	if info and info.message and info.fromName then
		WarnMe.ParseNpcChat(info.fromName, info.message)
	end
end

function WarnMe.UnitChangedHandler(eventTable)
	WarnMe.LoadUnknownUnits(eventTable)
	
	local refreshList = {}
	local groupRefresh = false
	for unitID, unitSpecifier in pairs(eventTable) do
		if type(unitSpecifier) == "string" then
			local groupPos = string.find(unitSpecifier, "group")
			if groupPos ~= nil then
				if string.find(unitSpecifier, "target") then
					unitSpecifier = "group.target"
				else
					unitSpecifier = "group"
				end
			end
			
			--if unitSpecifier == "player.target" or unitSpecifier == "focus" or unitSpecifier == "player.pet" or unitSpecifier == "group.target" then
			if unitSpecifier == "player.target" or unitSpecifier == "focus" or unitSpecifier == "player.pet" then
				refreshList[unitSpecifier] = 1
				WarnMe.unitIDMap[unitID] = unitSpecifier
			elseif unitSpecifier == "group" then
				groupRefresh = true
			elseif unitSpecifier == "player" then
				groupRefresh = true
			end
		elseif unitSpecifier == false then
			if WarnMe.unitIDMap[unitID] then
				refreshList[WarnMe.unitIDMap[unitID]] = 1
			end
			
			WarnMe.unitIDMap[unitID] = nil
		end
	end

	if groupRefresh == true then
		if WarnMe.CalculateGroupSize() == true then
			WarnMe.Reset()
		end
	end
	
	for unitSpec, _ in pairs(refreshList) do
		BuffManager.Refresh(unitSpec)
	end
end

function WarnMe.UnitAvailableHandler(eventTable)
	for _, unitSpec in pairs(eventTable) do
		if unitSpec == "player.pet" then
			BuffManager.Refresh(unitSpec)
		end
	end
end

local function Calculate2DDifference(x1, y1, z1, x2, y2, z2)
	if x1 == nil or y1 == nil or z1 == nil or x2 == nil or y2 == nil or z2 == nil then
		return nil
	end
	
	local x = math.abs(x1 - x2)
	local y = math.abs(y1 - y2)
	local z = math.abs(z1 - z2)
	return math.sqrt(x * x + y * y + z * z)
end

local function CalculateRange(unitSpec1, unitSpec2)
	local detail1 = Inspect.Unit.Detail(unitSpec1)
	local detail2 = Inspect.Unit.Detail(unitSpec2)
	
	if detail1 == nil or detail2 == nil then
		return nil
	else
		return Calculate2DDifference(detail1.coordX, detail1.coordY, detail1.coordZ, detail2.coordX, detail2.coordY, detail2.coordZ)
	end
end

local function FireCastbarEvent(thisEvent, targetIsPlayer, range)
	if thisEvent.spec.castbar.iAmTarget == true then
		if targetIsPlayer == true then
			WarnMe.SetWarningVisible(thisEvent, true)
		end
	elseif thisEvent.spec.castbar.range ~= nil and range ~= nil then
		if range <= thisEvent.spec.castbar.range then
			WarnMe.SetWarningVisible(thisEvent, true)
		end
	else
		WarnMe.SetWarningVisible(thisEvent, true)
	end
end

function WarnMe.Castbar(eventTable)
	WarnMe.LoadUnknownUnits(eventTable)

	local found = false
	local lookupTable = {}	
	for unitID, castStarted in pairs(eventTable) do
		if castStarted == true then
			lookupTable[unitID] = unitID
			found = true
		end
	end
	
	if found == true then
		local found = false
		for _, _ in pairs(WarnMe.events.castbar) do
			found = true
			break
		end
		if found == false then
			return
		end
		
		local unitLookup = {}
		for unitID, _ in pairs(lookupTable) do
			local details = Inspect.Unit.Castbar(unitID)
			if details ~= nil and details.abilityName and WarnMe.events.castbar[details.abilityName] then
				local targetIsPlayer = false
				local target = Inspect.Unit.Lookup(unitID .. ".target")
				if target == WarnMe.unitName2IDMap["player"] then
					targetIsPlayer = true
				end
				local range = CalculateRange(target, WarnMe.unitName2IDMap["player"])
				
				-- fire any all events 1st
				for _, thisEvent in ipairs(WarnMe.events.castbar[details.abilityName].all) do
					FireCastbarEvent(thisEvent, targetIsPlayer, range)
				end
				
				-- now fire any named individual events
				if WarnMe.unitID2NameMap[unitID] then
					local unitNamedEvents = WarnMe.events.castbar[details.abilityName].named[WarnMe.unitID2NameMap[unitID]]
					if unitNamedEvents then
						for _, thisEvent in ipairs(unitNamedEvents) do
							FireCastbarEvent(thisEvent, targetIsPlayer, range)
						end
					end
				end
				
				-- finally fire any unit spec events
				if not unitLookup[unitID] then
					local found = false
					local idLookup = {}
					for unitSpec, _ in pairs(WarnMe.events.castbar[details.abilityName].unitSpec) do
						idLookup[unitSpec] = 1
						found = true
					end
					
					if found == true then
						for unitSpec, unitID in pairs(Inspect.Unit.Lookup(idLookup)) do
							unitLookup[unitID] = unitSpec
						end
					end
				end
				
				local unitSpec = unitLookup[unitID]
				if unitSpec and WarnMe.events.castbar[details.abilityName].unitSpec[unitSpec] then
					for _, thisEvent in ipairs(WarnMe.events.castbar[details.abilityName].unitSpec[unitSpec]) do
						FireCastbarEvent(thisEvent, targetIsPlayer, range)
					end
				end
			end
		end
	end
end

function WarnMe.Health(eventTable)
	WarnMe.Stat(eventTable, WarnMe.events.health, WarnMe.GetHealthPct)
end

function WarnMe.Mana(eventTable)
	WarnMe.Stat(eventTable, WarnMe.events.mana, WarnMe.GetManaPct)
end

function WarnMe.Energy(eventTable)
	WarnMe.Stat(eventTable, WarnMe.events.energy, WarnMe.GetEnergyPct)
end

function WarnMe.Charge(eventTable)
	WarnMe.Stat(eventTable, WarnMe.events.charge, WarnMe.GetChargePct)
end

function WarnMe.Initialise(addonName)
	if addonName == WarnMe.name then
		WarnMe.CreateUI()
		Timer.AddCallback("Startup", 0.2, WarnMe.Startup, false)
	end
end

function WarnMe.SlashHandler(args)
	local arg = WarnMe.SplitArgs(args)

	if not arg[1] or arg[1] == "open" then
		WarnMeConfig.Show(WarnMe.config)
	elseif arg[1] == "off" then
		WarnMe.addonOff = true
		WarnMe.Reset()
	elseif arg[1] == "on" then
		WarnMe.addonOff = false
		WarnMe.Reset()
	elseif arg[1] == "role" then
		WarnMe.SetRole(arg[2])
	elseif arg[1] == "zone" then
		WarnMe.ChangeZone(arg)
	elseif arg[1] == "reset" then
		WarnMe.Reset()
	elseif arg[1] == "dump" then
		WarnMe.Dump()
	elseif arg[1] == "party" then
		WarnMe.inParty = true
		WarnMe.inRaid = false
		WarnMe.inCrossShardRaid = false
		WarnMe.Reset()
	elseif arg[1] == "settime" then
		WarnMe.SetTimeOffset(arg[2])
	elseif arg[1] == "gettime" then
		print(WarnMe.GetTimeString(Inspect.Time.Frame()))
	elseif arg[1] == "verbose" then
		WarnMe.verbose = not WarnMe.verbose
		print("Warn me verbose mode is " .. tostring(WarnMe.verbose))
	elseif arg[1] == "raid" then
		WarnMe.inParty = false
		WarnMe.inRaid = true
		WarnMe.inCrossShardRaid = false
		WarnMe.Reset()
	elseif arg[1] == "warfront" then
		WarnMe.inParty = false
		WarnMe.inRaid = false
		WarnMe.inCrossShardRaid = true
		WarnMe.Reset()
	elseif arg[1] == "solo" then
		WarnMe.inParty = false
		WarnMe.inRaid = false
		WarnMe.inCrossShardRaid = false
		WarnMe.Reset()
	else
		print(string.format("usage: %s [ dump ]", WarnMe.slashName));
		print(string.format("   %s dump", WarnMe.slashName));
		print(string.format("   %s role 1-5", WarnMe.slashName));
	end
end

function WarnMe.SaveVariables(addon)
	if addon == WarnMe.name then
		WarnMe_SavedVariables = {}
		WarnMe_SavedVariables.version = WarnMe.version
		WarnMe_SavedVariables.x = WarnMe.x
		WarnMe_SavedVariables.y = WarnMe.y
		WarnMe_SavedVariables.events = WarnMe.totalEvents
		WarnMe_SavedVariables.warnings = WarnMe.totalWarnings
		
		WarnMe.savedRoles[WarnMe.playerName] = WarnMe.currentRole
		WarnMe_SavedVariables.savedRoles = WarnMe.savedRoles
		WarnMe_SavedVariables.enabledCustomWarnings = WarnMe.enabledCustomWarnings
	end	
end

function WarnMe.UpdateEvents(events)
	if events then
		for name, event in pairs(events) do
			if not event.scope.isSolo and not event.scope.inParty and not event.scope.inRaid and not event.scope.inCrossShardRaid then
				event.scope.isSolo = true
				event.scope.inParty = true
				event.scope.inRaid = true
				event.scope.inCrossShardRaid = true
			end

			if event.health then
				event.eventType = WarnMe.eventTypeStat
				event.statType = WarnMe.statTypeHealth
				event.stat = {}
				event.stat.targetName = event.health.targetName
				event.stat.pct = event.health.pct
				event.stat.isLess = event.health.isLess
				event.stat.unitSpecs = {}
				for _, spec in ipairs(event.health.unitSpecs) do
					table.insert(event.stat.unitSpecs, spec)
				end

				event.health = nil
			end
			
			if event.stat and event.stat.targetName and string.len(event.stat.targetName) > 0 then
				event.stat.unitSpecs = { "named" }
			end
			
			if event.castbar and event.castbar.targetName and string.len(event.castbar.targetName) > 0 then
				event.castbar.unitSpecs = { "named" }
			end
			
			if event.ability and event.ability.isBeginEvent then
				if event.ability.isBeginEvent == true then
					event.ability.abilityType = WarnMe.abilityTypeBegin
				else
					event.ability.abilityType = WarnMe.abilityTypeEnd
				end
			end
			
			if event.scope.zone then
				local newZone = {}
				for name, value in pairs(event.scope.zone) do
					if name ~= "" then
						newZone[name] = value
					end
				end
				
				event.scope.zone = newZone
			end
		end
	end
end

function WarnMe.LoadVariables(addon)
	if addon == WarnMe.name then
		if WarnMe_SavedVariables then
			WarnMe.x = Utilities.GetValue(WarnMe_SavedVariables.x, 90)
			WarnMe.y = Utilities.GetValue(WarnMe_SavedVariables.y, 90)
			WarnMe.totalWarnings = Utilities.GetValue(WarnMe_SavedVariables.warnings, {})
			WarnMe.totalEvents = Utilities.GetValue(WarnMe_SavedVariables.events, {})
			WarnMe.savedRoles = Utilities.GetValue(WarnMe_SavedVariables.savedRoles, {})
			WarnMe.enabledCustomWarnings = Utilities.GetValue(WarnMe_SavedVariables.enabledCustomWarnings, {})
		else
			WarnMe.x = 90
			WarnMe.y = 90
			WarnMe.totalWarnings = {}
			WarnMe.totalEvents = {}
			WarnMe.savedRoles = {}
			WarnMe.enabledCustomWarnings = {}
		end

		WarnMe.UpdateEvents(WarnMe.totalEvents)
	end
end

function WarnMe.SetInCombat()
	-- temp bodge until 1.6
	if WarnMe.combatTimer then
		Timer.RemoveCallback(WarnMe.combatTimer)
		WarnMe.combatTimer = nil
		WarnMe.combatEvent = false
	end
	-- end temp bodge until 1.6
	
	WarnMe.SetCombatStatus(true)
end

-- temp bodge until 1.6
function WarnMe.CombatTimerCallback()
	if WarnMe.combatEvent == false then
		Timer.RemoveCallback(WarnMe.combatTimer)
		WarnMe.combatTimer = nil

		WarnMe.SetCombatStatus(false)
	end
	
	WarnMe.combatEvent = false
end

function WarnMe.DamageEvent()
	WarnMe.combatEvent = true
end
-- end temp bodge until 1.6

function WarnMe.StartCombatTimer()
	if WarnMe.combatTimer then
		Timer.RemoveCallback(WarnMe.combatTimer)
		WarnMe.combatTimer = nil
	end

	WarnMe.combatTimer = Timer.AddCallback("WarnMe.combatTimer", 10, function() WarnMe.CombatTimerCallback() end, true)
end

function WarnMe.SetOutCombat()
	-- calc group size will also check that all party members are out of combat as well
	WarnMe.CalculateGroupSize()
	
	-- temp bodge until 1.6
	WarnMe.combatEvent = false
	WarnMe.StartCombatTimer()
	-- end temp bodge until 1.6
end

--
-- Register events
--
Timer.Initialise(WarnMe.name)

function WarnMe.RegisterPostStartupEvents()
	AbilityManager.Initialise(WarnMe.name)
	BuffManager.Initialise(WarnMe.name, WarnMe.playerID)
	if Event.Unit.Availability ~= nil then
		table.insert(Event.Unit.Availability.Full, { WarnMe.UnitAvailableHandler, WarnMe.name, "UnitAvailableHandler" })
		table.insert(Event.Unit.Availability.Partial, { WarnMe.UnitAvailableHandler, WarnMe.name, "UnitAvailableHandler" })
		table.insert(Event.Unit.Availability.None, { WarnMe.UnitAvailableHandler, WarnMe.name, "UnitAvailableHandler" })
	else
		table.insert(Event.Unit.Available, { WarnMe.UnitAvailableHandler, WarnMe.name, "UnitAvailableHandler" })
	end
	table.insert(Event.Unit.Add, { WarnMe.UnitChangedHandler, WarnMe.name, "UnitAddHandler" })
	table.insert(Event.Unit.Remove, { WarnMe.UnitChangedHandler, WarnMe.name, "UnitRemovedHandler" })
	table.insert(Event.Chat.Notify, { WarnMe.Emote, WarnMe.name, "Emote" })
	table.insert(Event.Chat.Npc, { WarnMe.NpcChat, WarnMe.name, "NpcChat" })
	table.insert(Event.System.Secure.Enter, { WarnMe.SetInCombat, WarnMe.name, "SetInCombat" })
	table.insert(Event.System.Secure.Leave, { WarnMe.SetOutCombat, WarnMe.name, "SetInCombat" })
	table.insert(Event.Unit.Castbar, { WarnMe.Castbar, WarnMe.name, "Castbar" })
	table.insert(Event.Unit.Detail.Health, { WarnMe.Health, WarnMe.name, "Health" })
	table.insert(Event.Unit.Detail.Mana, { WarnMe.Mana, WarnMe.name, "Mana" })
	table.insert(Event.Unit.Detail.Energy, { WarnMe.Energy, WarnMe.name, "Energy" })
	table.insert(Event.Unit.Detail.Charge, { WarnMe.Charge, WarnMe.name, "Charge" })
	table.insert(Command.Slash.Register(WarnMe.slashName), { WarnMe.SlashHandler, WarnMe.name, "SlashHandler"})
	table.insert(Event.Addon.SavedVariables.Save.Begin, { WarnMe.SaveVariables, WarnMe.name, "SaveVariables" })
	-- temp bodge until 1.6
	table.insert(Event.Combat.Damage, { WarnMe.DamageEvent, WarnMe.name, "DamageEvent" })
	-- end temp bodge until 1.6
	if Event.TEMPORARY ~= nil and Event.TEMPORARY.Role ~= nil then
		table.insert(Event.TEMPORARY.Role, { WarnMe.SetRole, WarnMe.name, "RoleEvent" })
	end
end

table.insert(Event.Addon.Load.End, { WarnMe.Initialise, WarnMe.name, "Initialise" })
table.insert(Event.Addon.SavedVariables.Load.End, { WarnMe.LoadVariables, WarnMe.name, "LoadVariables" })
