-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

XYSelector = {}
XYSelector.name = "XYSelector"
XYSelector.version = "v0.1"

--
-- Main functions
--

--
-- UI functions
--

function XYSelector.Create(parentFrame, callback, dragMode)
	local xySelector = {}
	xySelector.parentFrame = parentFrame
	xySelector.callback = callback
	xySelector.dragMode = dragMode
	
	xySelector.xyFrame = UI.CreateFrame("Frame", "GridFrame", parentFrame)
	xySelector.xyFrame:SetAllPoints(UIParent)
	xySelector.xyFrame:SetLayer(5)
	xySelector.xyFrame:SetBackgroundColor(0.5, 0, 0, 0.1)
	xySelector.xyFrame:SetVisible(false)
	
	local totalWidth = UIParent:GetWidth()
	local totalHeight = UIParent:GetHeight()
	
	local rowHeight = totalHeight / 100
	for row = 1, 100 do
		local rowFrame = UI.CreateFrame("Frame", "GridRow" .. row, xySelector.xyFrame)
		local position = math.floor(rowHeight * row)
		rowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, position)
		rowFrame:SetPoint("BOTTOMRIGHT", UIParent, "TOPRIGHT", 0, position + 1)
		rowFrame:SetBackgroundColor(0.2, 0.2, 0.2, 0.75)
	end
	
	for row = 1, 19 do
		local rowFrame = UI.CreateFrame("Text", "GridRowTitleLeft" .. row, xySelector.xyFrame)
		rowFrame:SetText(string.format("%d%%", row * 5))
		local position = math.floor(rowHeight * row * 5 - rowFrame:GetHeight() / 2)
		rowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, position)
		rowFrame:SetFontColor(1, 1, 1, 0.75)

		local rowFrame = UI.CreateFrame("Text", "GridRowTitleRight" .. row, xySelector.xyFrame)
		rowFrame:SetText(string.format("%d%%", row * 5))
		local position = math.floor(rowHeight * row * 5 - rowFrame:GetHeight() / 2)
		rowFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", 0, position)
		rowFrame:SetFontColor(1, 1, 1, 0.75)
	end
	
	local colWidth = totalWidth / 100
	for col = 1, 100 do
		local colFrame = UI.CreateFrame("Frame", "GridCol" .. col, xySelector.xyFrame)
		local position = math.floor(colWidth * col)
		colFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", position, 0)
		colFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMLEFT", position + 1, 0)
		colFrame:SetBackgroundColor(0.2, 0.2, 0.2, 0.75)
	end

	for col = 1, 19 do
		local colFrame = UI.CreateFrame("Text", "GridColTitleTop" .. col, xySelector.xyFrame)
		colFrame:SetText(string.format("%d%%", col * 5))
		local position = math.floor(colWidth * col * 5 - colFrame:GetWidth() / 2)
		colFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", position, 0)
		colFrame:SetFontColor(1, 1, 1, 0.75)
		
		local colFrame = UI.CreateFrame("Text", "GridColTitleBottom" .. col, xySelector.xyFrame)
		colFrame:SetText(string.format("%d%%", col * 5))
		local position = math.floor(colWidth * col * 5 - colFrame:GetWidth() / 2)
		colFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", position, 0)
		colFrame:SetFontColor(1, 1, 1, 0.75)
	end

	xySelector.descFrame = UI.CreateFrame("Text", "DescFrame", xySelector.xyFrame)
	xySelector.descFrame:SetFontSize(200)

	xySelector.xyFrame.Event.LeftClick = function() XYSelector.Click(xySelector) end
	
	if dragMode == true then
		xySelector.descFrame:SetText("Right-click and drag to move warnings")	
		xySelector.descFrame:SetPoint("CENTER", xySelector.xyFrame, "CENTER")
		xySelector.descFrame:SetAlpha(0.5)
	else
		xySelector.descFrame:SetText("Click mouse at desired x,y location")
		xySelector.descFrame:SetPoint("CENTER", xySelector.xyFrame, "CENTER")
		
		xySelector.posnFrame = UI.CreateFrame("Text", "PosnFrame", xySelector.xyFrame)
		xySelector.posnFrame:SetFontSize(100)
		xySelector.posnFrame:SetText(" ")
		xySelector.posnFrame:SetPoint("TOPLEFT", xySelector.descFrame, "BOTTOMLEFT")

		xySelector.xyFrame.Event.MouseMove = function(a, x,y) XYSelector.MouseMove(xySelector, x, y) end
	end

	if dragMode == true then
		xySelector.warnings = {}
	else
		local warningTable = warnings
		if not warningTable then
			warningTable = WarnMe.totalWarnings
		end

		for _, warning in pairs(warningTable) do
			XYSelector.ShowWarningMarker(xySelector, warning)
		end
	end
	
	return xySelector
end

function XYSelector.SetWarnings(xySelector, warnings)
	xySelector.warnings = warnings
end

function XYSelector.GetXY(xIn, yIn)
	local x = Utilities.Round((xIn / UIParent:GetWidth()) * 100, 2)
	local y = Utilities.Round((yIn / UIParent:GetHeight()) * 100, 2)
	return x, y
end

function XYSelector.GetCoords(xIn, yIn)
	local x = Utilities.Round(UIParent:GetWidth() * (xIn / 100), 0)
	local y = Utilities.Round(UIParent:GetHeight() * (yIn / 100), 0)
	return x, y
end

function XYSelector.Click(xySelector)
	if xySelector then
		if xySelector.dragMode == true then
			xySelector.xyFrame:SetVisible(false)
			xySelector.callback(xySelector.warnings)
		else
			local mouse = Inspect.Mouse()
			xySelector.xyFrame:SetVisible(false)
			if mouse.x and mouse.y then
				local x, y = XYSelector.GetXY(mouse.x, mouse.y)
				xySelector.callback(x, y)
			end
		end
	end
end

function XYSelector.MouseMove(xySelector, xIn, yIn)
	if xySelector and xIn and yIn then
		local x, y = XYSelector.GetXY(xIn, yIn)
		xySelector.posnFrame:SetText(string.format("Current position = %0.2f%%, %0.2f%%", x, y))
	end
end

function XYSelector.SetVisible(xySelector, visible)
	if xySelector.dragMode == true then
		if xySelector.warnings then
			for _, warning in pairs(xySelector.warnings) do
				XYSelector.ShowWarningDragFrames(xySelector, warning)
			end
		end
	end
	
	xySelector.xyFrame:SetVisible(visible)
end

function XYSelector.ShowWarningMarker(xySelector, warning)
	if not warning.xPct then
		return
	end
	
	if not xySelector.warningFrames then
		xySelector.warningFrames = {}
	end
	
	if not xySelector.warningFrames[warning.name] then
		xySelector.warningFrames[warning.name] = {}
	end
	
	local x, y = XYSelector.GetCoords(warning.xPct, warning.yPct)
	xySelector.warningFrames[warning.name].point = UI.CreateFrame("Frame", "Warning" .. warning.name, xySelector.xyFrame)
	xySelector.warningFrames[warning.name].point:SetPoint("TOPLEFT", xySelector.xyFrame, "TOPLEFT", x, y)
	xySelector.warningFrames[warning.name].point:SetWidth(6)
	xySelector.warningFrames[warning.name].point:SetHeight(6)
	xySelector.warningFrames[warning.name].point:SetBackgroundColor(1, 1, 0, 0.5)
	xySelector.warningFrames[warning.name].tooltip = UI.CreateFrame("Text", "WarningTip" .. warning.name, xySelector.xyFrame)
	xySelector.warningFrames[warning.name].tooltip:SetPoint("TOPLEFT", xySelector.warningFrames[warning.name].point, "TOPRIGHT", 2, 0)
	xySelector.warningFrames[warning.name].tooltip:SetText(warning.name)
	xySelector.warningFrames[warning.name].tooltip:SetFontColor(1, 1, 0, 0.5)
end

function XYSelector.ShowWarningDragFrames(xySelector, warning)
	if not warning.setDragFrameVisible then
		return
	end
	
	warning.setDragFrameVisible(true)
end
